@@ -1,25 +1,16 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use crate::caps_features::*;
-use crate::structure::*;
-use std::fmt;
-use std::marker::PhantomData;
-use std::ptr;
-use std::str;
+use std::{fmt, marker::PhantomData, ptr, str};
 
-use crate::CapsIntersectMode;
+use glib::{prelude::*, translate::*, value::ToSendValue};
 
-use glib::translate::{
-    from_glib, from_glib_full, FromGlibPtrFull, IntoGlib, IntoGlibPtr, ToGlibPtr,
-};
-use glib::value::ToSendValue;
-use glib::StaticType;
+use crate::{caps_features::*, structure::*, CapsIntersectMode};
 
 mini_object_wrapper!(Caps, CapsRef, ffi::GstCaps, || { ffi::gst_caps_get_type() });
 
 impl Caps {
     #[doc(alias = "gst_caps_new_simple")]
-    pub fn builder(name: &str) -> Builder<NoFeature> {
+    pub fn builder(name: impl IntoGStr) -> Builder<NoFeature> {
         assert_initialized_main_thread!();
         Builder::new(name)
     }
@@ -54,26 +45,19 @@ impl Caps {
         unsafe { from_glib_full(ffi::gst_caps_new_any()) }
     }
 
-    #[doc(alias = "gst_caps_new_simple")]
-    pub fn new_simple(name: &str, values: &[(&str, &(dyn ToSendValue + Sync))]) -> Self {
-        assert_initialized_main_thread!();
+    #[doc(alias = "gst_caps_new_empty_simple")]
+    pub fn new_empty_simple(name: impl IntoGStr) -> Self {
+        skip_assert_initialized!();
         let mut caps = Caps::new_empty();
 
-        let structure = Structure::new(name, values);
+        let structure = Structure::new_empty(name);
         caps.get_mut().unwrap().append_structure(structure);
 
         caps
     }
 
-    #[doc(alias = "gst_caps_new_empty_simple")]
-    pub fn new_empty_simple(name: &str) -> Self {
-        skip_assert_initialized!();
-        Self::new_simple(name, &[])
-    }
-
     #[doc(alias = "gst_caps_fixate")]
     pub fn fixate(&mut self) {
-        skip_assert_initialized!();
         unsafe {
             // See https://gitlab.freedesktop.org/gstreamer/gstreamer/-/merge_requests/388
             assert!(!self.is_any());
@@ -88,7 +72,6 @@ impl Caps {
 
     #[doc(alias = "gst_caps_merge")]
     pub fn merge(&mut self, other: Self) {
-        skip_assert_initialized!();
         unsafe {
             let ptr = ffi::gst_caps_merge(self.as_mut_ptr(), other.into_glib_ptr());
             self.replace_ptr(ptr);
@@ -97,7 +80,6 @@ impl Caps {
 
     #[doc(alias = "gst_caps_merge_structure")]
     pub fn merge_structure(&mut self, structure: Structure) {
-        skip_assert_initialized!();
         unsafe {
             let ptr = ffi::gst_caps_merge_structure(self.as_mut_ptr(), structure.into_glib_ptr());
             self.replace_ptr(ptr);
@@ -106,7 +88,6 @@ impl Caps {
 
     #[doc(alias = "gst_caps_merge_structure_full")]
     pub fn merge_structure_full(&mut self, structure: Structure, features: Option<CapsFeatures>) {
-        skip_assert_initialized!();
         unsafe {
             let ptr = ffi::gst_caps_merge_structure_full(
                 self.as_mut_ptr(),
@@ -121,7 +102,6 @@ impl Caps {
 
     #[doc(alias = "gst_caps_normalize")]
     pub fn normalize(&mut self) {
-        skip_assert_initialized!();
         unsafe {
             let ptr = ffi::gst_caps_normalize(self.as_mut_ptr());
             self.replace_ptr(ptr);
@@ -130,7 +110,6 @@ impl Caps {
 
     #[doc(alias = "gst_caps_simplify")]
     pub fn simplify(&mut self) {
-        skip_assert_initialized!();
         unsafe {
             let ptr = ffi::gst_caps_simplify(self.as_mut_ptr());
             self.replace_ptr(ptr);
@@ -139,7 +118,6 @@ impl Caps {
 
     #[doc(alias = "gst_caps_truncate")]
     pub fn truncate(&mut self) {
-        skip_assert_initialized!();
         unsafe {
             let ptr = ffi::gst_caps_truncate(self.as_mut_ptr());
             self.replace_ptr(ptr);
@@ -154,15 +132,89 @@ impl str::FromStr for Caps {
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         assert_initialized_main_thread!();
         unsafe {
-            Option::<_>::from_glib_full(ffi::gst_caps_from_string(s.to_glib_none().0))
-                .ok_or_else(|| glib::bool_error!("Failed to parse caps from string"))
+            s.run_with_gstr(|s| {
+                Option::<_>::from_glib_full(ffi::gst_caps_from_string(s.as_ptr()))
+                    .ok_or_else(|| glib::bool_error!("Failed to parse caps from string"))
+            })
         }
     }
 }
 
+impl From<Structure> for Caps {
+    fn from(v: Structure) -> Caps {
+        skip_assert_initialized!();
+        let mut caps = Caps::new_empty();
+
+        {
+            let caps = caps.get_mut().unwrap();
+            caps.append_structure(v);
+        }
+
+        caps
+    }
+}
+
+impl<const N: usize> From<[Structure; N]> for Caps {
+    fn from(v: [Structure; N]) -> Caps {
+        skip_assert_initialized!();
+        let mut caps = Caps::new_empty();
+
+        {
+            let caps = caps.get_mut().unwrap();
+            v.into_iter().for_each(|s| caps.append_structure(s));
+        }
+
+        caps
+    }
+}
+
+impl From<(Structure, CapsFeatures)> for Caps {
+    fn from(v: (Structure, CapsFeatures)) -> Caps {
+        skip_assert_initialized!();
+        let mut caps = Caps::new_empty();
+
+        {
+            let caps = caps.get_mut().unwrap();
+            caps.append_structure_full(v.0, Some(v.1));
+        }
+
+        caps
+    }
+}
+
+impl<const N: usize> From<[(Structure, CapsFeatures); N]> for Caps {
+    fn from(v: [(Structure, CapsFeatures); N]) -> Caps {
+        skip_assert_initialized!();
+        let mut caps = Caps::new_empty();
+
+        {
+            let caps = caps.get_mut().unwrap();
+            v.into_iter()
+                .for_each(|s| caps.append_structure_full(s.0, Some(s.1)));
+        }
+
+        caps
+    }
+}
+
+impl<const N: usize> From<[(Structure, Option<CapsFeatures>); N]> for Caps {
+    fn from(v: [(Structure, Option<CapsFeatures>); N]) -> Caps {
+        skip_assert_initialized!();
+        let mut caps = Caps::new_empty();
+
+        {
+            let caps = caps.get_mut().unwrap();
+            v.into_iter()
+                .for_each(|s| caps.append_structure_full(s.0, s.1));
+        }
+
+        caps
+    }
+}
+
 impl std::iter::FromIterator<Structure> for Caps {
     fn from_iter<T: IntoIterator<Item = Structure>>(iter: T) -> Self {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
         let mut caps = Caps::new_empty();
 
         {
@@ -176,7 +228,7 @@ impl std::iter::FromIterator<Structure> for Caps {
 
 impl std::iter::FromIterator<(Structure, CapsFeatures)> for Caps {
     fn from_iter<T: IntoIterator<Item = (Structure, CapsFeatures)>>(iter: T) -> Self {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
         let mut caps = Caps::new_empty();
 
         {
@@ -191,7 +243,7 @@ impl std::iter::FromIterator<(Structure, CapsFeatures)> for Caps {
 
 impl std::iter::FromIterator<(Structure, Option<CapsFeatures>)> for Caps {
     fn from_iter<T: IntoIterator<Item = (Structure, Option<CapsFeatures>)>>(iter: T) -> Self {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
         let mut caps = Caps::new_empty();
 
         {
@@ -206,7 +258,7 @@ impl std::iter::FromIterator<(Structure, Option<CapsFeatures>)> for Caps {
 
 impl std::iter::FromIterator<Caps> for Caps {
     fn from_iter<T: IntoIterator<Item = Caps>>(iter: T) -> Self {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
         let mut caps = Caps::new_empty();
 
         {
@@ -246,18 +298,19 @@ impl std::iter::Extend<Caps> for CapsRef {
 }
 
 impl CapsRef {
+    #[doc(alias = "gst_caps_set_value")]
     #[doc(alias = "gst_caps_set_simple")]
-    pub fn set_simple(&mut self, values: &[(&str, &(dyn ToSendValue + Sync))]) {
-        for &(name, value) in values {
-            let value = value.to_value();
+    pub fn set(&mut self, name: impl IntoGStr, value: impl ToSendValue + Sync) {
+        let value = value.to_send_value();
+        self.set_value(name, value);
+    }
 
-            unsafe {
-                ffi::gst_caps_set_value(
-                    self.as_mut_ptr(),
-                    name.to_glib_none().0,
-                    value.to_glib_none().0,
-                );
-            }
+    #[doc(alias = "gst_caps_set_value")]
+    pub fn set_value(&mut self, name: impl IntoGStr, value: glib::SendValue) {
+        unsafe {
+            name.run_with_gstr(|name| {
+                ffi::gst_caps_set_value(self.as_mut_ptr(), name.as_ptr(), value.to_glib_none().0)
+            });
         }
     }
 
@@ -336,8 +389,8 @@ impl CapsRef {
         }
     }
 
-    #[cfg(any(feature = "v1_16", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+    #[cfg(feature = "v1_16")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
     #[doc(alias = "gst_caps_set_features_simple")]
     pub fn set_features_simple(&mut self, features: Option<CapsFeatures>) {
         unsafe {
@@ -498,7 +551,6 @@ impl CapsRef {
 
     #[doc(alias = "gst_caps_subtract")]
     pub fn subtract(&self, other: &Self) -> Caps {
-        skip_assert_initialized!();
         unsafe {
             from_glib_full(ffi::gst_caps_subtract(
                 self.as_mut_ptr(),
@@ -507,8 +559,8 @@ impl CapsRef {
         }
     }
 
-    #[cfg(any(feature = "v1_20", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
+    #[cfg(feature = "v1_20")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_20")))]
     #[doc(alias = "gst_caps_serialize")]
     pub fn serialize(&self, flags: crate::SerializeFlags) -> glib::GString {
         unsafe { from_glib_full(ffi::gst_caps_serialize(&self.0, flags.into_glib())) }
@@ -642,6 +694,7 @@ macro_rules! define_iter(
         }
     }
 
+    #[allow(clippy::redundant_closure_call)]
     impl<'a> Iterator for $name<'a> {
         type Item = $styp;
 
@@ -691,6 +744,7 @@ macro_rules! define_iter(
         }
     }
 
+    #[allow(clippy::redundant_closure_call)]
     impl<'a> DoubleEndedIterator for $name<'a> {
         fn next_back(&mut self) -> Option<Self::Item> {
             if self.idx == self.n_structures {
@@ -748,9 +802,7 @@ define_iter!(
         if ptr.is_null() {
             None
         } else {
-            Some(StructureRef::from_glib_borrow_mut(
-                ptr as *mut ffi::GstStructure,
-            ))
+            Some(StructureRef::from_glib_borrow_mut(ptr))
         }
     }
 );
@@ -765,8 +817,8 @@ define_iter!(
             None
         } else {
             Some((
-                StructureRef::from_glib_borrow(ptr1 as *const ffi::GstStructure),
-                CapsFeaturesRef::from_glib_borrow(ptr2 as *const ffi::GstCapsFeatures),
+                StructureRef::from_glib_borrow(ptr1),
+                CapsFeaturesRef::from_glib_borrow(ptr2),
             ))
         }
     }
@@ -782,8 +834,8 @@ define_iter!(
             None
         } else {
             Some((
-                StructureRef::from_glib_borrow_mut(ptr1 as *mut ffi::GstStructure),
-                CapsFeaturesRef::from_glib_borrow_mut(ptr2 as *mut ffi::GstCapsFeatures),
+                StructureRef::from_glib_borrow_mut(ptr1),
+                CapsFeaturesRef::from_glib_borrow_mut(ptr2),
             ))
         }
     }
@@ -926,7 +978,7 @@ impl<T> fmt::Debug for Builder<T> {
 }
 
 impl Builder<NoFeature> {
-    fn new(name: &str) -> Builder<NoFeature> {
+    fn new(name: impl IntoGStr) -> Builder<NoFeature> {
         skip_assert_initialized!();
         Builder {
             s: crate::Structure::new_empty(name),
@@ -935,7 +987,10 @@ impl Builder<NoFeature> {
         }
     }
 
-    pub fn features(self, features: &[&str]) -> Builder<HasFeatures> {
+    pub fn features(
+        self,
+        features: impl IntoIterator<Item = impl IntoGStr>,
+    ) -> Builder<HasFeatures> {
         Builder {
             s: self.s,
             features: Some(CapsFeatures::new(features)),
@@ -953,7 +1008,7 @@ impl Builder<NoFeature> {
 }
 
 impl<T> Builder<T> {
-    pub fn field<V: ToSendValue + Sync>(mut self, name: &str, value: V) -> Self {
+    pub fn field(mut self, name: impl IntoGStr, value: impl Into<glib::Value> + Send) -> Self {
         self.s.set(name, value);
         self
     }
@@ -1068,65 +1123,13 @@ impl<T> BuilderFull<T> {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::Array;
-    use crate::Fraction;
-
-    #[test]
-    fn test_simple() {
-        crate::init().unwrap();
-
-        let mut caps = Caps::new_simple(
-            "foo/bar",
-            &[
-                ("int", &12),
-                ("bool", &true),
-                ("string", &"bla"),
-                ("fraction", &Fraction::new(1, 2)),
-                ("array", &Array::new([1, 2])),
-            ],
-        );
-        assert_eq!(
-            caps.to_string(),
-            "foo/bar, int=(int)12, bool=(boolean)true, string=(string)bla, fraction=(fraction)1/2, array=(int)< 1, 2 >"
-        );
-
-        {
-            let s = caps.structure(0).unwrap();
-            assert_eq!(
-                s,
-                Structure::new(
-                    "foo/bar",
-                    &[
-                        ("int", &12),
-                        ("bool", &true),
-                        ("string", &"bla"),
-                        ("fraction", &Fraction::new(1, 2)),
-                        ("array", &Array::new([1, 2])),
-                    ],
-                )
-                .as_ref()
-            );
-        }
-        assert!(caps
-            .features(0)
-            .unwrap()
-            .is_equal(crate::CAPS_FEATURES_MEMORY_SYSTEM_MEMORY.as_ref()));
-
-        {
-            let caps = caps.get_mut().unwrap();
-            caps.set_features(0, Some(CapsFeatures::new(&["foo:bla"])));
-        }
-        assert!(caps
-            .features(0)
-            .unwrap()
-            .is_equal(CapsFeatures::new(&["foo:bla"]).as_ref()));
-    }
+    use crate::{Array, Fraction};
 
     #[test]
     fn test_builder() {
         crate::init().unwrap();
 
-        let caps = Caps::builder("foo/bar")
+        let mut caps = Caps::builder("foo/bar")
             .field("int", 12)
             .field("bool", true)
             .field("string", "bla")
@@ -1138,6 +1141,20 @@ mod tests {
             "foo/bar, int=(int)12, bool=(boolean)true, string=(string)bla, fraction=(fraction)1/2, array=(int)< 1, 2 >"
         );
 
+        assert!(caps
+            .features(0)
+            .unwrap()
+            .is_equal(crate::CAPS_FEATURES_MEMORY_SYSTEM_MEMORY.as_ref()));
+
+        {
+            let caps = caps.get_mut().unwrap();
+            caps.set_features(0, Some(CapsFeatures::new(["foo:bla"])));
+        }
+        assert!(caps
+            .features(0)
+            .unwrap()
+            .is_equal(CapsFeatures::new(["foo:bla"]).as_ref()));
+
         let caps = Caps::builder("foo/bar")
             .field("int", 12)
             .any_features()
@@ -1146,7 +1163,7 @@ mod tests {
 
         let caps = Caps::builder("foo/bar")
             .field("int", 12)
-            .features(&["foo:bla", "foo:baz"])
+            .features(["foo:bla", "foo:baz"])
             .build();
         assert_eq!(caps.to_string(), "foo/bar(foo:bla, foo:baz), int=(int)12");
     }
@@ -1156,7 +1173,7 @@ mod tests {
         crate::init().unwrap();
 
         let caps = Caps::builder("foo/bar").build();
-        format!("{}", caps);
+        format!("{caps}");
     }
 
     #[test]
@@ -1186,7 +1203,7 @@ mod tests {
             .structure_with_any_features(Structure::builder("audio/x-raw").build())
             .structure_with_features(
                 Structure::builder("video/x-raw").build(),
-                CapsFeatures::new(&["foo:bla", "foo:baz"]),
+                CapsFeatures::new(["foo:bla", "foo:baz"]),
             )
             .build();
         assert_eq!(
@@ -1199,11 +1216,11 @@ mod tests {
     fn test_builder_full_with_features() {
         crate::init().unwrap();
 
-        let caps = Caps::builder_full_with_features(CapsFeatures::new(&["foo:bla"]))
+        let caps = Caps::builder_full_with_features(CapsFeatures::new(["foo:bla"]))
             .structure(Structure::builder("audio/x-raw").build())
             .structure_with_features(
                 Structure::builder("video/x-raw").build(),
-                CapsFeatures::new(&["foo:baz"]),
+                CapsFeatures::new(["foo:baz"]),
             )
             .build();
         assert_eq!(
@@ -1257,17 +1274,17 @@ mod tests {
         crate::init().unwrap();
 
         let caps = Caps::new_any();
-        assert_eq!(format!("{:?}", caps), "Caps(\"ANY\")");
+        assert_eq!(format!("{caps:?}"), "Caps(\"ANY\")");
 
         let caps = Caps::new_empty();
-        assert_eq!(format!("{:?}", caps), "Caps(\"EMPTY\")");
+        assert_eq!(format!("{caps:?}"), "Caps(\"EMPTY\")");
 
         let caps = Caps::builder_full_with_any_features()
             .structure(Structure::builder("audio/x-raw").build())
             .build();
-        assert_eq!(format!("{:?}", caps), "Caps(audio/x-raw(ANY))");
+        assert_eq!(format!("{caps:?}"), "Caps(audio/x-raw(ANY))");
 
-        let caps = Caps::builder_full_with_features(CapsFeatures::new(&["foo:bla"]))
+        let caps = Caps::builder_full_with_features(CapsFeatures::new(["foo:bla"]))
             .structure(
                 Structure::builder("audio/x-raw")
                     .field(
@@ -1283,7 +1300,7 @@ mod tests {
             )
             .build();
 
-        assert_eq!(format!("{:?}", caps), "Caps(audio/x-raw(foo:bla) { struct: Structure(nested { badger: (gboolean) TRUE }) }, video/x-raw(foo:bla) { width: (guint) 800 })");
+        assert_eq!(format!("{caps:?}"), "Caps(audio/x-raw(foo:bla) { struct: Structure(nested { badger: (gboolean) TRUE }) }, video/x-raw(foo:bla) { width: (guint) 800 })");
 
         let caps = Caps::builder_full()
             .structure(
@@ -1294,6 +1311,6 @@ mod tests {
             )
             .build();
 
-        assert_eq!(format!("{:?}", caps), "Caps(video/x-raw(memory:SystemMemory) { array: Array([(gchararray) \"a\", (gchararray) \"b\", (gchararray) \"c\"]), list: List([(gchararray) \"d\", (gchararray) \"e\", (gchararray) \"f\"]) })");
+        assert_eq!(format!("{caps:?}"), "Caps(video/x-raw(memory:SystemMemory) { array: Array([(gchararray) \"a\", (gchararray) \"b\", (gchararray) \"c\"]), list: List([(gchararray) \"d\", (gchararray) \"e\", (gchararray) \"f\"]) })");
     }
 }
