@@ -1,18 +1,17 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use std::borrow::{Borrow, BorrowMut, ToOwned};
-use std::ffi::CStr;
-use std::fmt;
-use std::mem;
-use std::ops::{Deref, DerefMut};
-use std::ptr;
-use std::str;
+use std::{
+    borrow::{Borrow, BorrowMut, ToOwned},
+    fmt,
+    marker::PhantomData,
+    mem,
+    ops::{Deref, DerefMut},
+    ptr, str,
+};
 
+use glib::{prelude::*, translate::*};
 use once_cell::sync::Lazy;
 
-use glib::translate::*;
-use glib::StaticType;
-
 #[doc(alias = "GstCapsFeatures")]
 #[repr(transparent)]
 pub struct CapsFeatures(ptr::NonNull<ffi::GstCapsFeatures>);
@@ -21,8 +20,8 @@ unsafe impl Sync for CapsFeatures {}
 
 impl CapsFeatures {
     #[doc(alias = "gst_caps_features_new")]
-    pub fn new(features: &[&str]) -> Self {
-        assert_initialized_main_thread!();
+    pub fn new(features: impl IntoIterator<Item = impl IntoGStr>) -> Self {
+        skip_assert_initialized!();
         let mut f = Self::new_empty();
 
         for feature in features {
@@ -33,12 +32,12 @@ impl CapsFeatures {
     }
 
     #[doc(alias = "gst_caps_features_new_id")]
-    pub fn from_quarks(features: &[glib::Quark]) -> Self {
-        assert_initialized_main_thread!();
+    pub fn from_quarks(features: impl IntoIterator<Item = glib::Quark>) -> Self {
+        skip_assert_initialized!();
         let mut f = Self::new_empty();
 
-        for feature in features {
-            f.add_from_quark(*feature);
+        for feature in features.into_iter() {
+            f.add_from_quark(feature);
         }
 
         f
@@ -62,6 +61,7 @@ impl CapsFeatures {
 }
 
 impl IntoGlibPtr<*mut ffi::GstCapsFeatures> for CapsFeatures {
+    #[inline]
     unsafe fn into_glib_ptr(self) -> *mut ffi::GstCapsFeatures {
         let s = mem::ManuallyDrop::new(self);
         s.0.as_ptr()
@@ -71,40 +71,46 @@ impl IntoGlibPtr<*mut ffi::GstCapsFeatures> for CapsFeatures {
 impl Deref for CapsFeatures {
     type Target = CapsFeaturesRef;
 
+    #[inline]
     fn deref(&self) -> &CapsFeaturesRef {
         unsafe { &*(self.0.as_ref() as *const ffi::GstCapsFeatures as *const CapsFeaturesRef) }
     }
 }
 
 impl DerefMut for CapsFeatures {
+    #[inline]
     fn deref_mut(&mut self) -> &mut CapsFeaturesRef {
         unsafe { &mut *(self.0.as_mut() as *mut ffi::GstCapsFeatures as *mut CapsFeaturesRef) }
     }
 }
 
 impl AsRef<CapsFeaturesRef> for CapsFeatures {
+    #[inline]
     fn as_ref(&self) -> &CapsFeaturesRef {
         self.deref()
     }
 }
 
 impl AsMut<CapsFeaturesRef> for CapsFeatures {
+    #[inline]
     fn as_mut(&mut self) -> &mut CapsFeaturesRef {
         self.deref_mut()
     }
 }
 
 impl Clone for CapsFeatures {
+    #[inline]
     fn clone(&self) -> Self {
         unsafe {
             let ptr = ffi::gst_caps_features_copy(self.0.as_ref());
-            assert!(!ptr.is_null());
+            debug_assert!(!ptr.is_null());
             CapsFeatures(ptr::NonNull::new_unchecked(ptr))
         }
     }
 }
 
 impl Drop for CapsFeatures {
+    #[inline]
     fn drop(&mut self) {
         unsafe { ffi::gst_caps_features_free(self.0.as_mut()) }
     }
@@ -133,7 +139,7 @@ impl str::FromStr for CapsFeatures {
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         assert_initialized_main_thread!();
         unsafe {
-            let ptr = ffi::gst_caps_features_from_string(s.to_glib_none().0);
+            let ptr = s.run_with_gstr(|s| ffi::gst_caps_features_from_string(s.as_ptr()));
             if ptr.is_null() {
                 return Err(glib::bool_error!(
                     "Failed to parse caps features from string"
@@ -146,81 +152,92 @@ impl str::FromStr for CapsFeatures {
 }
 
 impl Borrow<CapsFeaturesRef> for CapsFeatures {
+    #[inline]
     fn borrow(&self) -> &CapsFeaturesRef {
         self.as_ref()
     }
 }
 
 impl BorrowMut<CapsFeaturesRef> for CapsFeatures {
+    #[inline]
     fn borrow_mut(&mut self) -> &mut CapsFeaturesRef {
         self.as_mut()
     }
 }
 
 impl glib::types::StaticType for CapsFeatures {
+    #[inline]
     fn static_type() -> glib::types::Type {
         unsafe { from_glib(ffi::gst_caps_features_get_type()) }
     }
 }
 
 impl<'a> ToGlibPtr<'a, *const ffi::GstCapsFeatures> for CapsFeatures {
-    type Storage = &'a Self;
+    type Storage = PhantomData<&'a Self>;
 
+    #[inline]
     fn to_glib_none(&'a self) -> Stash<'a, *const ffi::GstCapsFeatures, Self> {
-        unsafe { Stash(self.0.as_ref(), self) }
+        unsafe { Stash(self.0.as_ref(), PhantomData) }
     }
 
+    #[inline]
     fn to_glib_full(&self) -> *const ffi::GstCapsFeatures {
         unsafe { ffi::gst_caps_features_copy(self.0.as_ref()) }
     }
 }
 
 impl<'a> ToGlibPtr<'a, *mut ffi::GstCapsFeatures> for CapsFeatures {
-    type Storage = &'a Self;
+    type Storage = PhantomData<&'a Self>;
 
+    #[inline]
     fn to_glib_none(&'a self) -> Stash<'a, *mut ffi::GstCapsFeatures, Self> {
         unsafe {
             Stash(
                 self.0.as_ref() as *const ffi::GstCapsFeatures as *mut ffi::GstCapsFeatures,
-                self,
+                PhantomData,
             )
         }
     }
 
+    #[inline]
     fn to_glib_full(&self) -> *mut ffi::GstCapsFeatures {
         unsafe { ffi::gst_caps_features_copy(self.0.as_ref()) }
     }
 }
 
 impl<'a> ToGlibPtrMut<'a, *mut ffi::GstCapsFeatures> for CapsFeatures {
-    type Storage = &'a mut Self;
+    type Storage = PhantomData<&'a mut Self>;
 
+    #[inline]
     fn to_glib_none_mut(&'a mut self) -> StashMut<*mut ffi::GstCapsFeatures, Self> {
-        unsafe { StashMut(self.0.as_mut(), self) }
+        unsafe { StashMut(self.0.as_mut(), PhantomData) }
     }
 }
 
 impl FromGlibPtrNone<*const ffi::GstCapsFeatures> for CapsFeatures {
+    #[inline]
     unsafe fn from_glib_none(ptr: *const ffi::GstCapsFeatures) -> Self {
-        assert!(!ptr.is_null());
+        debug_assert!(!ptr.is_null());
         let ptr = ffi::gst_caps_features_copy(ptr);
-        assert!(!ptr.is_null());
+        debug_assert!(!ptr.is_null());
         CapsFeatures(ptr::NonNull::new_unchecked(ptr))
     }
 }
 
 impl FromGlibPtrNone<*mut ffi::GstCapsFeatures> for CapsFeatures {
+    #[inline]
     unsafe fn from_glib_none(ptr: *mut ffi::GstCapsFeatures) -> Self {
-        assert!(!ptr.is_null());
+        debug_assert!(!ptr.is_null());
         let ptr = ffi::gst_caps_features_copy(ptr);
-        assert!(!ptr.is_null());
+        debug_assert!(!ptr.is_null());
         CapsFeatures(ptr::NonNull::new_unchecked(ptr))
     }
 }
 
 impl FromGlibPtrFull<*const ffi::GstCapsFeatures> for CapsFeatures {
+    #[inline]
     unsafe fn from_glib_full(ptr: *const ffi::GstCapsFeatures) -> Self {
-        assert!(!ptr.is_null());
+        debug_assert!(!ptr.is_null());
         CapsFeatures(ptr::NonNull::new_unchecked(
             ptr as *mut ffi::GstCapsFeatures,
         ))
@@ -228,8 +245,9 @@ impl FromGlibPtrFull<*const ffi::GstCapsFeatures> for CapsFeatures {
 }
 
 impl FromGlibPtrFull<*mut ffi::GstCapsFeatures> for CapsFeatures {
+    #[inline]
     unsafe fn from_glib_full(ptr: *mut ffi::GstCapsFeatures) -> Self {
-        assert!(!ptr.is_null());
+        debug_assert!(!ptr.is_null());
         CapsFeatures(ptr::NonNull::new_unchecked(ptr))
     }
 }
@@ -281,33 +299,53 @@ impl glib::value::ToValueOptional for CapsFeatures {
     }
 }
 
+impl From<CapsFeatures> for glib::Value {
+    fn from(v: CapsFeatures) -> glib::Value {
+        skip_assert_initialized!();
+        let mut value = glib::Value::for_value_type::<CapsFeatures>();
+        unsafe {
+            glib::gobject_ffi::g_value_take_boxed(
+                value.to_glib_none_mut().0,
+                IntoGlibPtr::<*mut ffi::GstCapsFeatures>::into_glib_ptr(v) as *mut _,
+            )
+        }
+        value
+    }
+}
+
 impl GlibPtrDefault for CapsFeatures {
     type GlibType = *mut ffi::GstCapsFeatures;
 }
 
+unsafe impl TransparentPtrType for CapsFeatures {}
+
 #[repr(transparent)]
 #[doc(alias = "GstCapsFeatures")]
 pub struct CapsFeaturesRef(ffi::GstCapsFeatures);
 
 impl CapsFeaturesRef {
+    #[inline]
     pub unsafe fn from_glib_borrow<'a>(ptr: *const ffi::GstCapsFeatures) -> &'a CapsFeaturesRef {
-        assert!(!ptr.is_null());
+        debug_assert!(!ptr.is_null());
 
         &*(ptr as *mut CapsFeaturesRef)
     }
 
+    #[inline]
     pub unsafe fn from_glib_borrow_mut<'a>(
         ptr: *mut ffi::GstCapsFeatures,
     ) -> &'a mut CapsFeaturesRef {
-        assert!(!ptr.is_null());
+        debug_assert!(!ptr.is_null());
 
         &mut *(ptr as *mut CapsFeaturesRef)
     }
 
+    #[inline]
     pub fn as_ptr(&self) -> *const ffi::GstCapsFeatures {
         self as *const Self as *const ffi::GstCapsFeatures
     }
 
+    #[inline]
     pub fn as_mut_ptr(&self) -> *mut ffi::GstCapsFeatures {
         self as *const Self as *mut ffi::GstCapsFeatures
     }
@@ -322,12 +360,14 @@ impl CapsFeaturesRef {
     }
 
     #[doc(alias = "gst_caps_features_contains")]
-    pub fn contains(&self, feature: &str) -> bool {
+    pub fn contains(&self, feature: impl IntoGStr) -> bool {
         unsafe {
-            from_glib(ffi::gst_caps_features_contains(
-                self.as_ptr(),
-                feature.to_glib_none().0,
-            ))
+            feature.run_with_gstr(|feature| {
+                from_glib(ffi::gst_caps_features_contains(
+                    self.as_ptr(),
+                    feature.as_ptr(),
+                ))
+            })
         }
     }
 
@@ -349,7 +389,7 @@ impl CapsFeaturesRef {
 
     #[doc(alias = "get_nth")]
     #[doc(alias = "gst_caps_features_get_nth")]
-    pub fn nth(&self, idx: u32) -> Option<&str> {
+    pub fn nth(&self, idx: u32) -> Option<&glib::GStr> {
         if idx >= self.size() {
             return None;
         }
@@ -360,7 +400,7 @@ impl CapsFeaturesRef {
                 return None;
             }
 
-            Some(CStr::from_ptr(feature).to_str().unwrap())
+            Some(glib::GStr::from_ptr(feature))
         }
     }
 
@@ -377,13 +417,21 @@ impl CapsFeaturesRef {
     }
 
     #[doc(alias = "gst_caps_features_add")]
-    pub fn add(&mut self, feature: &str) {
-        unsafe { ffi::gst_caps_features_add(self.as_mut_ptr(), feature.to_glib_none().0) }
+    pub fn add(&mut self, feature: impl IntoGStr) {
+        unsafe {
+            feature.run_with_gstr(|feature| {
+                ffi::gst_caps_features_add(self.as_mut_ptr(), feature.as_ptr())
+            })
+        }
     }
 
     #[doc(alias = "gst_caps_features_remove")]
-    pub fn remove(&mut self, feature: &str) {
-        unsafe { ffi::gst_caps_features_remove(self.as_mut_ptr(), feature.to_glib_none().0) }
+    pub fn remove(&mut self, feature: impl IntoGStr) {
+        unsafe {
+            feature.run_with_gstr(|feature| {
+                ffi::gst_caps_features_remove(self.as_mut_ptr(), feature.as_ptr())
+            })
+        }
     }
 
     #[doc(alias = "gst_caps_features_add_id")]
@@ -413,6 +461,7 @@ impl CapsFeaturesRef {
 }
 
 impl glib::types::StaticType for CapsFeaturesRef {
+    #[inline]
     fn static_type() -> glib::types::Type {
         unsafe { from_glib(ffi::gst_structure_get_type()) }
     }
@@ -424,12 +473,24 @@ impl<'a> std::iter::Extend<&'a str> for CapsFeaturesRef {
     }
 }
 
+impl<'a> std::iter::Extend<&'a glib::GStr> for CapsFeaturesRef {
+    fn extend<T: IntoIterator<Item = &'a glib::GStr>>(&mut self, iter: T) {
+        iter.into_iter().for_each(|f| self.add(f));
+    }
+}
+
 impl std::iter::Extend<String> for CapsFeaturesRef {
     fn extend<T: IntoIterator<Item = String>>(&mut self, iter: T) {
         iter.into_iter().for_each(|f| self.add(&f));
     }
 }
 
+impl std::iter::Extend<glib::GString> for CapsFeaturesRef {
+    fn extend<T: IntoIterator<Item = glib::GString>>(&mut self, iter: T) {
+        iter.into_iter().for_each(|f| self.add(&f));
+    }
+}
+
 impl std::iter::Extend<glib::Quark> for CapsFeaturesRef {
     fn extend<T: IntoIterator<Item = glib::Quark>>(&mut self, iter: T) {
         iter.into_iter().for_each(|f| self.add_from_quark(f));
@@ -497,7 +558,7 @@ impl<'a> Iter<'a> {
 }
 
 impl<'a> Iterator for Iter<'a> {
-    type Item = &'a str;
+    type Item = &'a glib::GStr;
 
     fn next(&mut self) -> Option<Self::Item> {
         if self.idx >= self.n_features {
@@ -507,11 +568,11 @@ impl<'a> Iterator for Iter<'a> {
         unsafe {
             let feature =
                 ffi::gst_caps_features_get_nth(self.caps_features.as_ptr(), self.idx as u32);
-            assert!(!feature.is_null());
+            debug_assert!(!feature.is_null());
 
             self.idx += 1;
 
-            Some(CStr::from_ptr(feature).to_str().unwrap())
+            Some(glib::GStr::from_ptr(feature))
         }
     }
 
@@ -536,8 +597,8 @@ impl<'a> Iterator for Iter<'a> {
                 self.idx = end + 1;
                 let feature =
                     ffi::gst_caps_features_get_nth(self.caps_features.as_ptr(), end as u32);
-                assert!(!feature.is_null());
-                Some(CStr::from_ptr(feature).to_str().unwrap())
+                debug_assert!(!feature.is_null());
+                Some(glib::GStr::from_ptr(feature))
             }
         }
     }
@@ -551,8 +612,8 @@ impl<'a> Iterator for Iter<'a> {
                     self.caps_features.as_ptr(),
                     self.n_features as u32 - 1,
                 );
-                assert!(!feature.is_null());
-                Some(CStr::from_ptr(feature).to_str().unwrap())
+                debug_assert!(!feature.is_null());
+                Some(glib::GStr::from_ptr(feature))
             }
         }
     }
@@ -569,9 +630,9 @@ impl<'a> DoubleEndedIterator for Iter<'a> {
         unsafe {
             let feature =
                 ffi::gst_caps_features_get_nth(self.caps_features.as_ptr(), self.n_features as u32);
-            assert!(!feature.is_null());
+            debug_assert!(!feature.is_null());
 
-            Some(CStr::from_ptr(feature).to_str().unwrap())
+            Some(glib::GStr::from_ptr(feature))
         }
     }
 
@@ -587,9 +648,9 @@ impl<'a> DoubleEndedIterator for Iter<'a> {
                     self.caps_features.as_ptr(),
                     self.n_features as u32,
                 );
-                assert!(!feature.is_null());
+                debug_assert!(!feature.is_null());
 
-                Some(CStr::from_ptr(feature).to_str().unwrap())
+                Some(glib::GStr::from_ptr(feature))
             }
         }
     }
@@ -601,15 +662,114 @@ impl<'a> std::iter::FusedIterator for Iter<'a> {}
 
 impl<'a> IntoIterator for &'a CapsFeaturesRef {
     type IntoIter = Iter<'a>;
-    type Item = &'a str;
+    type Item = &'a glib::GStr;
 
     fn into_iter(self) -> Self::IntoIter {
         self.iter()
     }
 }
 
+impl<'a> From<&'a str> for CapsFeatures {
+    fn from(value: &'a str) -> Self {
+        skip_assert_initialized!();
+        let mut features = CapsFeatures::new_empty();
+
+        features.add(value);
+
+        features
+    }
+}
+
+impl<'a> From<&'a glib::GStr> for CapsFeatures {
+    fn from(value: &'a glib::GStr) -> Self {
+        skip_assert_initialized!();
+        let mut features = CapsFeatures::new_empty();
+
+        features.add(value);
+
+        features
+    }
+}
+
+impl From<glib::Quark> for CapsFeatures {
+    fn from(value: glib::Quark) -> Self {
+        skip_assert_initialized!();
+        let mut features = CapsFeatures::new_empty();
+
+        features.add_from_quark(value);
+
+        features
+    }
+}
+
+impl<'a, const N: usize> From<[&'a str; N]> for CapsFeatures {
+    fn from(value: [&'a str; N]) -> Self {
+        skip_assert_initialized!();
+        let mut features = CapsFeatures::new_empty();
+
+        value.into_iter().for_each(|f| features.add(f));
+
+        features
+    }
+}
+
+impl<'a, const N: usize> From<[&'a glib::GStr; N]> for CapsFeatures {
+    fn from(value: [&'a glib::GStr; N]) -> Self {
+        skip_assert_initialized!();
+        let mut features = CapsFeatures::new_empty();
+
+        value.into_iter().for_each(|f| features.add(f));
+
+        features
+    }
+}
+
+impl<const N: usize> From<[String; N]> for CapsFeatures {
+    fn from(value: [String; N]) -> Self {
+        skip_assert_initialized!();
+        let mut features = CapsFeatures::new_empty();
+
+        value.into_iter().for_each(|f| features.add(&f));
+
+        features
+    }
+}
+
+impl<const N: usize> From<[glib::GString; N]> for CapsFeatures {
+    fn from(value: [glib::GString; N]) -> Self {
+        skip_assert_initialized!();
+        let mut features = CapsFeatures::new_empty();
+
+        value.into_iter().for_each(|f| features.add(&f));
+
+        features
+    }
+}
+
+impl<const N: usize> From<[glib::Quark; N]> for CapsFeatures {
+    fn from(value: [glib::Quark; N]) -> Self {
+        skip_assert_initialized!();
+        let mut features = CapsFeatures::new_empty();
+
+        value.into_iter().for_each(|f| features.add_from_quark(f));
+
+        features
+    }
+}
+
 impl<'a> std::iter::FromIterator<&'a str> for CapsFeatures {
     fn from_iter<T: IntoIterator<Item = &'a str>>(iter: T) -> Self {
+        skip_assert_initialized!();
+        let mut features = CapsFeatures::new_empty();
+
+        iter.into_iter().for_each(|f| features.add(f));
+
+        features
+    }
+}
+
+impl<'a> std::iter::FromIterator<&'a glib::GStr> for CapsFeatures {
+    fn from_iter<T: IntoIterator<Item = &'a glib::GStr>>(iter: T) -> Self {
         assert_initialized_main_thread!();
 
         let mut features = CapsFeatures::new_empty();
@@ -622,6 +782,17 @@ impl<'a> std::iter::FromIterator<&'a str> for CapsFeatures {
 
 impl std::iter::FromIterator<String> for CapsFeatures {
     fn from_iter<T: IntoIterator<Item = String>>(iter: T) -> Self {
+        skip_assert_initialized!();
+        let mut features = CapsFeatures::new_empty();
+
+        iter.into_iter().for_each(|f| features.add(&f));
+
+        features
+    }
+}
+
+impl std::iter::FromIterator<glib::GString> for CapsFeatures {
+    fn from_iter<T: IntoIterator<Item = glib::GString>>(iter: T) -> Self {
         assert_initialized_main_thread!();
 
         let mut features = CapsFeatures::new_empty();
@@ -634,8 +805,7 @@ impl std::iter::FromIterator<String> for CapsFeatures {
 
 impl std::iter::FromIterator<glib::Quark> for CapsFeatures {
     fn from_iter<T: IntoIterator<Item = glib::Quark>>(iter: T) -> Self {
-        assert_initialized_main_thread!();
-
+        skip_assert_initialized!();
         let mut features = CapsFeatures::new_empty();
 
         iter.into_iter().for_each(|f| features.add_from_quark(f));
@@ -664,6 +834,7 @@ impl fmt::Display for CapsFeaturesRef {
 impl ToOwned for CapsFeaturesRef {
     type Owned = CapsFeatures;
 
+    #[inline]
     fn to_owned(&self) -> CapsFeatures {
         unsafe { from_glib_full(ffi::gst_caps_features_copy(self.as_ptr() as *const _) as *mut _) }
     }
@@ -672,13 +843,10 @@ impl ToOwned for CapsFeaturesRef {
 unsafe impl Sync for CapsFeaturesRef {}
 unsafe impl Send for CapsFeaturesRef {}
 
-pub static CAPS_FEATURE_MEMORY_SYSTEM_MEMORY: Lazy<&'static str> = Lazy::new(|| unsafe {
-    CStr::from_ptr(ffi::GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY)
-        .to_str()
-        .unwrap()
-});
+pub static CAPS_FEATURE_MEMORY_SYSTEM_MEMORY: &glib::GStr =
+    unsafe { glib::GStr::from_utf8_with_nul_unchecked(ffi::GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY) };
 pub static CAPS_FEATURES_MEMORY_SYSTEM_MEMORY: Lazy<CapsFeatures> =
-    Lazy::new(|| CapsFeatures::new(&[*CAPS_FEATURE_MEMORY_SYSTEM_MEMORY]));
+    Lazy::new(|| CapsFeatures::new([CAPS_FEATURE_MEMORY_SYSTEM_MEMORY]));
 
 #[cfg(test)]
 mod tests {
@@ -686,7 +854,7 @@ mod tests {
 
     #[test]
     fn test_from_value_optional() {
-        use glib::ToValue;
+        use glib::value::ToValue;
 
         crate::init().unwrap();
 
