@@ -1,24 +1,21 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use crate::ClockReturn;
-use crate::FlowReturn;
-use crate::PadLinkReturn;
-use crate::State;
-use crate::StateChange;
-use crate::StateChangeReturn;
 use std::{cmp, ops};
+
+use glib::{
+    prelude::*,
+    translate::*,
+    value::{FromValue, ToValue, Value},
+    Type,
+};
 use thiserror::Error;
 
-use glib::translate::*;
-use glib::value::FromValue;
-use glib::value::ToValue;
-use glib::value::Value;
-use glib::StaticType;
-use glib::Type;
+use crate::{ClockReturn, State, StateChange, StateChangeReturn};
 
 macro_rules! impl_return_result_traits {
     ($ffi_type:ident, $ret_type:ident, $ok_type:ident, $err_type:ident) => {
         impl From<$ok_type> for $ret_type {
+            #[inline]
             fn from(value: $ok_type) -> Self {
                 skip_assert_initialized!();
                 $ret_type::from_ok(value)
@@ -28,12 +25,14 @@ macro_rules! impl_return_result_traits {
         impl IntoGlib for $ok_type {
             type GlibType = <$ret_type as IntoGlib>::GlibType;
 
+            #[inline]
             fn into_glib(self) -> Self::GlibType {
                 $ret_type::from_ok(self).into_glib()
             }
         }
 
         impl From<$err_type> for $ret_type {
+            #[inline]
             fn from(value: $err_type) -> Self {
                 skip_assert_initialized!();
                 $ret_type::from_error(value)
@@ -43,12 +42,14 @@ macro_rules! impl_return_result_traits {
         impl IntoGlib for $err_type {
             type GlibType = <$ret_type as IntoGlib>::GlibType;
 
+            #[inline]
             fn into_glib(self) -> Self::GlibType {
                 $ret_type::from_error(self).into_glib()
             }
         }
 
         impl From<Result<$ok_type, $err_type>> for $ret_type {
+            #[inline]
             fn from(res: Result<$ok_type, $err_type>) -> Self {
                 skip_assert_initialized!();
                 match res {
@@ -60,6 +61,8 @@ macro_rules! impl_return_result_traits {
 
         impl TryFromGlib<ffi::$ffi_type> for $ok_type {
             type Error = $err_type;
+
+            #[inline]
             unsafe fn try_from_glib(val: ffi::$ffi_type) -> Result<$ok_type, $err_type> {
                 skip_assert_initialized!();
                 $ret_type::from_glib(val).into_result()
@@ -69,36 +72,33 @@ macro_rules! impl_return_result_traits {
 }
 
 impl StateChangeReturn {
+    #[inline]
     pub fn into_result(self) -> Result<StateChangeSuccess, StateChangeError> {
         match self {
-            StateChangeReturn::Success => Ok(StateChangeSuccess::Success),
-            StateChangeReturn::Async => Ok(StateChangeSuccess::Async),
-            StateChangeReturn::NoPreroll => Ok(StateChangeSuccess::NoPreroll),
             StateChangeReturn::Failure => Err(StateChangeError),
-            _ => Err(StateChangeError),
+            _ => Ok(unsafe { std::mem::transmute(self) }),
         }
     }
 
+    #[inline]
     pub fn from_error(_: StateChangeError) -> Self {
         skip_assert_initialized!();
         StateChangeReturn::Failure
     }
 
+    #[inline]
     pub fn from_ok(v: StateChangeSuccess) -> Self {
         skip_assert_initialized!();
-        match v {
-            StateChangeSuccess::Success => StateChangeReturn::Success,
-            StateChangeSuccess::Async => StateChangeReturn::Async,
-            StateChangeSuccess::NoPreroll => StateChangeReturn::NoPreroll,
-        }
+        unsafe { std::mem::transmute(v) }
     }
 }
 
 #[derive(Clone, Copy, Debug, Eq, PartialEq, Hash)]
+#[repr(i32)]
 pub enum StateChangeSuccess {
-    Success,
-    Async,
-    NoPreroll,
+    Success = ffi::GST_STATE_CHANGE_SUCCESS,
+    Async = ffi::GST_STATE_CHANGE_ASYNC,
+    NoPreroll = ffi::GST_STATE_CHANGE_NO_PREROLL,
 }
 
 #[derive(Clone, Copy, Debug, Eq, PartialEq, Hash, Error)]
@@ -113,111 +113,281 @@ impl_return_result_traits!(
     StateChangeError
 );
 
+#[must_use]
+#[derive(Debug, Eq, PartialEq, Ord, PartialOrd, Hash, Clone, Copy)]
+#[repr(i32)]
+#[doc(alias = "GstFlowReturn")]
+pub enum FlowReturn {
+    #[doc(alias = "GST_FLOW_CUSTOM_SUCCESS_2")]
+    CustomSuccess2 = ffi::GST_FLOW_CUSTOM_SUCCESS_2,
+    #[doc(alias = "GST_FLOW_CUSTOM_SUCCESS_1")]
+    CustomSuccess1 = ffi::GST_FLOW_CUSTOM_SUCCESS_1,
+    #[doc(alias = "GST_FLOW_CUSTOM_SUCCESS")]
+    CustomSuccess = ffi::GST_FLOW_CUSTOM_SUCCESS,
+    #[doc(alias = "GST_FLOW_OK")]
+    Ok = ffi::GST_FLOW_OK,
+    #[doc(alias = "GST_FLOW_NOT_LINKED")]
+    NotLinked = ffi::GST_FLOW_NOT_LINKED,
+    #[doc(alias = "GST_FLOW_FLUSHING")]
+    Flushing = ffi::GST_FLOW_FLUSHING,
+    #[doc(alias = "GST_FLOW_EOS")]
+    Eos = ffi::GST_FLOW_EOS,
+    #[doc(alias = "GST_FLOW_NOT_NEGOTIATED")]
+    NotNegotiated = ffi::GST_FLOW_NOT_NEGOTIATED,
+    #[doc(alias = "GST_FLOW_ERROR")]
+    Error = ffi::GST_FLOW_ERROR,
+    #[doc(alias = "GST_FLOW_NOT_SUPPORTED")]
+    NotSupported = ffi::GST_FLOW_NOT_SUPPORTED,
+    #[doc(alias = "GST_FLOW_CUSTOM_ERROR")]
+    CustomError = ffi::GST_FLOW_CUSTOM_ERROR,
+    #[doc(alias = "GST_FLOW_CUSTOM_ERROR_1")]
+    CustomError1 = ffi::GST_FLOW_CUSTOM_ERROR_1,
+    #[doc(alias = "GST_FLOW_CUSTOM_ERROR_2")]
+    CustomError2 = ffi::GST_FLOW_CUSTOM_ERROR_2,
+}
+
+#[doc(hidden)]
+impl IntoGlib for FlowReturn {
+    type GlibType = ffi::GstFlowReturn;
+
+    #[inline]
+    fn into_glib(self) -> ffi::GstFlowReturn {
+        self as ffi::GstFlowReturn
+    }
+}
+
+#[doc(hidden)]
+impl FromGlib<ffi::GstFlowReturn> for FlowReturn {
+    #[inline]
+    unsafe fn from_glib(value: ffi::GstFlowReturn) -> Self {
+        skip_assert_initialized!();
+
+        if value < ffi::GST_FLOW_NOT_SUPPORTED
+            && (value > ffi::GST_FLOW_CUSTOM_ERROR || value < ffi::GST_FLOW_CUSTOM_ERROR_2)
+        {
+            FlowReturn::Error
+        } else if value > 0
+            && (value < ffi::GST_FLOW_CUSTOM_SUCCESS || value > ffi::GST_FLOW_CUSTOM_SUCCESS_2)
+        {
+            FlowReturn::Ok
+        } else {
+            std::mem::transmute(value)
+        }
+    }
+}
+
+impl StaticType for FlowReturn {
+    #[inline]
+    fn static_type() -> Type {
+        unsafe { from_glib(ffi::gst_flow_return_get_type()) }
+    }
+}
+
+impl glib::value::ValueType for FlowReturn {
+    type Type = Self;
+}
+
+unsafe impl<'a> FromValue<'a> for FlowReturn {
+    type Checker = glib::value::GenericValueTypeChecker<Self>;
+
+    #[inline]
+    unsafe fn from_value(value: &'a glib::Value) -> Self {
+        skip_assert_initialized!();
+        from_glib(glib::gobject_ffi::g_value_get_enum(value.to_glib_none().0))
+    }
+}
+
+impl ToValue for FlowReturn {
+    #[inline]
+    fn to_value(&self) -> glib::Value {
+        let mut value = glib::Value::for_value_type::<Self>();
+        unsafe {
+            glib::gobject_ffi::g_value_set_enum(value.to_glib_none_mut().0, self.into_glib());
+        }
+        value
+    }
+
+    #[inline]
+    fn value_type(&self) -> glib::Type {
+        Self::static_type()
+    }
+}
+
+impl From<FlowReturn> for glib::Value {
+    #[inline]
+    fn from(v: FlowReturn) -> Self {
+        skip_assert_initialized!();
+        ToValue::to_value(&v)
+    }
+}
+
 impl FlowReturn {
+    #[inline]
     pub fn into_result(self) -> Result<FlowSuccess, FlowError> {
-        match self {
-            FlowReturn::CustomSuccess2 => Ok(FlowSuccess::CustomSuccess2),
-            FlowReturn::CustomSuccess1 => Ok(FlowSuccess::CustomSuccess1),
-            FlowReturn::CustomSuccess => Ok(FlowSuccess::CustomSuccess),
-            FlowReturn::Ok => Ok(FlowSuccess::Ok),
-            FlowReturn::NotLinked => Err(FlowError::NotLinked),
-            FlowReturn::Flushing => Err(FlowError::Flushing),
-            FlowReturn::Eos => Err(FlowError::Eos),
-            FlowReturn::NotNegotiated => Err(FlowError::NotNegotiated),
-            FlowReturn::Error => Err(FlowError::Error),
-            FlowReturn::NotSupported => Err(FlowError::NotSupported),
-            FlowReturn::CustomError => Err(FlowError::CustomError),
-            FlowReturn::CustomError1 => Err(FlowError::CustomError1),
-            FlowReturn::CustomError2 => Err(FlowError::CustomError2),
-            _ => Err(FlowError::Error),
+        if self.into_glib() >= 0 {
+            Ok(unsafe { std::mem::transmute(self) })
+        } else {
+            Err(unsafe { std::mem::transmute(self) })
         }
     }
 
+    #[inline]
     pub fn from_error(v: FlowError) -> Self {
         skip_assert_initialized!();
-        match v {
-            FlowError::NotLinked => FlowReturn::NotLinked,
-            FlowError::Flushing => FlowReturn::Flushing,
-            FlowError::Eos => FlowReturn::Eos,
-            FlowError::NotNegotiated => FlowReturn::NotNegotiated,
-            FlowError::Error => FlowReturn::Error,
-            FlowError::NotSupported => FlowReturn::NotSupported,
-            FlowError::CustomError => FlowReturn::CustomError,
-            FlowError::CustomError1 => FlowReturn::CustomError1,
-            FlowError::CustomError2 => FlowReturn::CustomError2,
-        }
+        unsafe { std::mem::transmute(v) }
     }
 
+    #[inline]
     pub fn from_ok(v: FlowSuccess) -> Self {
         skip_assert_initialized!();
-        match v {
-            FlowSuccess::CustomSuccess2 => FlowReturn::CustomSuccess2,
-            FlowSuccess::CustomSuccess1 => FlowReturn::CustomSuccess1,
-            FlowSuccess::CustomSuccess => FlowReturn::CustomSuccess,
-            FlowSuccess::Ok => FlowReturn::Ok,
-        }
+        unsafe { std::mem::transmute(v) }
     }
 }
 
 #[derive(Clone, Copy, Debug, Eq, PartialEq, Hash)]
+#[repr(i32)]
 pub enum FlowSuccess {
-    CustomSuccess2,
-    CustomSuccess1,
-    CustomSuccess,
-    Ok,
+    CustomSuccess2 = ffi::GST_FLOW_CUSTOM_SUCCESS_2,
+    CustomSuccess1 = ffi::GST_FLOW_CUSTOM_SUCCESS_1,
+    CustomSuccess = ffi::GST_FLOW_CUSTOM_SUCCESS,
+    Ok = ffi::GST_FLOW_OK,
 }
 
 #[derive(Clone, Copy, Debug, Eq, PartialEq, Hash, Error)]
 #[must_use]
+#[repr(i32)]
 pub enum FlowError {
     #[error("Pad is not linked")]
-    NotLinked,
+    NotLinked = ffi::GST_FLOW_NOT_LINKED,
     #[error("Pad is flushing")]
-    Flushing,
+    Flushing = ffi::GST_FLOW_FLUSHING,
     #[error("Pad is EOS")]
-    Eos,
+    Eos = ffi::GST_FLOW_EOS,
     #[error("Pad is not negotiated")]
-    NotNegotiated,
+    NotNegotiated = ffi::GST_FLOW_NOT_NEGOTIATED,
     #[error("Some (fatal) error occurred. Element generating this error should post an error message with more details")]
-    Error,
+    Error = ffi::GST_FLOW_ERROR,
     #[error("This operation is not supported")]
-    NotSupported,
+    NotSupported = ffi::GST_FLOW_NOT_SUPPORTED,
     #[error("Elements can use values starting from this (and lower) to define custom error codes")]
-    CustomError,
+    CustomError = ffi::GST_FLOW_CUSTOM_ERROR,
     #[error("Pre-defined custom error code")]
-    CustomError1,
+    CustomError1 = ffi::GST_FLOW_CUSTOM_ERROR_1,
     #[error("Pre-defined custom error code")]
-    CustomError2,
+    CustomError2 = ffi::GST_FLOW_CUSTOM_ERROR_2,
 }
 
 impl_return_result_traits!(GstFlowReturn, FlowReturn, FlowSuccess, FlowError);
 
+#[must_use]
+#[derive(Debug, Eq, PartialEq, Ord, PartialOrd, Hash, Clone, Copy)]
+#[repr(i32)]
+#[doc(alias = "GstPadLinkReturn")]
+pub enum PadLinkReturn {
+    #[doc(alias = "GST_PAD_LINK_OK")]
+    Ok = ffi::GST_PAD_LINK_OK,
+    #[doc(alias = "GST_PAD_LINK_WRONG_HIERARCHY")]
+    WrongHierarchy = ffi::GST_PAD_LINK_WRONG_HIERARCHY,
+    #[doc(alias = "GST_PAD_LINK_WAS_LINKED")]
+    WasLinked = ffi::GST_PAD_LINK_WAS_LINKED,
+    #[doc(alias = "GST_PAD_LINK_WRONG_DIRECTION")]
+    WrongDirection = ffi::GST_PAD_LINK_WRONG_DIRECTION,
+    #[doc(alias = "GST_PAD_LINK_NOFORMAT")]
+    Noformat = ffi::GST_PAD_LINK_NOFORMAT,
+    #[doc(alias = "GST_PAD_LINK_NOSCHED")]
+    Nosched = ffi::GST_PAD_LINK_NOSCHED,
+    #[doc(alias = "GST_PAD_LINK_REFUSED")]
+    Refused = ffi::GST_PAD_LINK_REFUSED,
+}
+
+#[doc(hidden)]
+impl IntoGlib for PadLinkReturn {
+    type GlibType = ffi::GstPadLinkReturn;
+
+    #[inline]
+    fn into_glib(self) -> ffi::GstPadLinkReturn {
+        self as ffi::GstPadLinkReturn
+    }
+}
+
+#[doc(hidden)]
+impl FromGlib<ffi::GstPadLinkReturn> for PadLinkReturn {
+    #[inline]
+    unsafe fn from_glib(value: ffi::GstPadLinkReturn) -> Self {
+        skip_assert_initialized!();
+
+        if value >= 0 {
+            PadLinkReturn::Ok
+        } else if value < ffi::GST_PAD_LINK_REFUSED {
+            PadLinkReturn::Refused
+        } else {
+            std::mem::transmute(value)
+        }
+    }
+}
+
+impl StaticType for PadLinkReturn {
+    #[inline]
+    fn static_type() -> Type {
+        unsafe { from_glib(ffi::gst_pad_link_return_get_type()) }
+    }
+}
+
+impl glib::value::ValueType for PadLinkReturn {
+    type Type = Self;
+}
+
+unsafe impl<'a> FromValue<'a> for PadLinkReturn {
+    type Checker = glib::value::GenericValueTypeChecker<Self>;
+
+    #[inline]
+    unsafe fn from_value(value: &'a glib::Value) -> Self {
+        skip_assert_initialized!();
+        from_glib(glib::gobject_ffi::g_value_get_enum(value.to_glib_none().0))
+    }
+}
+
+impl ToValue for PadLinkReturn {
+    #[inline]
+    fn to_value(&self) -> glib::Value {
+        let mut value = glib::Value::for_value_type::<Self>();
+        unsafe {
+            glib::gobject_ffi::g_value_set_enum(value.to_glib_none_mut().0, self.into_glib());
+        }
+        value
+    }
+
+    #[inline]
+    fn value_type(&self) -> glib::Type {
+        Self::static_type()
+    }
+}
+
+impl From<PadLinkReturn> for glib::Value {
+    #[inline]
+    fn from(v: PadLinkReturn) -> Self {
+        skip_assert_initialized!();
+        ToValue::to_value(&v)
+    }
+}
+
 impl PadLinkReturn {
+    #[inline]
     pub fn into_result(self) -> Result<PadLinkSuccess, PadLinkError> {
-        match self {
-            PadLinkReturn::Ok => Ok(PadLinkSuccess),
-            PadLinkReturn::WrongHierarchy => Err(PadLinkError::WrongHierarchy),
-            PadLinkReturn::WasLinked => Err(PadLinkError::WasLinked),
-            PadLinkReturn::WrongDirection => Err(PadLinkError::WrongDirection),
-            PadLinkReturn::Noformat => Err(PadLinkError::Noformat),
-            PadLinkReturn::Nosched => Err(PadLinkError::Nosched),
-            PadLinkReturn::Refused => Err(PadLinkError::Refused),
-            _ => Err(PadLinkError::Refused),
+        if self == PadLinkReturn::Ok {
+            Ok(PadLinkSuccess)
+        } else {
+            Err(unsafe { std::mem::transmute(self) })
         }
     }
 
+    #[inline]
     pub fn from_error(v: PadLinkError) -> Self {
         skip_assert_initialized!();
-        match v {
-            PadLinkError::WrongHierarchy => PadLinkReturn::WrongHierarchy,
-            PadLinkError::WasLinked => PadLinkReturn::WasLinked,
-            PadLinkError::WrongDirection => PadLinkReturn::WrongDirection,
-            PadLinkError::Noformat => PadLinkReturn::Noformat,
-            PadLinkError::Nosched => PadLinkReturn::Nosched,
-            PadLinkError::Refused => PadLinkReturn::Refused,
-        }
+        unsafe { std::mem::transmute(v) }
     }
 
+    #[inline]
     pub fn from_ok(_: PadLinkSuccess) -> Self {
         skip_assert_initialized!();
         PadLinkReturn::Ok
@@ -229,19 +399,20 @@ pub struct PadLinkSuccess;
 
 #[derive(Clone, Copy, Debug, Eq, PartialEq, Hash, Error)]
 #[must_use]
+#[repr(i32)]
 pub enum PadLinkError {
     #[error("Pads have no common grandparent")]
-    WrongHierarchy,
+    WrongHierarchy = ffi::GST_PAD_LINK_WRONG_HIERARCHY,
     #[error("Pad was already linked")]
-    WasLinked,
+    WasLinked = ffi::GST_PAD_LINK_WAS_LINKED,
     #[error("Pads have wrong direction")]
-    WrongDirection,
+    WrongDirection = ffi::GST_PAD_LINK_WRONG_DIRECTION,
     #[error("Pads do not have common format")]
-    Noformat,
+    Noformat = ffi::GST_PAD_LINK_NOFORMAT,
     #[error("Pads cannot cooperate in scheduling")]
-    Nosched,
+    Nosched = ffi::GST_PAD_LINK_NOSCHED,
     #[error("Refused for some other reason")]
-    Refused,
+    Refused = ffi::GST_PAD_LINK_REFUSED,
 }
 
 impl_return_result_traits!(
@@ -252,67 +423,57 @@ impl_return_result_traits!(
 );
 
 impl ClockReturn {
+    #[inline]
     pub fn into_result(self) -> Result<ClockSuccess, ClockError> {
         match self {
             ClockReturn::Ok => Ok(ClockSuccess::Ok),
             ClockReturn::Done => Ok(ClockSuccess::Done),
-            ClockReturn::Early => Err(ClockError::Early),
-            ClockReturn::Unscheduled => Err(ClockError::Unscheduled),
-            ClockReturn::Busy => Err(ClockError::Busy),
-            ClockReturn::Badtime => Err(ClockError::Badtime),
-            ClockReturn::Error => Err(ClockError::Error),
-            ClockReturn::Unsupported => Err(ClockError::Unsupported),
-            _ => Err(ClockError::Error),
+            _ => Err(unsafe { std::mem::transmute(self) }),
         }
     }
 
+    #[inline]
     pub fn from_error(v: ClockError) -> Self {
         skip_assert_initialized!();
-        match v {
-            ClockError::Early => ClockReturn::Early,
-            ClockError::Unscheduled => ClockReturn::Unscheduled,
-            ClockError::Busy => ClockReturn::Busy,
-            ClockError::Badtime => ClockReturn::Badtime,
-            ClockError::Error => ClockReturn::Error,
-            ClockError::Unsupported => ClockReturn::Unsupported,
-        }
+        unsafe { std::mem::transmute(v) }
     }
 
+    #[inline]
     pub fn from_ok(v: ClockSuccess) -> Self {
         skip_assert_initialized!();
-        match v {
-            ClockSuccess::Ok => ClockReturn::Ok,
-            ClockSuccess::Done => ClockReturn::Done,
-        }
+        unsafe { std::mem::transmute(v) }
     }
 }
 
 #[derive(Clone, Copy, Debug, Eq, PartialEq, Hash)]
+#[repr(i32)]
 pub enum ClockSuccess {
-    Ok,
-    Done,
+    Ok = ffi::GST_CLOCK_OK,
+    Done = ffi::GST_CLOCK_DONE,
 }
 
 #[derive(Clone, Copy, Debug, Eq, PartialEq, Hash, Error)]
 #[must_use]
+#[repr(i32)]
 pub enum ClockError {
     #[error("The operation was scheduled too late")]
-    Early,
+    Early = ffi::GST_CLOCK_EARLY,
     #[error("The clockID was unscheduled")]
-    Unscheduled,
+    Unscheduled = ffi::GST_CLOCK_UNSCHEDULED,
     #[error("The ClockID is busy")]
-    Busy,
+    Busy = ffi::GST_CLOCK_BUSY,
     #[error("A bad time was provided to a function")]
-    Badtime,
+    Badtime = ffi::GST_CLOCK_BADTIME,
     #[error("An error occurred")]
-    Error,
+    Error = ffi::GST_CLOCK_ERROR,
     #[error("Operation is not supported")]
-    Unsupported,
+    Unsupported = ffi::GST_CLOCK_UNSUPPORTED,
 }
 
 impl_return_result_traits!(GstClockReturn, ClockReturn, ClockSuccess, ClockError);
 
 impl PartialEq for crate::TypeFindProbability {
+    #[inline]
     fn eq(&self, other: &crate::TypeFindProbability) -> bool {
         (self.into_glib() as u32).eq(&(other.into_glib() as u32))
     }
@@ -321,12 +482,14 @@ impl PartialEq for crate::TypeFindProbability {
 impl Eq for crate::TypeFindProbability {}
 
 impl PartialOrd for crate::TypeFindProbability {
+    #[inline]
     fn partial_cmp(&self, other: &Self) -> Option<cmp::Ordering> {
-        (self.into_glib() as u32).partial_cmp(&(other.into_glib() as u32))
+        Some(self.cmp(other))
     }
 }
 
 impl Ord for crate::TypeFindProbability {
+    #[inline]
     fn cmp(&self, other: &Self) -> cmp::Ordering {
         (self.into_glib() as u32).cmp(&(other.into_glib() as u32))
     }
@@ -335,6 +498,7 @@ impl Ord for crate::TypeFindProbability {
 impl ops::Add<u32> for crate::TypeFindProbability {
     type Output = crate::TypeFindProbability;
 
+    #[inline]
     fn add(self, rhs: u32) -> crate::TypeFindProbability {
         let res = (self.into_glib() as u32).saturating_add(rhs);
         unsafe { from_glib(res as i32) }
@@ -342,6 +506,7 @@ impl ops::Add<u32> for crate::TypeFindProbability {
 }
 
 impl ops::AddAssign<u32> for crate::TypeFindProbability {
+    #[inline]
     fn add_assign(&mut self, rhs: u32) {
         let res = (self.into_glib() as u32).saturating_add(rhs);
         *self = unsafe { from_glib(res as i32) };
@@ -351,6 +516,7 @@ impl ops::AddAssign<u32> for crate::TypeFindProbability {
 impl ops::Sub<u32> for crate::TypeFindProbability {
     type Output = crate::TypeFindProbability;
 
+    #[inline]
     fn sub(self, rhs: u32) -> crate::TypeFindProbability {
         let res = (self.into_glib() as u32).saturating_sub(rhs);
         unsafe { from_glib(res as i32) }
@@ -358,58 +524,7 @@ impl ops::Sub<u32> for crate::TypeFindProbability {
 }
 
 impl ops::SubAssign<u32> for crate::TypeFindProbability {
-    fn sub_assign(&mut self, rhs: u32) {
-        let res = (self.into_glib() as u32).saturating_sub(rhs);
-        *self = unsafe { from_glib(res as i32) };
-    }
-}
-
-impl PartialEq for crate::Rank {
-    fn eq(&self, other: &crate::Rank) -> bool {
-        (self.into_glib() as u32).eq(&(other.into_glib() as u32))
-    }
-}
-
-impl Eq for crate::Rank {}
-
-impl PartialOrd for crate::Rank {
-    fn partial_cmp(&self, other: &Self) -> Option<cmp::Ordering> {
-        (self.into_glib() as u32).partial_cmp(&(other.into_glib() as u32))
-    }
-}
-
-impl Ord for crate::Rank {
-    fn cmp(&self, other: &Self) -> cmp::Ordering {
-        (self.into_glib() as u32).cmp(&(other.into_glib() as u32))
-    }
-}
-
-impl ops::Add<u32> for crate::Rank {
-    type Output = crate::Rank;
-
-    fn add(self, rhs: u32) -> crate::Rank {
-        let res = (self.into_glib() as u32).saturating_add(rhs);
-        unsafe { from_glib(res as i32) }
-    }
-}
-
-impl ops::AddAssign<u32> for crate::Rank {
-    fn add_assign(&mut self, rhs: u32) {
-        let res = (self.into_glib() as u32).saturating_add(rhs);
-        *self = unsafe { from_glib(res as i32) };
-    }
-}
-
-impl ops::Sub<u32> for crate::Rank {
-    type Output = crate::Rank;
-
-    fn sub(self, rhs: u32) -> crate::Rank {
-        let res = (self.into_glib() as u32).saturating_sub(rhs);
-        unsafe { from_glib(res as i32) }
-    }
-}
-
-impl ops::SubAssign<u32> for crate::Rank {
+    #[inline]
     fn sub_assign(&mut self, rhs: u32) {
         let res = (self.into_glib() as u32).saturating_sub(rhs);
         *self = unsafe { from_glib(res as i32) };
@@ -427,6 +542,7 @@ pub enum TagError {
 // enum instead
 #[derive(Debug, PartialEq, Eq, Hash, Clone, Copy)]
 #[doc(alias = "GstMessageType")]
+#[non_exhaustive]
 pub enum MessageType {
     #[doc(alias = "GST_MESSAGE_UNKNOWN")]
     Unknown,
@@ -611,6 +727,7 @@ impl FromGlib<ffi::GstMessageType> for MessageType {
 }
 
 impl StaticType for MessageType {
+    #[inline]
     fn static_type() -> Type {
         unsafe { from_glib(ffi::gst_message_type_get_type()) }
     }
@@ -623,6 +740,7 @@ impl glib::value::ValueType for MessageType {
 unsafe impl<'a> FromValue<'a> for MessageType {
     type Checker = glib::value::GenericValueTypeChecker<Self>;
 
+    #[inline]
     unsafe fn from_value(value: &glib::Value) -> Self {
         skip_assert_initialized!();
         from_glib(glib::gobject_ffi::g_value_get_enum(value.to_glib_none().0) as ffi::GstMessageType)
@@ -630,6 +748,7 @@ unsafe impl<'a> FromValue<'a> for MessageType {
 }
 
 impl ToValue for MessageType {
+    #[inline]
     fn to_value(&self) -> Value {
         let mut value = glib::Value::for_value_type::<Self>();
         unsafe {
@@ -638,13 +757,23 @@ impl ToValue for MessageType {
         value
     }
 
+    #[inline]
     fn value_type(&self) -> Type {
         Self::static_type()
     }
 }
 
+impl From<MessageType> for glib::Value {
+    #[inline]
+    fn from(v: MessageType) -> glib::Value {
+        skip_assert_initialized!();
+        ToValue::to_value(&v)
+    }
+}
+
 impl State {
     #[must_use]
+    #[inline]
     pub fn next(self, pending: Self) -> Self {
         let current = self.into_glib();
         let pending = pending.into_glib();
@@ -656,6 +785,7 @@ impl State {
 }
 
 impl StateChange {
+    #[inline]
     pub fn new(current: State, next: State) -> Self {
         skip_assert_initialized!();
         let current = current.into_glib();
@@ -663,35 +793,13 @@ impl StateChange {
         unsafe { from_glib((current << 3) | next) }
     }
 
+    #[inline]
     pub fn current(self) -> State {
-        match self {
-            StateChange::NullToReady => State::Null,
-            StateChange::ReadyToPaused => State::Ready,
-            StateChange::PausedToPlaying => State::Paused,
-            StateChange::PlayingToPaused => State::Playing,
-            StateChange::PausedToReady => State::Paused,
-            StateChange::ReadyToNull => State::Ready,
-            StateChange::NullToNull => State::Null,
-            StateChange::ReadyToReady => State::Ready,
-            StateChange::PausedToPaused => State::Paused,
-            StateChange::PlayingToPlaying => State::Playing,
-            StateChange::__Unknown(value) => State::__Unknown(value >> 3),
-        }
+        unsafe { from_glib(self.into_glib() >> 3) }
     }
 
+    #[inline]
     pub fn next(self) -> State {
-        match self {
-            StateChange::NullToReady => State::Ready,
-            StateChange::ReadyToPaused => State::Paused,
-            StateChange::PausedToPlaying => State::Playing,
-            StateChange::PlayingToPaused => State::Paused,
-            StateChange::PausedToReady => State::Ready,
-            StateChange::ReadyToNull => State::Null,
-            StateChange::NullToNull => State::Null,
-            StateChange::ReadyToReady => State::Ready,
-            StateChange::PausedToPaused => State::Paused,
-            StateChange::PlayingToPlaying => State::Playing,
-            StateChange::__Unknown(value) => State::__Unknown(value & 0x7),
-        }
+        unsafe { from_glib(self.into_glib() & 0x7) }
     }
 }
