@@ -74,6 +74,14 @@ impl TryFromGlib<i64> for Other {
     }
 }
 
+impl TryFrom<Other> for usize {
+    type Error = std::num::TryFromIntError;
+
+    fn try_from(value: Other) -> Result<Self, Self::Error> {
+        value.0.try_into()
+    }
+}
+
 // FIXME `functions in traits cannot be const` (rustc 1.64.0)
 // rustdoc-stripper-ignore-next
 /// `Other` formatted value constructor trait.
@@ -452,7 +460,7 @@ mod tests {
     #[test]
     #[allow(clippy::eq_op, clippy::op_ref)]
     fn other() {
-        // Check a few ops on `Other`, better converage for
+        // Check a few ops on `Other`, better coverage for
         // the macro ops impl ensured as part of the `clock_time` module.
 
         use opt_ops::prelude::*;
