@@ -1,6 +1,6 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-#![cfg_attr(feature = "dox", feature(doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![recursion_limit = "256"]
 #![allow(clippy::missing_safety_doc)]
 #![allow(clippy::manual_range_contains)]
@@ -41,17 +41,27 @@ macro_rules! skip_assert_initialized {
 #[allow(clippy::needless_borrow)]
 #[allow(clippy::let_unit_value)]
 mod auto;
-pub use crate::auto::{functions::*, *};
+pub use crate::auto::*;
 
 #[macro_use]
 #[cfg(feature = "serde")]
 mod serde_macros;
-#[cfg(feature = "serde")]
-pub use crate::serde_macros::*;
 
 #[macro_use]
-mod log;
-pub use crate::log::*;
+pub mod log;
+pub use crate::log::{
+    DebugCategory, DebugLogFunction, DebugMessage, LoggedObject, CAT_BUFFER, CAT_BUFFER_LIST,
+    CAT_BUS, CAT_CALL_TRACE, CAT_CAPS, CAT_CLOCK, CAT_CONTEXT, CAT_DEFAULT, CAT_ELEMENT_PADS,
+    CAT_ERROR_SYSTEM, CAT_EVENT, CAT_GST_INIT, CAT_LOCKING, CAT_MEMORY, CAT_MESSAGE, CAT_META,
+    CAT_NEGOTIATION, CAT_PADS, CAT_PARAMS, CAT_PARENTAGE, CAT_PERFORMANCE, CAT_PIPELINE,
+    CAT_PLUGIN_INFO, CAT_PLUGIN_LOADING, CAT_PROBE, CAT_PROPERTIES, CAT_QOS, CAT_REFCOUNTING,
+    CAT_REGISTRY, CAT_RUST, CAT_SCHEDULING, CAT_SIGNAL, CAT_STATES,
+};
+
+#[cfg(target_os = "macos")]
+mod macos;
+#[cfg(target_os = "macos")]
+pub use crate::macos::*;
 
 #[macro_use]
 mod error;
@@ -100,8 +110,8 @@ pub use crate::tags::{
 mod tags_serde;
 
 pub mod meta;
-#[cfg(any(feature = "v1_16", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+#[cfg(feature = "v1_16")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
 pub use crate::meta::MetaSeqnum;
 pub use crate::meta::{
     Meta, MetaAPI, MetaAPIExt, MetaRef, MetaRefMut, ParentBufferMeta, ProtectionMeta,
@@ -114,9 +124,11 @@ pub use crate::buffer::{
 mod buffer_cursor;
 pub use crate::buffer_cursor::{BufferCursor, BufferRefCursor};
 pub mod memory;
+mod memory_wrapped;
 pub use crate::memory::{MappedMemory, Memory, MemoryMap, MemoryRef};
 #[cfg(feature = "serde")]
 mod buffer_serde;
+pub mod slice;
 
 pub mod sample;
 pub use crate::sample::{Sample, SampleRef};
@@ -134,6 +146,8 @@ pub mod event;
 pub use crate::event::{Event, EventRef, EventView, GroupId, Seqnum};
 pub mod context;
 pub use crate::context::{Context, ContextRef};
+mod rank;
+pub use crate::rank::Rank;
 mod static_caps;
 pub use crate::static_caps::*;
 mod static_pad_template;
@@ -147,8 +161,10 @@ mod element;
 pub mod element_factory;
 
 mod bin;
+pub use bin::BinBuilder;
 
 mod pipeline;
+pub use pipeline::PipelineBuilder;
 
 mod allocation_params;
 pub use self::allocation_params::AllocationParams;
@@ -161,9 +177,9 @@ mod tracer;
 mod tracer_factory;
 
 // OS dependent Bus extensions (also import the other platform mod for doc)
-#[cfg(any(unix, feature = "dox"))]
+#[cfg(any(unix, docsrs))]
 mod bus_unix;
-#[cfg(any(windows, feature = "dox"))]
+#[cfg(any(windows, docsrs))]
 mod bus_windows;
 
 mod child_proxy;
@@ -242,13 +258,15 @@ pub use pad_template::PadTemplateBuilder;
 pub mod param_spec;
 pub use crate::param_spec::{ParamSpecArray, ParamSpecFraction};
 
-pub mod functions;
+mod functions;
 pub use crate::functions::*;
 
 mod utils;
 pub use crate::utils::ObjectLockGuard;
 
-#[cfg(any(feature = "v1_18", feature = "dox"))]
+pub mod parse;
+
+#[cfg(feature = "v1_18")]
 mod gtype;
 
 use std::ptr;
@@ -288,9 +306,9 @@ pub const PARAM_FLAG_CONTROLLABLE: glib::ParamFlags = glib::ParamFlags::USER_1;
 pub const PARAM_FLAG_MUTABLE_READY: glib::ParamFlags = glib::ParamFlags::USER_2;
 pub const PARAM_FLAG_MUTABLE_PAUSED: glib::ParamFlags = glib::ParamFlags::USER_3;
 pub const PARAM_FLAG_MUTABLE_PLAYING: glib::ParamFlags = glib::ParamFlags::USER_4;
-#[cfg(any(feature = "v1_18", feature = "dox"))]
+#[cfg(feature = "v1_18")]
 pub const PARAM_FLAG_DOC_SHOW_DEFAULT: glib::ParamFlags = glib::ParamFlags::USER_5;
-#[cfg(any(feature = "v1_18", feature = "dox"))]
+#[cfg(feature = "v1_18")]
 pub const PARAM_FLAG_CONDITIONALLY_AVAILABLE: glib::ParamFlags = glib::ParamFlags::USER_6;
 
 // Re-export all the traits in a prelude module, so that applications
@@ -302,11 +320,11 @@ pub mod prelude {
     pub use opt_ops::prelude::*;
 
     // OS dependent Bus extensions (also import the other platform trait for doc)
-    #[cfg(any(unix, feature = "dox"))]
+    #[cfg(any(unix, docsrs))]
     pub use crate::bus_unix::UnixBusExtManual;
-    #[cfg(any(windows, feature = "dox"))]
+    #[cfg(any(windows, docsrs))]
     pub use crate::bus_windows::WindowsBusExtManual;
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
+    #[cfg(feature = "v1_18")]
     pub use crate::gtype::PluginApiExt;
     pub use crate::{
         auto::traits::*,
@@ -315,7 +333,7 @@ pub mod prelude {
         child_proxy::ChildProxyExtManual,
         clock::ClockExtManual,
         device_monitor::DeviceMonitorExtManual,
-        device_provider::DeviceProviderExtManual,
+        device_provider::{DeviceProviderClassExt, DeviceProviderExtManual},
         element::{ElementClassExt, ElementExtManual},
         format::prelude::*,
         gobject::GObjectExtManualGst,
@@ -328,6 +346,7 @@ pub mod prelude {
         param_spec::GstParamSpecBuilderExt,
         pipeline::GstPipelineExtManual,
         plugin_feature::PluginFeatureExtManual,
+        slice::Dump,
         tag_setter::TagSetterExtManual,
         tags::{CustomTag, Tag},
         task_pool::{TaskHandle, TaskPoolExtManual},
