@@ -1,17 +1,16 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use std::fmt;
-use std::marker::PhantomData;
-use std::mem;
-use std::ops::{Deref, DerefMut};
-use std::ptr;
-use std::slice;
+use std::{
+    fmt,
+    marker::PhantomData,
+    mem,
+    ops::{Bound, Deref, DerefMut, RangeBounds},
+    ptr, slice,
+};
 
-use glib::translate::{from_glib, from_glib_full, from_glib_none, IntoGlibPtr, ToGlibPtr};
+use glib::translate::*;
 
-use crate::AllocationParams;
-use crate::Allocator;
-use crate::MemoryFlags;
+use crate::{AllocationParams, Allocator, MemoryFlags};
 
 mini_object_wrapper!(Memory, MemoryRef, ffi::GstMemory, || {
     ffi::gst_memory_get_type()
@@ -24,7 +23,7 @@ pub struct MemoryMap<'a, T> {
 }
 
 pub struct MappedMemory<T> {
-    memory: Option<Memory>,
+    memory: Memory,
     map_info: ffi::GstMapInfo,
     phantom: PhantomData<T>,
 }
@@ -54,11 +53,7 @@ pub enum Readable {}
 pub enum Writable {}
 
 impl Memory {
-    unsafe extern "C" fn drop_box<T>(vec: glib::ffi::gpointer) {
-        let slice: Box<T> = Box::from_raw(vec as *mut T);
-        drop(slice);
-    }
-
+    #[inline]
     pub fn with_size(size: usize) -> Self {
         assert_initialized_main_thread!();
         unsafe {
@@ -70,6 +65,7 @@ impl Memory {
         }
     }
 
+    #[inline]
     pub fn with_size_and_params(size: usize, params: &AllocationParams) -> Self {
         assert_initialized_main_thread!();
         unsafe {
@@ -81,52 +77,10 @@ impl Memory {
         }
     }
 
-    pub fn from_slice<T: AsRef<[u8]> + Send + 'static>(slice: T) -> Self {
-        assert_initialized_main_thread!();
-        unsafe {
-            let b = Box::new(slice);
-            let (size, data) = {
-                let slice = (*b).as_ref();
-                (slice.len(), slice.as_ptr())
-            };
-            let user_data = Box::into_raw(b);
-            from_glib_full(ffi::gst_memory_new_wrapped(
-                ffi::GST_MEMORY_FLAG_READONLY,
-                data as glib::ffi::gpointer,
-                size,
-                0,
-                size,
-                user_data as glib::ffi::gpointer,
-                Some(Self::drop_box::<T>),
-            ))
-        }
-    }
-
-    pub fn from_mut_slice<T: AsMut<[u8]> + Send + 'static>(slice: T) -> Self {
-        assert_initialized_main_thread!();
-
-        unsafe {
-            let mut b = Box::new(slice);
-            let (size, data) = {
-                let slice = (*b).as_mut();
-                (slice.len(), slice.as_mut_ptr())
-            };
-            let user_data = Box::into_raw(b);
-            from_glib_full(ffi::gst_memory_new_wrapped(
-                0,
-                data as glib::ffi::gpointer,
-                size,
-                0,
-                size,
-                user_data as glib::ffi::gpointer,
-                Some(Self::drop_box::<T>),
-            ))
-        }
-    }
-
+    #[inline]
     pub fn into_mapped_memory_readable(self) -> Result<MappedMemory<Readable>, Self> {
         unsafe {
-            let mut map_info = mem::MaybeUninit::zeroed();
+            let mut map_info = mem::MaybeUninit::uninit();
             let res: bool = from_glib(ffi::gst_memory_map(
                 self.as_mut_ptr(),
                 map_info.as_mut_ptr(),
@@ -134,7 +88,7 @@ impl Memory {
             ));
             if res {
                 Ok(MappedMemory {
-                    memory: Some(self),
+                    memory: self,
                     map_info: map_info.assume_init(),
                     phantom: PhantomData,
                 })
@@ -144,9 +98,10 @@ impl Memory {
         }
     }
 
+    #[inline]
     pub fn into_mapped_memory_writable(self) -> Result<MappedMemory<Writable>, Self> {
         unsafe {
-            let mut map_info = mem::MaybeUninit::zeroed();
+            let mut map_info = mem::MaybeUninit::uninit();
             let res: bool = from_glib(ffi::gst_memory_map(
                 self.as_mut_ptr(),
                 map_info.as_mut_ptr(),
@@ -154,7 +109,7 @@ impl Memory {
             ));
             if res {
                 Ok(MappedMemory {
-                    memory: Some(self),
+                    memory: self,
                     map_info: map_info.assume_init(),
                     phantom: PhantomData,
                 })
@@ -167,11 +122,19 @@ impl Memory {
 
 impl MemoryRef {
     #[doc(alias = "get_allocator")]
-    pub fn allocator(&self) -> Option<Allocator> {
-        unsafe { from_glib_none(self.0.allocator) }
+    #[inline]
+    pub fn allocator(&self) -> Option<&Allocator> {
+        unsafe {
+            if self.0.allocator.is_null() {
+                None
+            } else {
+                Some(&*(&self.0.allocator as *const *mut ffi::GstAllocator as *const Allocator))
+            }
+        }
     }
 
     #[doc(alias = "get_parent")]
+    #[inline]
     pub fn parent(&self) -> Option<&MemoryRef> {
         unsafe {
             if self.0.parent.is_null() {
@@ -183,46 +146,101 @@ impl MemoryRef {
     }
 
     #[doc(alias = "get_maxsize")]
+    #[inline]
     pub fn maxsize(&self) -> usize {
         self.0.maxsize
     }
 
     #[doc(alias = "get_align")]
+    #[inline]
     pub fn align(&self) -> usize {
         self.0.align
     }
 
     #[doc(alias = "get_offset")]
+    #[inline]
     pub fn offset(&self) -> usize {
         self.0.offset
     }
 
     #[doc(alias = "get_size")]
+    #[inline]
     pub fn size(&self) -> usize {
         self.0.size
     }
 
     #[doc(alias = "get_flags")]
+    #[inline]
     pub fn flags(&self) -> MemoryFlags {
         unsafe { from_glib(self.0.mini_object.flags) }
     }
 
-    pub fn copy_part(&self, offset: isize, size: Option<usize>) -> Memory {
-        let pos_sz = match size {
-            Some(val) => val as isize,
-            None => 0,
+    fn calculate_offset_size(&self, range: impl RangeBounds<usize>) -> (isize, isize) {
+        let size = self.size();
+
+        let start_offset = match range.start_bound() {
+            Bound::Included(v) => *v,
+            Bound::Excluded(v) => v.checked_add(1).expect("Invalid start offset"),
+            Bound::Unbounded => 0,
         };
-        assert!(offset + pos_sz < (self.maxsize() as isize));
-        unsafe {
-            from_glib_full(ffi::gst_memory_copy(
-                self.as_mut_ptr(),
-                offset,
-                match size {
-                    Some(val) => val as isize,
-                    None => -1,
-                },
-            ))
-        }
+        assert!(start_offset < size, "Start offset after valid range");
+
+        let end_offset = match range.end_bound() {
+            Bound::Included(v) => v.checked_add(1).expect("Invalid end offset"),
+            Bound::Excluded(v) => *v,
+            Bound::Unbounded => size,
+        };
+        assert!(end_offset <= size, "End offset after valid range");
+
+        // Cast from usize to isize because that's literally how this works in the
+        // implementation and how the upper half of the usize range can be made use of.
+        //
+        // The implementation works exploiting wraparounds.
+        let new_offset = start_offset as isize;
+        let new_size = end_offset.saturating_sub(start_offset) as isize;
+
+        (new_offset, new_size)
+    }
+
+    fn calculate_offset_size_maxsize(&self, range: impl RangeBounds<usize>) -> (isize, isize) {
+        let maxsize = self.maxsize();
+
+        let start_offset = match range.start_bound() {
+            Bound::Included(v) => *v,
+            Bound::Excluded(v) => v.checked_add(1).expect("Invalid start offset"),
+            Bound::Unbounded => 0,
+        };
+        assert!(start_offset < maxsize, "Start offset after valid range");
+
+        let end_offset = match range.end_bound() {
+            Bound::Included(v) => v.checked_add(1).expect("Invalid end offset"),
+            Bound::Excluded(v) => *v,
+            Bound::Unbounded => maxsize,
+        };
+        assert!(end_offset <= maxsize, "End offset after valid range");
+
+        // Cast from usize to isize because that's literally how this works in the
+        // implementation and how the upper half of the usize range can be made use of.
+        //
+        // The implementation works by exploiting wraparounds.
+        let offset = self.offset();
+
+        let new_offset = start_offset.wrapping_sub(offset) as isize;
+        let new_size = end_offset.saturating_sub(start_offset) as isize;
+
+        (new_offset, new_size)
+    }
+
+    #[doc(alias = "gst_memory_copy")]
+    pub fn copy_range(&self, range: impl RangeBounds<usize>) -> Memory {
+        let (offset, size) = self.calculate_offset_size(range);
+        unsafe { from_glib_full(ffi::gst_memory_copy(self.as_mut_ptr(), offset, size)) }
+    }
+
+    #[doc(alias = "gst_memory_copy")]
+    pub fn copy_range_maxsize(&self, range: impl RangeBounds<usize>) -> Memory {
+        let (offset, size) = self.calculate_offset_size_maxsize(range);
+        unsafe { from_glib_full(ffi::gst_memory_copy(self.as_mut_ptr(), offset, size)) }
     }
 
     #[doc(alias = "gst_memory_is_span")]
@@ -252,9 +270,10 @@ impl MemoryRef {
         }
     }
 
+    #[inline]
     pub fn map_readable(&self) -> Result<MemoryMap<Readable>, glib::BoolError> {
         unsafe {
-            let mut map_info = mem::MaybeUninit::zeroed();
+            let mut map_info = mem::MaybeUninit::uninit();
             let res =
                 ffi::gst_memory_map(self.as_mut_ptr(), map_info.as_mut_ptr(), ffi::GST_MAP_READ);
             if res == glib::ffi::GTRUE {
@@ -269,9 +288,10 @@ impl MemoryRef {
         }
     }
 
+    #[inline]
     pub fn map_writable(&mut self) -> Result<MemoryMap<Writable>, glib::BoolError> {
         unsafe {
-            let mut map_info = mem::MaybeUninit::zeroed();
+            let mut map_info = mem::MaybeUninit::uninit();
             let res = ffi::gst_memory_map(
                 self.as_mut_ptr(),
                 map_info.as_mut_ptr(),
@@ -290,70 +310,89 @@ impl MemoryRef {
     }
 
     #[doc(alias = "gst_memory_share")]
-    pub fn share(&self, offset: isize, size: Option<usize>) -> Memory {
-        let pos_sz = match size {
-            Some(val) => val as isize,
-            None => 0,
-        };
-        assert!(offset + pos_sz < (self.maxsize() as isize));
-        unsafe {
-            from_glib_full(ffi::gst_memory_share(
-                self.as_ptr() as *mut _,
-                offset,
-                match size {
-                    Some(val) => val as isize,
-                    None => -1,
-                },
-            ))
-        }
+    pub fn share(&self, range: impl RangeBounds<usize>) -> Memory {
+        let (offset, size) = self.calculate_offset_size(range);
+        unsafe { from_glib_full(ffi::gst_memory_share(self.as_ptr() as *mut _, offset, size)) }
+    }
+
+    #[doc(alias = "gst_memory_share")]
+    pub fn share_maxsize(&self, range: impl RangeBounds<usize>) -> Memory {
+        let (offset, size) = self.calculate_offset_size_maxsize(range);
+        unsafe { from_glib_full(ffi::gst_memory_share(self.as_ptr() as *mut _, offset, size)) }
     }
 
     #[doc(alias = "gst_memory_resize")]
-    pub fn resize(&mut self, offset: isize, size: usize) {
-        assert!(offset + (size as isize) < (self.maxsize() as isize));
-        unsafe { ffi::gst_memory_resize(self.as_mut_ptr(), offset, size) }
+    pub fn resize(&mut self, range: impl RangeBounds<usize>) {
+        let (offset, size) = self.calculate_offset_size(range);
+        unsafe { ffi::gst_memory_resize(self.as_mut_ptr(), offset, size as usize) }
     }
 
-    pub fn dump(&self, size: Option<usize>) -> Dump {
-        Dump { memory: self, size }
+    #[doc(alias = "gst_memory_resize")]
+    pub fn resize_maxsize(&mut self, range: impl RangeBounds<usize>) {
+        let (offset, size) = self.calculate_offset_size_maxsize(range);
+        unsafe { ffi::gst_memory_resize(self.as_mut_ptr(), offset, size as usize) }
+    }
+
+    #[doc(alias = "gst_util_dump_mem")]
+    pub fn dump(&self) -> Dump {
+        Dump {
+            memory: self,
+            start: Bound::Unbounded,
+            end: Bound::Unbounded,
+        }
+    }
+
+    #[doc(alias = "gst_util_dump_mem")]
+    pub fn dump_range(&self, range: impl RangeBounds<usize>) -> Dump {
+        Dump {
+            memory: self,
+            start: range.start_bound().cloned(),
+            end: range.end_bound().cloned(),
+        }
     }
 }
 
 impl<'a, T> MemoryMap<'a, T> {
     #[doc(alias = "get_size")]
+    #[inline]
     pub fn size(&self) -> usize {
         self.map_info.size
     }
 
     #[doc(alias = "get_memory")]
+    #[inline]
     pub fn memory(&self) -> &MemoryRef {
         self.memory
     }
 
+    #[inline]
     pub fn as_slice(&self) -> &[u8] {
         if self.map_info.size == 0 {
             return &[];
         }
-        unsafe { slice::from_raw_parts(self.map_info.data as *const u8, self.map_info.size) }
+        unsafe { slice::from_raw_parts(self.map_info.data, self.map_info.size) }
     }
 }
 
 impl<'a> MemoryMap<'a, Writable> {
+    #[inline]
     pub fn as_mut_slice(&mut self) -> &mut [u8] {
         if self.map_info.size == 0 {
             return &mut [];
         }
-        unsafe { slice::from_raw_parts_mut(self.map_info.data as *mut u8, self.map_info.size) }
+        unsafe { slice::from_raw_parts_mut(self.map_info.data, self.map_info.size) }
     }
 }
 
 impl<'a, T> AsRef<[u8]> for MemoryMap<'a, T> {
+    #[inline]
     fn as_ref(&self) -> &[u8] {
         self.as_slice()
     }
 }
 
 impl<'a> AsMut<[u8]> for MemoryMap<'a, Writable> {
+    #[inline]
     fn as_mut(&mut self) -> &mut [u8] {
         self.as_mut_slice()
     }
@@ -362,12 +401,14 @@ impl<'a> AsMut<[u8]> for MemoryMap<'a, Writable> {
 impl<'a, T> Deref for MemoryMap<'a, T> {
     type Target = [u8];
 
+    #[inline]
     fn deref(&self) -> &[u8] {
         self.as_slice()
     }
 }
 
 impl<'a> DerefMut for MemoryMap<'a, Writable> {
+    #[inline]
     fn deref_mut(&mut self) -> &mut [u8] {
         self.as_mut_slice()
     }
@@ -388,6 +429,7 @@ impl<'a, T> PartialEq for MemoryMap<'a, T> {
 impl<'a, T> Eq for MemoryMap<'a, T> {}
 
 impl<'a, T> Drop for MemoryMap<'a, T> {
+    #[inline]
     fn drop(&mut self) {
         unsafe {
             ffi::gst_memory_unmap(self.memory.as_mut_ptr(), &mut self.map_info);
@@ -399,27 +441,32 @@ unsafe impl<'a, T> Send for MemoryMap<'a, T> {}
 unsafe impl<'a, T> Sync for MemoryMap<'a, T> {}
 
 impl<T> MappedMemory<T> {
+    #[inline]
     pub fn as_slice(&self) -> &[u8] {
         if self.map_info.size == 0 {
             return &[];
         }
-        unsafe { slice::from_raw_parts(self.map_info.data as *const u8, self.map_info.size) }
+        unsafe { slice::from_raw_parts(self.map_info.data, self.map_info.size) }
     }
 
     #[doc(alias = "get_size")]
+    #[inline]
     pub fn size(&self) -> usize {
         self.map_info.size
     }
 
     #[doc(alias = "get_memory")]
+    #[inline]
     pub fn memory(&self) -> &MemoryRef {
-        self.memory.as_ref().unwrap().as_ref()
+        self.memory.as_ref()
     }
 
-    pub fn into_memory(mut self) -> Memory {
-        let memory = self.memory.take().unwrap();
+    #[inline]
+    pub fn into_memory(self) -> Memory {
+        let mut s = mem::ManuallyDrop::new(self);
+        let memory = unsafe { ptr::read(&s.memory) };
         unsafe {
-            ffi::gst_memory_unmap(memory.as_mut_ptr(), &mut self.map_info);
+            ffi::gst_memory_unmap(memory.as_mut_ptr(), &mut s.map_info);
         }
 
         memory
@@ -427,21 +474,24 @@ impl<T> MappedMemory<T> {
 }
 
 impl MappedMemory<Writable> {
+    #[inline]
     pub fn as_mut_slice(&mut self) -> &mut [u8] {
         if self.map_info.size == 0 {
             return &mut [];
         }
-        unsafe { slice::from_raw_parts_mut(self.map_info.data as *mut u8, self.map_info.size) }
+        unsafe { slice::from_raw_parts_mut(self.map_info.data, self.map_info.size) }
     }
 }
 
 impl<T> AsRef<[u8]> for MappedMemory<T> {
+    #[inline]
     fn as_ref(&self) -> &[u8] {
         self.as_slice()
     }
 }
 
 impl AsMut<[u8]> for MappedMemory<Writable> {
+    #[inline]
     fn as_mut(&mut self) -> &mut [u8] {
         self.as_mut_slice()
     }
@@ -450,23 +500,24 @@ impl AsMut<[u8]> for MappedMemory<Writable> {
 impl<T> Deref for MappedMemory<T> {
     type Target = [u8];
 
+    #[inline]
     fn deref(&self) -> &[u8] {
         self.as_slice()
     }
 }
 
 impl DerefMut for MappedMemory<Writable> {
+    #[inline]
     fn deref_mut(&mut self) -> &mut [u8] {
         self.as_mut_slice()
     }
 }
 
 impl<T> Drop for MappedMemory<T> {
+    #[inline]
     fn drop(&mut self) {
-        if let Some(ref memory) = self.memory {
-            unsafe {
-                ffi::gst_memory_unmap(memory.as_mut_ptr(), &mut self.map_info);
-            }
+        unsafe {
+            ffi::gst_memory_unmap(self.memory.as_mut_ptr(), &mut self.map_info);
         }
     }
 }
@@ -490,22 +541,25 @@ unsafe impl<T> Sync for MappedMemory<T> {}
 
 pub struct Dump<'a> {
     memory: &'a MemoryRef,
-    size: Option<usize>,
+    start: Bound<usize>,
+    end: Bound<usize>,
 }
 
 impl<'a> Dump<'a> {
     fn fmt(&self, f: &mut fmt::Formatter, debug: bool) -> fmt::Result {
-        use pretty_hex::*;
-
         let map = self.memory.map_readable().expect("Failed to map memory");
         let data = map.as_slice();
-        let size = self.size.unwrap_or_else(|| self.memory.size());
-        let data = &data[0..size];
+
+        let dump = crate::slice::Dump {
+            data,
+            start: self.start,
+            end: self.end,
+        };
 
         if debug {
-            write!(f, "{:?}", data.hex_dump())
+            <crate::slice::Dump as fmt::Debug>::fmt(&dump, f)
         } else {
-            write!(f, "{}", data.hex_dump())
+            <crate::slice::Dump as fmt::Display>::fmt(&dump, f)
         }
     }
 }
@@ -541,7 +595,7 @@ pub struct MemoryTypeValueTypeChecker<M>(PhantomData<M>);
 
 unsafe impl<M> glib::value::ValueTypeChecker for MemoryTypeValueTypeChecker<M>
 where
-    M: MemoryType + glib::StaticType,
+    M: MemoryType + glib::prelude::StaticType,
     <M as crate::prelude::IsMiniObject>::RefType: AsRef<MemoryRef> + AsMut<MemoryRef>,
 {
     type Error = glib::value::ValueTypeMismatchOrNoneError<MemoryTypeMismatchError>;
@@ -572,24 +626,28 @@ where
 }
 
 impl AsRef<MemoryRef> for MemoryRef {
+    #[inline]
     fn as_ref(&self) -> &MemoryRef {
         self
     }
 }
 
 impl AsMut<MemoryRef> for MemoryRef {
+    #[inline]
     fn as_mut(&mut self) -> &mut MemoryRef {
         self
     }
 }
 
 impl AsRef<Memory> for Memory {
+    #[inline]
     fn as_ref(&self) -> &Memory {
         self
     }
 }
 
 unsafe impl MemoryType for Memory {
+    #[inline]
     fn check_memory_type(_mem: &MemoryRef) -> bool {
         skip_assert_initialized!();
         true
@@ -597,6 +655,7 @@ unsafe impl MemoryType for Memory {
 }
 
 impl Memory {
+    #[inline]
     pub fn downcast_memory<M: MemoryType>(self) -> Result<M, Self>
     where
         <M as crate::prelude::IsMiniObject>::RefType: AsRef<MemoryRef> + AsMut<MemoryRef>,
@@ -610,6 +669,7 @@ impl Memory {
 }
 
 impl MemoryRef {
+    #[inline]
     pub fn is_memory_type<M: MemoryType>(&self) -> bool
     where
         <M as crate::prelude::IsMiniObject>::RefType: AsRef<MemoryRef> + AsMut<MemoryRef>,
@@ -617,6 +677,7 @@ impl MemoryRef {
         M::check_memory_type(self)
     }
 
+    #[inline]
     pub fn downcast_memory_ref<M: MemoryType>(&self) -> Option<&M::RefType>
     where
         <M as crate::prelude::IsMiniObject>::RefType: AsRef<MemoryRef> + AsMut<MemoryRef>,
@@ -628,6 +689,7 @@ impl MemoryRef {
         }
     }
 
+    #[inline]
     pub fn downcast_memory_mut<M: MemoryType>(&mut self) -> Option<&mut M::RefType>
     where
         <M as crate::prelude::IsMiniObject>::RefType: AsRef<MemoryRef> + AsMut<MemoryRef>,
@@ -646,6 +708,7 @@ macro_rules! memory_object_wrapper {
         $crate::mini_object_wrapper!($name, $ref_name, $ffi_name);
 
         unsafe impl $crate::memory::MemoryType for $name {
+            #[inline]
             fn check_memory_type(mem: &$crate::MemoryRef) -> bool {
                 skip_assert_initialized!();
                 $mem_type_check(mem)
@@ -653,6 +716,7 @@ macro_rules! memory_object_wrapper {
         }
 
         impl $name {
+            #[inline]
             pub fn downcast_memory<M: $crate::memory::MemoryType>(self) -> Result<M, Self>
             where
                 <M as $crate::miniobject::IsMiniObject>::RefType: AsRef<$crate::MemoryRef>
@@ -671,6 +735,7 @@ macro_rules! memory_object_wrapper {
                 }
             }
 
+            #[inline]
             pub fn upcast_memory<M>(self) -> M
             where
                 M: $crate::memory::MemoryType
@@ -690,6 +755,7 @@ macro_rules! memory_object_wrapper {
         }
 
         impl $ref_name {
+            #[inline]
             pub fn upcast_memory_ref<M>(&self) -> &M::RefType
             where
                 M: $crate::memory::MemoryType,
@@ -700,6 +766,7 @@ macro_rules! memory_object_wrapper {
                 self.as_ref()
             }
 
+            #[inline]
             pub fn upcast_memory_mut<M>(&mut self) -> &mut M::RefType
             where
                 M: $crate::memory::MemoryType,
@@ -714,42 +781,49 @@ macro_rules! memory_object_wrapper {
         impl std::ops::Deref for $ref_name {
             type Target = $parent_memory_ref_type;
 
+            #[inline]
             fn deref(&self) -> &Self::Target {
                 unsafe { &*(self as *const _ as *const Self::Target) }
             }
         }
 
         impl std::ops::DerefMut for $ref_name {
+            #[inline]
             fn deref_mut(&mut self) -> &mut Self::Target {
                 unsafe { &mut *(self as *mut _ as *mut Self::Target) }
             }
         }
 
         impl AsRef<$parent_memory_type> for $name {
+            #[inline]
             fn as_ref(&self) -> &$parent_memory_type {
                 unsafe { &*(self as *const _ as *const $parent_memory_type) }
             }
         }
 
         impl AsRef<$parent_memory_ref_type> for $ref_name {
+            #[inline]
             fn as_ref(&self) -> &$parent_memory_ref_type {
                 self
             }
         }
 
         impl AsMut<$parent_memory_ref_type> for $ref_name {
+            #[inline]
             fn as_mut(&mut self) -> &mut $parent_memory_ref_type {
                 &mut *self
             }
         }
 
         impl $crate::glib::types::StaticType for $name {
+            #[inline]
             fn static_type() -> glib::types::Type {
                 $ref_name::static_type()
             }
         }
 
         impl $crate::glib::types::StaticType for $ref_name {
+            #[inline]
             fn static_type() -> $crate::glib::types::Type {
                 unsafe { $crate::glib::translate::from_glib($crate::ffi::gst_memory_get_type()) }
             }
@@ -782,7 +856,7 @@ macro_rules! memory_object_wrapper {
                 let value = &*(value as *const $crate::glib::Value as *const $crate::glib::gobject_ffi::GValue);
                 let ptr = &value.data[0].v_pointer as *const $crate::glib::ffi::gpointer
                     as *const *const $ffi_name;
-                assert!(!(*ptr).is_null());
+                debug_assert!(!(*ptr).is_null());
                 &*(ptr as *const $name)
             }
         }
@@ -801,7 +875,7 @@ macro_rules! memory_object_wrapper {
             }
 
             fn value_type(&self) -> glib::Type {
-                <Self as glib::StaticType>::static_type()
+                <Self as $crate::glib::prelude::StaticType>::static_type()
             }
         }
 
@@ -820,6 +894,20 @@ macro_rules! memory_object_wrapper {
             }
         }
 
+        impl From<$name> for $crate::glib::Value {
+            fn from(v: $name) -> $crate::glib::Value {
+                skip_assert_initialized!();
+                let mut value = $crate::glib::Value::for_value_type::<$name>();
+                unsafe {
+                    $crate::glib::gobject_ffi::g_value_take_boxed(
+                        $crate::glib::translate::ToGlibPtrMut::to_glib_none_mut(&mut value).0,
+                        $crate::glib::translate::IntoGlibPtr::<*mut $ffi_name>::into_glib_ptr(v) as *mut _,
+                    )
+                }
+                value
+            }
+        }
+
         unsafe impl<'a> $crate::glib::value::FromValue<'a> for &'a $ref_name {
             type Checker = $crate::memory::MemoryTypeValueTypeChecker<$name>;
 
@@ -838,18 +926,21 @@ macro_rules! memory_object_wrapper {
 
         $(
             impl AsRef<$parent_parent_memory_type> for $name {
+                #[inline]
                 fn as_ref(&self) -> &$parent_parent_memory_type {
                     unsafe { &*(self as *const _ as *const $parent_parent_memory_type) }
                 }
             }
 
             impl AsRef<$parent_parent_memory_ref_type> for $ref_name {
+                #[inline]
                 fn as_ref(&self) -> &$parent_parent_memory_ref_type {
                     self
                 }
             }
 
             impl AsMut<$parent_parent_memory_ref_type> for $ref_name {
+                #[inline]
                 fn as_mut(&mut self) -> &mut $parent_parent_memory_ref_type {
                     &mut *self
                 }
@@ -861,17 +952,100 @@ macro_rules! memory_object_wrapper {
 #[cfg(test)]
 mod tests {
     #[test]
-    fn test_dump() {
+    fn test_map() {
         crate::init().unwrap();
 
         let mem = crate::Memory::from_slice(vec![1, 2, 3, 4]);
-        println!("{}", mem.dump(Some(mem.size())));
+        let map = mem.map_readable().unwrap();
+        assert_eq!(map.as_slice(), &[1, 2, 3, 4]);
+        drop(map);
+
+        let mem = mem.into_mapped_memory_readable().unwrap();
+        assert_eq!(mem.as_slice(), &[1, 2, 3, 4]);
+
+        let mem = mem.into_memory();
+        let map = mem.map_readable().unwrap();
+        assert_eq!(map.as_slice(), &[1, 2, 3, 4]);
+    }
+
+    #[test]
+    fn test_share() {
+        crate::init().unwrap();
 
         let mem = crate::Memory::from_slice(vec![1, 2, 3, 4]);
-        println!("{:?}", mem.dump(Some(2)));
+        let sub = mem.share(1..=2); // [2, 3]
+        let sub_sub1 = sub.share(1..=1); // [3]
+        let sub_sub2 = sub.share_maxsize(0..4); // [1, 2, 3, 4]
 
-        let mem = crate::Memory::from_slice(vec![0; 64]);
-        dbg!(mem.dump(None));
+        let map = mem.map_readable().unwrap();
+        assert_eq!(map.as_slice(), &[1, 2, 3, 4]);
+        drop(map);
+
+        let map = sub.map_readable().unwrap();
+        assert_eq!(map.as_slice(), &[2, 3]);
+        drop(map);
+
+        let map = sub_sub1.map_readable().unwrap();
+        assert_eq!(map.as_slice(), &[3]);
+        drop(map);
+
+        let map = sub_sub2.map_readable().unwrap();
+        assert_eq!(map.as_slice(), &[1, 2, 3, 4]);
+        drop(map);
+    }
+
+    #[test]
+    fn test_dump() {
+        use std::fmt::Write;
+
+        crate::init().unwrap();
+
+        let mut s = String::new();
+        let mem = crate::Memory::from_slice(vec![1, 2, 3, 4]);
+        write!(&mut s, "{:?}", mem.dump()).unwrap();
+        assert_eq!(
+            s,
+            "0000:  01 02 03 04                                       ...."
+        );
+        s.clear();
+        write!(&mut s, "{}", mem.dump()).unwrap();
+        assert_eq!(s, "01 02 03 04");
+        s.clear();
+
+        let mem = crate::Memory::from_slice(vec![1, 2, 3, 4]);
+        write!(&mut s, "{:?}", mem.dump_range(..)).unwrap();
+        assert_eq!(
+            s,
+            "0000:  01 02 03 04                                       ...."
+        );
+        s.clear();
+        write!(&mut s, "{:?}", mem.dump_range(..2)).unwrap();
+        assert_eq!(
+            s,
+            "0000:  01 02                                             .."
+        );
+        s.clear();
+        write!(&mut s, "{:?}", mem.dump_range(2..=3)).unwrap();
+        assert_eq!(
+            s,
+            "0002:  03 04                                             .."
+        );
+        s.clear();
+        write!(&mut s, "{:?}", mem.dump_range(..100)).unwrap();
+        assert_eq!(s, "<end out of range>",);
+        s.clear();
+        write!(&mut s, "{:?}", mem.dump_range(90..100)).unwrap();
+        assert_eq!(s, "<start out of range>",);
+        s.clear();
+
+        let mem = crate::Memory::from_slice(vec![0; 19]);
+        write!(&mut s, "{:?}", mem.dump()).unwrap();
+        assert_eq!(
+            s,
+            "0000:  00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00   ................\n\
+             0010:  00 00 00                                          ..."
+        );
+        s.clear();
     }
 
     #[test]
@@ -886,6 +1060,6 @@ mod tests {
         let mem = crate::Memory::from_slice(vec![1, 2, 3, 4]);
         let v = mem.to_value();
         assert!(matches!(v.get::<Option<crate::Memory>>(), Ok(Some(_))));
-        assert!(matches!(v.get::<crate::Memory>(), Ok(_)));
+        assert!(v.get::<crate::Memory>().is_ok());
     }
 }
