@@ -1,9 +1,13 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-#[cfg(any(feature = "v1_20", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
+#[cfg(feature = "v1_20")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_20")))]
 use std::ptr;
-use std::{fmt, marker::PhantomData, ops};
+use std::{
+    fmt,
+    marker::PhantomData,
+    ops::{self, RangeBounds},
+};
 
 use glib::translate::*;
 
@@ -17,14 +21,6 @@ pub unsafe trait MetaAPI: Sync + Send + Sized {
 }
 
 pub trait MetaAPIExt: MetaAPI {
-    unsafe fn from_ptr(buffer: &BufferRef, ptr: *const Self::GstType) -> MetaRef<Self>;
-    unsafe fn from_mut_ptr<T>(
-        buffer: &mut BufferRef,
-        ptr: *mut Self::GstType,
-    ) -> MetaRefMut<Self, T>;
-}
-
-impl<A: MetaAPI> MetaAPIExt for A {
     #[inline]
     unsafe fn from_ptr(buffer: &BufferRef, ptr: *const Self::GstType) -> MetaRef<Self> {
         debug_assert!(!ptr.is_null());
@@ -66,8 +62,10 @@ impl<A: MetaAPI> MetaAPIExt for A {
     }
 }
 
-#[cfg(any(feature = "v1_16", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+impl<A: MetaAPI> MetaAPIExt for A {}
+
+#[cfg(feature = "v1_16")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
 #[derive(Debug, Clone, Copy, PartialOrd, Ord, PartialEq, Eq)]
 pub struct MetaSeqnum(u64);
 
@@ -120,6 +118,22 @@ impl<'a, T> AsRef<MetaRef<'a, T>> for MetaRef<'a, T> {
     }
 }
 
+impl<'a, T> AsRef<T> for MetaRef<'a, T> {
+    #[inline]
+    fn as_ref(&self) -> &T {
+        self.meta
+    }
+}
+
+impl<'a, T: 'a> Clone for MetaRef<'a, T> {
+    fn clone(&self) -> Self {
+        MetaRef {
+            meta: self.meta,
+            buffer: self.buffer,
+        }
+    }
+}
+
 impl<'a, T, U> ops::Deref for MetaRefMut<'a, T, U> {
     type Target = T;
 
@@ -143,6 +157,13 @@ impl<'a, T, U> AsRef<MetaRef<'a, T>> for MetaRefMut<'a, T, U> {
     }
 }
 
+impl<'a, T, U> AsMut<T> for MetaRefMut<'a, T, U> {
+    #[inline]
+    fn as_mut(&mut self) -> &mut T {
+        self.meta
+    }
+}
+
 impl<'a, T> MetaRef<'a, T> {
     #[doc(alias = "get_api")]
     #[inline]
@@ -171,8 +192,8 @@ impl<'a, T> MetaRef<'a, T> {
         }
     }
 
-    #[cfg(any(feature = "v1_16", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+    #[cfg(feature = "v1_16")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
     #[doc(alias = "get_seqnum")]
     #[doc(alias = "gst_meta_get_seqnum")]
     #[inline]
@@ -183,6 +204,72 @@ impl<'a, T> MetaRef<'a, T> {
         }
     }
 
+    #[inline]
+    #[doc(alias = "gst_meta_api_type_has_tag")]
+    pub fn has_tag(&self, tag: glib::Quark) -> bool {
+        unsafe {
+            from_glib(ffi::gst_meta_api_type_has_tag(
+                self.api().into_glib(),
+                tag.into_glib(),
+            ))
+        }
+    }
+
+    #[inline]
+    #[doc(alias = "gst_meta_api_type_get_tags")]
+    pub fn tags<'b>(&self) -> &'b [glib::GStringPtr] {
+        unsafe {
+            glib::StrV::from_glib_borrow(ffi::gst_meta_api_type_get_tags(self.api().into_glib()))
+        }
+    }
+
+    #[inline]
+    pub fn upcast_ref(&self) -> &MetaRef<'a, Meta> {
+        unsafe { &*(self as *const MetaRef<'a, T> as *const MetaRef<'a, Meta>) }
+    }
+
+    pub fn copy(
+        &self,
+        buffer: &mut BufferRef,
+        region: bool,
+        range: impl RangeBounds<usize>,
+    ) -> Result<(), glib::BoolError>
+    where
+        T: MetaAPI,
+    {
+        static TRANSFORM_COPY: std::sync::OnceLock<glib::Quark> = std::sync::OnceLock::new();
+
+        let transform_copy = TRANSFORM_COPY.get_or_init(|| glib::Quark::from_str("gst-copy"));
+
+        let (offset, size) = self.buffer.byte_range_into_offset_len(range)?;
+
+        unsafe {
+            let info = *(*self.upcast_ref().as_ptr()).info;
+            let Some(transform_func) = info.transform_func else {
+                return Err(glib::bool_error!(
+                    "Can't copy meta without transform function"
+                ));
+            };
+
+            let mut copy_data = ffi::GstMetaTransformCopy {
+                region: region.into_glib(),
+                offset,
+                size,
+            };
+
+            glib::result_from_gboolean!(
+                transform_func(
+                    buffer.as_mut_ptr(),
+                    mut_override(self.upcast_ref().as_ptr()),
+                    mut_override(self.buffer.as_ptr()),
+                    transform_copy.into_glib(),
+                    &mut copy_data as *mut _ as *mut _,
+                ),
+                "Failed to copy meta"
+            )
+        }
+    }
+
     #[inline]
     pub fn as_ptr(&self) -> *const T::GstType
     where
@@ -190,6 +277,129 @@ impl<'a, T> MetaRef<'a, T> {
     {
         self.meta as *const _ as *const <T as MetaAPI>::GstType
     }
+
+    #[cfg(feature = "v1_24")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_24")))]
+    #[doc(alias = "gst_meta_serialize")]
+    pub fn serialize<B: ByteArrayInterface + ?Sized>(
+        &self,
+        writer: &mut B,
+    ) -> Result<usize, glib::BoolError> {
+        unsafe {
+            #[repr(C)]
+            struct Writer<'a, B: ?Sized> {
+                iface_: ffi::GstByteArrayInterface,
+                writer: &'a mut B,
+            }
+
+            unsafe extern "C" fn resize<B: ByteArrayInterface + ?Sized>(
+                iface_: *mut ffi::GstByteArrayInterface,
+                size: usize,
+            ) -> glib::ffi::gboolean {
+                let iface_ = &mut *(iface_ as *mut Writer<B>);
+
+                match iface_.writer.resize(size) {
+                    Some(new_data) => {
+                        iface_.iface_.data = new_data.as_mut_ptr();
+                        iface_.iface_.len = size;
+
+                        glib::ffi::GTRUE
+                    }
+                    None => glib::ffi::GFALSE,
+                }
+            }
+
+            let initial_len = writer.initial_len();
+
+            let mut iface_ = Writer {
+                iface_: ffi::GstByteArrayInterface {
+                    data: writer.as_mut().as_mut_ptr(),
+                    len: initial_len,
+                    resize: Some(resize::<B>),
+                    _gst_reserved: [ptr::null_mut(); 4],
+                },
+                writer: &mut *writer,
+            };
+
+            let res = bool::from_glib(ffi::gst_meta_serialize(
+                self.meta as *const T as *const ffi::GstMeta,
+                &mut iface_.iface_,
+            ));
+
+            if !res {
+                return Err(glib::bool_error!("Failed to serialize meta"));
+            }
+
+            assert!(iface_.iface_.len >= initial_len);
+
+            Ok(iface_.iface_.len - initial_len)
+        }
+    }
+}
+
+#[cfg(feature = "v1_24")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_24")))]
+pub trait ByteArrayInterface: AsMut<[u8]> {
+    fn initial_len(&self) -> usize;
+    fn resize(&mut self, size: usize) -> Option<&mut [u8]>;
+}
+
+#[cfg(feature = "v1_24")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_24")))]
+impl ByteArrayInterface for Vec<u8> {
+    fn initial_len(&self) -> usize {
+        self.len()
+    }
+
+    fn resize(&mut self, size: usize) -> Option<&mut [u8]> {
+        self.resize(size, 0);
+        Some(&mut self[0..size])
+    }
+}
+
+#[cfg(feature = "v1_24")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_24")))]
+impl<A: smallvec::Array<Item = u8>> ByteArrayInterface for smallvec::SmallVec<A> {
+    fn initial_len(&self) -> usize {
+        self.len()
+    }
+
+    fn resize(&mut self, size: usize) -> Option<&mut [u8]> {
+        self.resize(size, 0);
+        Some(&mut self[0..size])
+    }
+}
+
+#[cfg(feature = "v1_24")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_24")))]
+impl ByteArrayInterface for &mut [u8] {
+    fn initial_len(&self) -> usize {
+        0
+    }
+
+    fn resize(&mut self, size: usize) -> Option<&mut [u8]> {
+        if self.len() < size {
+            return None;
+        }
+
+        Some(&mut self[0..size])
+    }
+}
+
+#[cfg(feature = "v1_24")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_24")))]
+impl<const N: usize> ByteArrayInterface for [u8; N] {
+    fn initial_len(&self) -> usize {
+        0
+    }
+
+    fn resize(&mut self, size: usize) -> Option<&mut [u8]> {
+        if N < size {
+            return None;
+        }
+
+        Some(&mut self[0..size])
+    }
 }
 
 impl<'a> MetaRef<'a, Meta> {
@@ -205,8 +415,8 @@ impl<'a> MetaRef<'a, Meta> {
         }
     }
 
-    #[cfg(any(feature = "v1_20", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
+    #[cfg(feature = "v1_20")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_20")))]
     #[inline]
     pub fn try_as_custom_meta(&self) -> Option<&MetaRef<'a, CustomMeta>> {
         unsafe {
@@ -223,42 +433,70 @@ impl<'a, T, U> MetaRefMut<'a, T, U> {
     #[doc(alias = "get_api")]
     #[inline]
     pub fn api(&self) -> glib::Type {
-        unsafe {
-            let meta = self.meta as *const _ as *const ffi::GstMeta;
-            let info = (*meta).info;
-            glib::Type::from_glib((*info).api)
-        }
+        self.as_meta_ref().api()
     }
 
     #[inline]
     pub fn flags(&self) -> crate::MetaFlags {
-        unsafe {
-            let meta = self.meta as *const _ as *const ffi::GstMeta;
-            from_glib((*meta).flags)
-        }
+        self.as_meta_ref().flags()
     }
 
     #[inline]
     pub fn type_(&self) -> glib::Type {
-        unsafe {
-            let meta = self.meta as *const _ as *const ffi::GstMeta;
-            let info = (*meta).info;
-            glib::Type::from_glib((*info).type_)
-        }
+        self.as_meta_ref().type_()
     }
 
-    #[cfg(any(feature = "v1_16", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+    #[cfg(feature = "v1_16")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
     #[doc(alias = "get_seqnum")]
     #[doc(alias = "gst_meta_get_seqnum")]
     #[inline]
-    pub fn seqnum(&self) -> u64 {
-        unsafe {
-            let meta = self.meta as *const _ as *const ffi::GstMeta;
-            ffi::gst_meta_get_seqnum(meta)
+    pub fn seqnum(&self) -> MetaSeqnum {
+        self.as_meta_ref().seqnum()
+    }
+
+    #[inline]
+    #[doc(alias = "gst_meta_api_type_has_tag")]
+    pub fn has_tag(&self, tag: glib::Quark) -> bool {
+        self.as_meta_ref().has_tag(tag)
+    }
+
+    #[inline]
+    #[doc(alias = "gst_meta_api_type_get_tags")]
+    pub fn tags<'b>(&self) -> &'b [glib::GStringPtr] {
+        self.as_meta_ref().tags()
+    }
+
+    #[inline]
+    pub fn upcast_ref(&self) -> &MetaRef<'a, Meta> {
+        unsafe { &*(self as *const MetaRefMut<'a, T, U> as *const MetaRef<'a, Meta>) }
+    }
+
+    #[inline]
+    pub fn upcast_mut(&mut self) -> &mut MetaRefMut<'a, Meta, U> {
+        unsafe { &mut *(self as *mut MetaRefMut<'a, T, U> as *mut MetaRefMut<'a, Meta, U>) }
+    }
+
+    #[inline]
+    pub fn as_meta_ref(&self) -> MetaRef<T> {
+        MetaRef {
+            meta: self.meta,
+            buffer: self.buffer,
         }
     }
 
+    pub fn copy(
+        &self,
+        buffer: &mut BufferRef,
+        region: bool,
+        range: impl RangeBounds<usize>,
+    ) -> Result<(), glib::BoolError>
+    where
+        T: MetaAPI,
+    {
+        self.as_meta_ref().copy(buffer, region, range)
+    }
+
     #[inline]
     pub fn as_ptr(&self) -> *const T::GstType
     where
@@ -274,6 +512,34 @@ impl<'a, T, U> MetaRefMut<'a, T, U> {
     {
         self.meta as *mut _ as *mut <T as MetaAPI>::GstType
     }
+
+    #[cfg(feature = "v1_24")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_24")))]
+    #[doc(alias = "gst_meta_serialize")]
+    pub fn serialize<B: ByteArrayInterface + ?Sized>(
+        &self,
+        writer: &mut B,
+    ) -> Result<usize, glib::BoolError> {
+        self.as_meta_ref().serialize(writer)
+    }
+
+    #[cfg(feature = "v1_24")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_24")))]
+    pub fn clear(&mut self) -> Result<(), glib::BoolError>
+    where
+        T: MetaAPI,
+    {
+        unsafe {
+            let info = *(*self.upcast_ref().as_ptr()).info;
+
+            if let Some(clear_func) = info.clear_func {
+                clear_func(self.buffer.as_mut_ptr(), self.upcast_mut().as_mut_ptr());
+                Ok(())
+            } else {
+                Err(glib::bool_error!("Failed to clear meta"))
+            }
+        }
+    }
 }
 
 impl<'a, T> MetaRefMut<'a, T, Standalone> {
@@ -322,8 +588,8 @@ impl<'a, U> MetaRefMut<'a, Meta, U> {
         }
     }
 
-    #[cfg(any(feature = "v1_20", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
+    #[cfg(feature = "v1_20")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_20")))]
     #[inline]
     pub fn try_as_custom_meta(&self) -> Option<&MetaRefMut<'a, CustomMeta, U>> {
         unsafe {
@@ -335,8 +601,8 @@ impl<'a, U> MetaRefMut<'a, Meta, U> {
         }
     }
 
-    #[cfg(any(feature = "v1_20", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
+    #[cfg(feature = "v1_20")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_20")))]
     #[inline]
     pub fn try_as_mut_custom_meta(&mut self) -> Option<&mut MetaRefMut<'a, CustomMeta, U>> {
         unsafe {
@@ -356,19 +622,6 @@ pub struct Meta(ffi::GstMeta);
 unsafe impl Send for Meta {}
 unsafe impl Sync for Meta {}
 
-impl Meta {
-    #[doc(alias = "get_api")]
-    #[inline]
-    fn api(&self) -> glib::Type {
-        unsafe { glib::Type::from_glib((*self.0.info).api) }
-    }
-
-    #[inline]
-    pub fn flags(&self) -> crate::MetaFlags {
-        unsafe { from_glib(self.0.flags) }
-    }
-}
-
 unsafe impl MetaAPI for Meta {
     type GstType = ffi::GstMeta;
 
@@ -380,7 +633,53 @@ unsafe impl MetaAPI for Meta {
 
 impl fmt::Debug for Meta {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        f.debug_struct("Meta").field("api", &self.api()).finish()
+        f.debug_struct("Meta")
+            .field("api", &unsafe { glib::Type::from_glib((*self.0.info).api) })
+            .field("type", &unsafe {
+                glib::Type::from_glib((*self.0.info).type_)
+            })
+            .field("flags", &unsafe {
+                crate::MetaFlags::from_glib(self.0.flags)
+            })
+            .finish()
+    }
+}
+
+impl Meta {
+    #[cfg(feature = "v1_24")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_24")))]
+    #[doc(alias = "gst_meta_deserialize")]
+    pub fn deserialize<'a>(
+        buffer: &'a mut BufferRef,
+        data: &[u8],
+        consumed: &mut usize,
+    ) -> Result<MetaRefMut<'a, Self, Standalone>, glib::BoolError> {
+        skip_assert_initialized!();
+
+        unsafe {
+            use std::mem;
+
+            let mut consumed_u32 = mem::MaybeUninit::uninit();
+
+            let res = ffi::gst_meta_deserialize(
+                buffer.as_mut_ptr(),
+                data.as_ptr(),
+                data.len(),
+                consumed_u32.as_mut_ptr(),
+            );
+
+            *consumed = consumed_u32.assume_init() as usize;
+
+            if res.is_null() {
+                return Err(glib::bool_error!("Failed to deserialize meta"));
+            }
+
+            Ok(MetaRefMut {
+                meta: &mut *(res as *mut Self),
+                buffer,
+                mode: PhantomData,
+            })
+        }
     }
 }
 
@@ -520,9 +819,9 @@ impl ReferenceTimestampMeta {
         unsafe { CapsRef::from_ptr(self.0.reference) }
     }
 
-    #[doc(alias = "get_parent_owned")]
+    #[doc(alias = "get_reference_owned")]
     #[inline]
-    pub fn parent_owned(&self) -> Caps {
+    pub fn reference_owned(&self) -> Caps {
         unsafe { from_glib_none(self.0.reference) }
     }
 
@@ -561,21 +860,21 @@ impl fmt::Debug for ReferenceTimestampMeta {
     }
 }
 
-#[cfg(any(feature = "v1_20", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
+#[cfg(feature = "v1_20")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_20")))]
 #[repr(transparent)]
 #[doc(alias = "GstCustomMeta")]
 pub struct CustomMeta(ffi::GstCustomMeta);
 
-#[cfg(any(feature = "v1_20", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
+#[cfg(feature = "v1_20")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_20")))]
 unsafe impl Send for CustomMeta {}
-#[cfg(any(feature = "v1_20", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
+#[cfg(feature = "v1_20")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_20")))]
 unsafe impl Sync for CustomMeta {}
 
-#[cfg(any(feature = "v1_20", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
+#[cfg(feature = "v1_20")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_20")))]
 impl CustomMeta {
     #[doc(alias = "gst_meta_register_custom")]
     pub fn register(name: &str, tags: &[&str]) {
@@ -638,6 +937,25 @@ impl CustomMeta {
         }
     }
 
+    #[doc(alias = "gst_meta_register_simple")]
+    pub fn register_simple(name: &str) {
+        assert_initialized_main_thread!();
+        unsafe {
+            ffi::gst_meta_register_custom(
+                name.to_glib_none().0,
+                ptr::null_mut(),
+                None,
+                ptr::null_mut(),
+                None,
+            );
+        }
+    }
+
+    pub fn is_registered(name: &str) -> bool {
+        assert_initialized_main_thread!();
+        unsafe { name.run_with_gstr(|name| !ffi::gst_meta_get_info(name.as_ptr()).is_null()) }
+    }
+
     #[doc(alias = "gst_buffer_add_custom_meta")]
     pub fn add<'a>(
         buffer: &'a mut BufferRef,
@@ -768,6 +1086,11 @@ mod tests {
                 .collect::<Vec<_>>();
             assert_eq!(metas.len(), 1);
             assert_eq!(metas[0].parent().as_ptr(), parent.as_ptr());
+            assert!(!metas[0].has_tag(glib::Quark::from_str("video")));
+            assert!(metas[0].has_tag(glib::Quark::from_str("memory-reference")));
+            assert_eq!(metas[0].tags().len(), 1);
+
+            assert_eq!(metas[0].tags(), metas[0].upcast_ref().tags());
         }
 
         {
@@ -802,4 +1125,80 @@ mod tests {
 
         assert!(buffer.meta::<ParentBufferMeta>().is_none());
     }
+
+    #[test]
+    fn test_copy_reference_timestamp_meta() {
+        crate::init().unwrap();
+
+        let caps = crate::Caps::new_empty_simple("timestamp/x-ntp");
+        let mut buffer = crate::Buffer::new();
+        {
+            ReferenceTimestampMeta::add(
+                buffer.get_mut().unwrap(),
+                &caps,
+                crate::ClockTime::from_seconds(1),
+                crate::ClockTime::NONE,
+            );
+        }
+
+        let mut buffer_dest = crate::Buffer::new();
+        {
+            let meta = buffer.meta::<ReferenceTimestampMeta>().unwrap();
+            let buffer_dest = buffer_dest.get_mut().unwrap();
+            meta.copy(buffer_dest, false, ..).unwrap();
+        }
+
+        let meta = buffer_dest.meta::<ReferenceTimestampMeta>().unwrap();
+        assert_eq!(meta.reference(), &caps);
+        assert_eq!(meta.timestamp(), crate::ClockTime::from_seconds(1));
+        assert_eq!(meta.duration(), crate::ClockTime::NONE);
+    }
+
+    #[cfg(feature = "v1_24")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_24")))]
+    #[test]
+    fn test_meta_serialize() {
+        use smallvec::SmallVec;
+
+        crate::init().unwrap();
+
+        let caps = crate::Caps::new_empty_simple("timestamp/x-ntp");
+        let mut buffer = crate::Buffer::new();
+
+        let meta = ReferenceTimestampMeta::add(
+            buffer.get_mut().unwrap(),
+            &caps,
+            crate::ClockTime::from_seconds(1),
+            crate::ClockTime::NONE,
+        );
+
+        let mut data_1 = Vec::new();
+        let mut data_2 = [0u8; 128];
+        let mut data_3 = SmallVec::<[u8; 128]>::new();
+
+        let len_1 = meta.serialize(&mut data_1).unwrap();
+        let len_2 = meta.serialize(&mut data_2).unwrap();
+        let len_3 = meta.serialize(&mut data_3).unwrap();
+        assert_eq!(&data_1[..len_1], &data_2[..len_2]);
+        assert_eq!(&data_1[..len_1], &data_3[..len_3]);
+
+        assert!(meta.serialize(&mut [0]).is_err());
+
+        let mut buffer_dest = crate::Buffer::new();
+        let mut consumed = 0;
+        let mut meta =
+            Meta::deserialize(buffer_dest.get_mut().unwrap(), &data_1, &mut consumed).unwrap();
+        assert_eq!(consumed, len_1);
+
+        let meta = meta.downcast_ref::<ReferenceTimestampMeta>().unwrap();
+        assert_eq!(meta.reference(), &caps);
+        assert_eq!(meta.timestamp(), crate::ClockTime::from_seconds(1));
+        assert_eq!(meta.duration(), crate::ClockTime::NONE);
+
+        let mut consumed = 0;
+        assert!(
+            Meta::deserialize(buffer_dest.get_mut().unwrap(), &[0, 1, 2], &mut consumed).is_err()
+        );
+        assert_eq!(consumed, 0);
+    }
 }
