@@ -1,8 +1,10 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use glib::translate::*;
 use std::fmt;
 
+use crate::prelude::*;
+use glib::translate::*;
+
 pub trait IsMiniObject:
     AsRef<Self::RefType> + FromGlibPtrFull<*mut Self::FfiType> + Send + Sync + 'static
 {
@@ -27,9 +29,17 @@ macro_rules! mini_object_wrapper (
         }
 
         impl $name {
+            #[inline]
+            pub unsafe fn from_glib_ptr_borrow<'a>(
+                ptr: *const *const $ffi_name,
+            ) -> &'a Self {
+                &*(ptr as *const $name)
+            }
+
+            #[inline]
             pub unsafe fn from_glib_none(ptr: *const $ffi_name) -> Self {
                 skip_assert_initialized!();
-                assert!(!ptr.is_null());
+                debug_assert!(!ptr.is_null());
 
                 $crate::ffi::gst_mini_object_ref(ptr as *mut $crate::ffi::GstMiniObject);
 
@@ -38,29 +48,34 @@ macro_rules! mini_object_wrapper (
                 }
             }
 
+            #[inline]
             pub unsafe fn from_glib_full(ptr: *const $ffi_name) -> Self {
                 skip_assert_initialized!();
-                assert!(!ptr.is_null());
+                debug_assert!(!ptr.is_null());
 
                 $name {
                     obj: std::ptr::NonNull::new_unchecked(ptr as *mut $ffi_name),
                 }
             }
 
+            #[inline]
             pub unsafe fn from_glib_borrow(ptr: *const $ffi_name) -> $crate::glib::translate::Borrowed<Self> {
                 skip_assert_initialized!();
-                assert!(!ptr.is_null());
+                debug_assert!(!ptr.is_null());
 
                 $crate::glib::translate::Borrowed::new($name {
                     obj: std::ptr::NonNull::new_unchecked(ptr as *mut $ffi_name),
                 })
             }
 
+            #[inline]
             pub unsafe fn replace_ptr(&mut self, ptr: *mut $ffi_name) {
-                assert!(!ptr.is_null());
+                debug_assert!(!ptr.is_null());
                 self.obj = std::ptr::NonNull::new_unchecked(ptr);
             }
 
+            #[inline]
+            #[doc(alias = "gst_mini_object_make_writable")]
             pub fn make_mut(&mut self) -> &mut $ref_name {
                 unsafe {
                     if self.is_writable() {
@@ -71,12 +86,13 @@ macro_rules! mini_object_wrapper (
                         self.as_mut_ptr() as *mut $crate::ffi::GstMiniObject
                     );
                     self.replace_ptr(ptr as *mut $ffi_name);
-                    assert!(self.is_writable());
+                    debug_assert!(self.is_writable());
 
                     &mut *(self.obj.as_mut() as *mut $ffi_name as *mut $ref_name)
                 }
             }
 
+            #[inline]
             pub fn get_mut(&mut self) -> Option<&mut $ref_name> {
                 if self.is_writable() {
                     Some(unsafe { &mut *(self.obj.as_mut() as *mut $ffi_name as *mut $ref_name) })
@@ -86,6 +102,7 @@ macro_rules! mini_object_wrapper (
             }
 
             #[doc(alias = "gst_mini_object_is_writable")]
+            #[inline]
             pub fn is_writable(&self) -> bool {
                 unsafe {
                     $crate::glib::translate::from_glib($crate::ffi::gst_mini_object_is_writable(
@@ -95,6 +112,7 @@ macro_rules! mini_object_wrapper (
             }
 
             #[must_use]
+            #[inline]
             pub fn upcast(self) -> $crate::miniobject::MiniObject {
                 use $crate::glib::translate::IntoGlibPtr;
 
@@ -105,6 +123,7 @@ macro_rules! mini_object_wrapper (
         }
 
         impl $crate::glib::translate::IntoGlibPtr<*mut $ffi_name> for $name {
+            #[inline]
             unsafe fn into_glib_ptr(self) -> *mut $ffi_name {
                 let s = std::mem::ManuallyDrop::new(self);
                 s.as_mut_ptr()
@@ -112,12 +131,14 @@ macro_rules! mini_object_wrapper (
         }
 
         impl Clone for $name {
+            #[inline]
             fn clone(&self) -> Self {
                 unsafe { $name::from_glib_none(self.as_ptr()) }
             }
         }
 
         impl Drop for $name {
+            #[inline]
             fn drop(&mut self) {
                 unsafe {
                     $crate::ffi::gst_mini_object_unref(self.as_mut_ptr() as *mut $crate::ffi::GstMiniObject);
@@ -128,30 +149,35 @@ macro_rules! mini_object_wrapper (
         impl std::ops::Deref for $name {
             type Target = $ref_name;
 
+            #[inline]
             fn deref(&self) -> &Self::Target {
                 unsafe { &*(self.obj.as_ref() as *const $ffi_name as *const $ref_name) }
             }
         }
 
         impl AsRef<$ref_name> for $name {
+            #[inline]
             fn as_ref(&self) -> &$ref_name {
                 &*self
             }
         }
 
         impl std::borrow::Borrow<$ref_name> for $name {
+            #[inline]
             fn borrow(&self) -> &$ref_name {
                 &*self
             }
         }
 
         impl<'a> $crate::glib::translate::ToGlibPtr<'a, *const $ffi_name> for $name {
-            type Storage = &'a Self;
+            type Storage = std::marker::PhantomData<&'a Self>;
 
+            #[inline]
             fn to_glib_none(&'a self) -> $crate::glib::translate::Stash<'a, *const $ffi_name, Self> {
-                $crate::glib::translate::Stash( self.as_ptr() , self)
+                $crate::glib::translate::Stash(self.as_ptr(), std::marker::PhantomData)
             }
 
+            #[inline]
             fn to_glib_full(&self) -> *const $ffi_name {
                 unsafe {
                     $crate::ffi::gst_mini_object_ref(self.as_mut_ptr() as *mut $crate::ffi::GstMiniObject);
@@ -161,12 +187,14 @@ macro_rules! mini_object_wrapper (
         }
 
         impl<'a> $crate::glib::translate::ToGlibPtr<'a, *mut $ffi_name> for $name {
-            type Storage = &'a Self;
+            type Storage = std::marker::PhantomData<&'a Self>;
 
+            #[inline]
             fn to_glib_none(&'a self) -> $crate::glib::translate::Stash<'a, *mut $ffi_name, Self> {
-                $crate::glib::translate::Stash( self.as_mut_ptr() , self)
+                $crate::glib::translate::Stash(self.as_mut_ptr(), std::marker::PhantomData)
             }
 
+            #[inline]
             fn to_glib_full(&self) -> *mut $ffi_name {
                 unsafe {
                     $crate::ffi::gst_mini_object_ref(self.as_mut_ptr() as *mut $crate::ffi::GstMiniObject);
@@ -176,57 +204,61 @@ macro_rules! mini_object_wrapper (
         }
 
         impl<'a> $crate::glib::translate::ToGlibPtrMut<'a, *mut $ffi_name> for $name {
-            type Storage = &'a mut Self;
+            type Storage = std::marker::PhantomData<&'a mut Self>;
 
+            #[inline]
             fn to_glib_none_mut(&'a mut self) -> $crate::glib::translate::StashMut<*mut $ffi_name, Self> {
                 self.make_mut();
-                $crate::glib::translate::StashMut( self.as_mut_ptr() , self)
+                $crate::glib::translate::StashMut(self.as_mut_ptr(), std::marker::PhantomData)
             }
         }
 
         impl<'a> $crate::glib::translate::ToGlibContainerFromSlice<'a, *mut *mut $ffi_name> for $name {
             #[allow(clippy::type_complexity)]
             type Storage = (
-                Vec<$crate::glib::translate::Stash<'a, *mut $ffi_name, Self>>,
+                std::marker::PhantomData<&'a [$name]>,
                 Option<Vec<*mut $ffi_name>>,
             );
 
             fn to_glib_none_from_slice(t: &'a [$name]) -> (*mut *mut $ffi_name, Self::Storage) {
                 skip_assert_initialized!();
-                let v: Vec<_> = t.iter().map(|s| $crate::glib::translate::ToGlibPtr::to_glib_none(s)).collect();
-                let mut v_ptr: Vec<_> = v.iter().map(|s| s.0).collect();
-                v_ptr.push(std::ptr::null_mut() as *mut $ffi_name);
+                let mut v_ptr = Vec::with_capacity(t.len() + 1);
+                unsafe {
+                    let ptr = v_ptr.as_mut_ptr();
+                    std::ptr::copy_nonoverlapping(t.as_ptr() as *mut *mut $ffi_name, ptr, t.len());
+                    std::ptr::write(ptr.add(t.len()), std::ptr::null_mut());
+                    v_ptr.set_len(t.len() + 1);
+                }
 
-                (v_ptr.as_ptr() as *mut *mut $ffi_name, (v, Some(v_ptr)))
+                (v_ptr.as_ptr() as *mut *mut $ffi_name, (std::marker::PhantomData, Some(v_ptr)))
             }
 
             fn to_glib_container_from_slice(t: &'a [$name]) -> (*mut *mut $ffi_name, Self::Storage) {
                 skip_assert_initialized!();
-                let v: Vec<_> = t.iter().map(|s| $crate::glib::translate::ToGlibPtr::to_glib_none(s)).collect();
 
                 let v_ptr = unsafe {
-                    let v_ptr = $crate::glib::ffi::g_malloc0(std::mem::size_of::<*mut $ffi_name>() * t.len() + 1)
+                    let v_ptr = $crate::glib::ffi::g_malloc(std::mem::size_of::<*mut $ffi_name>() * t.len() + 1)
                         as *mut *mut $ffi_name;
 
-                    for (i, s) in v.iter().enumerate() {
-                        std::ptr::write(v_ptr.add(i), s.0);
-                    }
+                    std::ptr::copy_nonoverlapping(t.as_ptr() as *mut *mut $ffi_name, v_ptr, t.len());
+                    std::ptr::write(v_ptr.add(t.len()), std::ptr::null_mut());
 
                     v_ptr
                 };
 
-                (v_ptr, (v, None))
+                (v_ptr, (std::marker::PhantomData, None))
             }
 
             fn to_glib_full_from_slice(t: &[$name]) -> *mut *mut $ffi_name {
                 skip_assert_initialized!();
                 unsafe {
-                    let v_ptr = $crate::glib::ffi::g_malloc0(std::mem::size_of::<*mut $ffi_name>() * t.len() + 1)
+                    let v_ptr = $crate::glib::ffi::g_malloc(std::mem::size_of::<*mut $ffi_name>() * t.len() + 1)
                         as *mut *mut $ffi_name;
 
                     for (i, s) in t.iter().enumerate() {
                         std::ptr::write(v_ptr.add(i), $crate::glib::translate::ToGlibPtr::to_glib_full(s));
                     }
+                    std::ptr::write(v_ptr.add(t.len()), std::ptr::null_mut());
 
                     v_ptr
                 }
@@ -238,7 +270,7 @@ macro_rules! mini_object_wrapper (
         {
             #[allow(clippy::type_complexity)]
             type Storage = (
-                Vec<$crate::glib::translate::Stash<'a, *mut $ffi_name, $name>>,
+                std::marker::PhantomData<&'a [$name]>,
                 Option<Vec<*mut $ffi_name>>,
             );
 
@@ -263,36 +295,42 @@ macro_rules! mini_object_wrapper (
         }
 
         impl $crate::glib::translate::FromGlibPtrNone<*const $ffi_name> for $name {
+            #[inline]
             unsafe fn from_glib_none(ptr: *const $ffi_name) -> Self {
                 Self::from_glib_none(ptr)
             }
         }
 
         impl $crate::glib::translate::FromGlibPtrNone<*mut $ffi_name> for $name {
+            #[inline]
             unsafe fn from_glib_none(ptr: *mut $ffi_name) -> Self {
                 Self::from_glib_none(ptr)
             }
         }
 
         impl $crate::glib::translate::FromGlibPtrFull<*const $ffi_name> for $name {
+            #[inline]
             unsafe fn from_glib_full(ptr: *const $ffi_name) -> Self {
                 Self::from_glib_full(ptr)
             }
         }
 
         impl $crate::glib::translate::FromGlibPtrFull<*mut $ffi_name> for $name {
+            #[inline]
             unsafe fn from_glib_full(ptr: *mut $ffi_name) -> Self {
                 Self::from_glib_full(ptr)
             }
         }
 
         impl $crate::glib::translate::FromGlibPtrBorrow<*const $ffi_name> for $name {
+            #[inline]
             unsafe fn from_glib_borrow(ptr: *const $ffi_name) -> $crate::glib::translate::Borrowed<Self> {
                 Self::from_glib_borrow(ptr)
             }
         }
 
         impl $crate::glib::translate::FromGlibPtrBorrow<*mut $ffi_name> for $name {
+            #[inline]
             unsafe fn from_glib_borrow(ptr: *mut $ffi_name) -> $crate::glib::translate::Borrowed<Self> {
                 Self::from_glib_borrow(ptr)
             }
@@ -306,10 +344,12 @@ macro_rules! mini_object_wrapper (
                     return Vec::new();
                 }
 
-                let mut res = Vec::with_capacity(num);
+                let mut res = Vec::<Self>::with_capacity(num);
+                let res_ptr = res.as_mut_ptr();
                 for i in 0..num {
-                    res.push($crate::glib::translate::from_glib_none(std::ptr::read(ptr.add(i))));
+                    ::std::ptr::write(res_ptr.add(i), $crate::glib::translate::from_glib_none(std::ptr::read(ptr.add(i))));
                 }
+                res.set_len(num);
                 res
             }
 
@@ -325,9 +365,9 @@ macro_rules! mini_object_wrapper (
                 }
 
                 let mut res = Vec::with_capacity(num);
-                for i in 0..num {
-                    res.push($crate::glib::translate::from_glib_full(std::ptr::read(ptr.add(i))));
-                }
+                let res_ptr = res.as_mut_ptr();
+                ::std::ptr::copy_nonoverlapping(ptr as *mut Self, res_ptr, num);
+                res.set_len(num);
                 $crate::glib::ffi::g_free(ptr as *mut _);
                 res
             }
@@ -388,23 +428,29 @@ macro_rules! mini_object_wrapper (
             type GlibType = *mut $ffi_name;
         }
 
+        unsafe impl $crate::glib::translate::TransparentPtrType for $name {}
+
         impl $ref_name {
+            #[inline]
             pub fn as_ptr(&self) -> *const $ffi_name {
                 self as *const Self as *const $ffi_name
             }
 
+            #[inline]
             pub fn as_mut_ptr(&self) -> *mut $ffi_name {
                 self as *const Self as *mut $ffi_name
             }
 
+            #[inline]
             pub unsafe fn from_ptr<'a>(ptr: *const $ffi_name) -> &'a Self {
-                assert!(!ptr.is_null());
+                debug_assert!(!ptr.is_null());
                 &*(ptr as *const Self)
             }
 
+            #[inline]
             pub unsafe fn from_mut_ptr<'a>(ptr: *mut $ffi_name) -> &'a mut Self {
-                assert!(!ptr.is_null());
-                assert_ne!(
+                debug_assert!(!ptr.is_null());
+                debug_assert_ne!(
                     $crate::ffi::gst_mini_object_is_writable(ptr as *mut $crate::ffi::GstMiniObject),
                     $crate::glib::ffi::GFALSE
                 );
@@ -412,6 +458,7 @@ macro_rules! mini_object_wrapper (
             }
 
             #[doc(alias = "gst_mini_object_copy")]
+            #[inline]
             pub fn copy(&self) -> $name {
                 unsafe {
                     $name::from_glib_full($crate::ffi::gst_mini_object_copy(
@@ -420,18 +467,21 @@ macro_rules! mini_object_wrapper (
                 }
             }
 
+            #[inline]
             pub fn upcast_ref(&self) -> &$crate::miniobject::MiniObjectRef {
                 unsafe {
                     &*(self.as_ptr() as *const $crate::miniobject::MiniObjectRef)
                 }
             }
 
+            #[inline]
             pub fn upcast_mut(&mut self) -> &mut $crate::miniobject::MiniObjectRef {
                 unsafe {
                     &mut *(self.as_mut_ptr() as *mut $crate::miniobject::MiniObjectRef)
                 }
             }
 
+            #[inline]
             pub fn ptr_eq(this: &$ref_name, other: &$ref_name) -> bool {
                 skip_assert_initialized!();
                 this.as_ptr() == other.as_ptr()
@@ -445,6 +495,7 @@ macro_rules! mini_object_wrapper (
         impl ToOwned for $ref_name {
             type Owned = $name;
 
+            #[inline]
             fn to_owned(&self) -> $name {
                 self.copy()
             }
@@ -459,12 +510,15 @@ macro_rules! mini_object_wrapper (
         $crate::mini_object_wrapper!($name, $ref_name, $ffi_name);
 
         impl $crate::glib::types::StaticType for $name {
+            #[inline]
             fn static_type() -> $crate::glib::types::Type {
                 $ref_name::static_type()
             }
         }
 
+        #[allow(clippy::redundant_closure_call)]
         impl $crate::glib::types::StaticType for $ref_name {
+            #[inline]
             fn static_type() -> $crate::glib::types::Type {
                 unsafe { $crate::glib::translate::from_glib($get_type()) }
             }
@@ -479,6 +533,7 @@ macro_rules! mini_object_wrapper (
         unsafe impl<'a> $crate::glib::value::FromValue<'a> for $name {
             type Checker = $crate::glib::value::GenericValueTypeOrNoneChecker<Self>;
 
+            #[inline]
             unsafe fn from_value(value: &'a $crate::glib::Value) -> Self {
                 skip_assert_initialized!();
                 $crate::glib::translate::from_glib_none(
@@ -490,17 +545,19 @@ macro_rules! mini_object_wrapper (
         unsafe impl<'a> $crate::glib::value::FromValue<'a> for &'a $name {
             type Checker = $crate::glib::value::GenericValueTypeOrNoneChecker<Self>;
 
+            #[inline]
             unsafe fn from_value(value: &'a $crate::glib::Value) -> Self {
                 skip_assert_initialized!();
                 assert_eq!(std::mem::size_of::<$name>(), std::mem::size_of::<$crate::glib::ffi::gpointer>());
                 let value = &*(value as *const $crate::glib::Value as *const $crate::glib::gobject_ffi::GValue);
                 let ptr = &value.data[0].v_pointer as *const $crate::glib::ffi::gpointer as *const *const $ffi_name;
-                assert!(!(*ptr).is_null());
+                debug_assert!(!(*ptr).is_null());
                 &*(ptr as *const $name)
             }
         }
 
         impl $crate::glib::value::ToValue for $name {
+            #[inline]
             fn to_value(&self) -> $crate::glib::Value {
                 let mut value = $crate::glib::Value::for_value_type::<Self>();
                 unsafe {
@@ -512,12 +569,14 @@ macro_rules! mini_object_wrapper (
                 value
             }
 
+            #[inline]
             fn value_type(&self) -> $crate::glib::Type {
-                <Self as $crate::glib::StaticType>::static_type()
+                <Self as $crate::glib::prelude::StaticType>::static_type()
             }
         }
 
         impl $crate::glib::value::ToValueOptional for $name {
+            #[inline]
             fn to_value_optional(s: Option<&Self>) -> $crate::glib::Value {
                 skip_assert_initialized!();
                 let mut value = $crate::glib::Value::for_value_type::<Self>();
@@ -531,9 +590,25 @@ macro_rules! mini_object_wrapper (
             }
         }
 
+        impl From<$name> for $crate::glib::Value {
+            #[inline]
+            fn from(v: $name) -> $crate::glib::Value {
+                skip_assert_initialized!();
+                let mut value = $crate::glib::Value::for_value_type::<$name>();
+                unsafe {
+                    $crate::glib::gobject_ffi::g_value_take_boxed(
+                        $crate::glib::translate::ToGlibPtrMut::to_glib_none_mut(&mut value).0,
+                        $crate::glib::translate::IntoGlibPtr::<*mut $ffi_name>::into_glib_ptr(v) as *mut _,
+                    )
+                }
+                value
+            }
+        }
+
         unsafe impl<'a> $crate::glib::value::FromValue<'a> for &'a $ref_name {
             type Checker = $crate::glib::value::GenericValueTypeOrNoneChecker<Self>;
 
+            #[inline]
             unsafe fn from_value(value: &'a $crate::glib::Value) -> Self {
                 skip_assert_initialized!();
                 &*($crate::glib::gobject_ffi::g_value_get_boxed($crate::glib::translate::ToGlibPtr::to_glib_none(value).0) as *const $ref_name)
@@ -542,19 +617,30 @@ macro_rules! mini_object_wrapper (
 
         // Can't have SetValue/SetValueOptional impls as otherwise one could use it to get
         // immutable references from a mutable reference without borrowing via the value
+
+        impl $crate::glib::prelude::HasParamSpec for $name {
+            type ParamSpec = $crate::glib::ParamSpecBoxed;
+            type SetValue = Self;
+            type BuilderFn = fn(&str) -> $crate::glib::ParamSpecBoxedBuilder<Self>;
+
+            fn param_spec_builder() -> Self::BuilderFn {
+                |name| Self::ParamSpec::builder(name)
+            }
+        }
     };
 );
 
-#[cfg(not(any(feature = "v1_20", feature = "dox")))]
+#[cfg(not(any(feature = "v1_20", docsrs)))]
 mini_object_wrapper!(MiniObject, MiniObjectRef, ffi::GstMiniObject);
 
-#[cfg(any(feature = "v1_20", feature = "dox"))]
+#[cfg(feature = "v1_20")]
 mini_object_wrapper!(MiniObject, MiniObjectRef, ffi::GstMiniObject, || {
     ffi::gst_mini_object_get_type()
 });
 
 impl MiniObject {
-    pub fn downcast<T: IsMiniObject + glib::StaticType>(self) -> Result<T, Self> {
+    #[inline]
+    pub fn downcast<T: IsMiniObject + StaticType>(self) -> Result<T, Self> {
         if self.type_().is_a(T::static_type()) {
             unsafe { Ok(from_glib_full(self.into_glib_ptr() as *mut T::FfiType)) }
         } else {
@@ -579,11 +665,13 @@ impl fmt::Debug for MiniObjectRef {
 }
 
 impl MiniObjectRef {
+    #[inline]
     pub fn type_(&self) -> glib::Type {
         unsafe { from_glib((*self.as_ptr()).type_) }
     }
 
-    pub fn downcast_ref<T: IsMiniObject + glib::StaticType>(&self) -> Option<&T::RefType> {
+    #[inline]
+    pub fn downcast_ref<T: IsMiniObject + StaticType>(&self) -> Option<&T::RefType> {
         if self.type_().is_a(T::static_type()) {
             unsafe { Some(&*(self as *const Self as *const T::RefType)) }
         } else {
@@ -591,7 +679,8 @@ impl MiniObjectRef {
         }
     }
 
-    pub fn downcast_mut<T: IsMiniObject + glib::StaticType>(&mut self) -> Option<&mut T::RefType> {
+    #[inline]
+    pub fn downcast_mut<T: IsMiniObject + StaticType>(&mut self) -> Option<&mut T::RefType> {
         if self.type_().is_a(T::static_type()) {
             unsafe { Some(&mut *(self as *mut Self as *mut T::RefType)) }
         } else {
