@@ -1,12 +1,8 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use crate::Plugin;
-use crate::PluginFlags;
-use crate::Structure;
-use crate::StructureRef;
+use glib::{prelude::*, translate::*};
 
-use glib::prelude::*;
-use glib::translate::*;
+use crate::{prelude::*, Plugin, PluginFlags, StructureRef};
 
 impl Plugin {
     #[doc(alias = "get_cache_data")]
@@ -22,24 +18,11 @@ impl Plugin {
         }
     }
 
-    #[doc(alias = "gst_plugin_set_cache_data")]
-    pub fn set_cache_data(&self, cache_data: Structure) {
-        unsafe {
-            ffi::gst_plugin_set_cache_data(self.to_glib_none().0, cache_data.into_glib_ptr());
-        }
-    }
-}
-
-pub trait GstPluginExtManual: 'static {
     #[doc(alias = "get_plugin_flags")]
-    fn plugin_flags(&self) -> PluginFlags;
-}
-
-impl<O: IsA<crate::Plugin>> GstPluginExtManual for O {
-    fn plugin_flags(&self) -> PluginFlags {
+    pub fn plugin_flags(&self) -> PluginFlags {
         unsafe {
             let ptr: *mut ffi::GstObject = self.as_ptr() as *mut _;
-            let _guard = crate::utils::MutexGuard::lock(&(*ptr).lock);
+            let _guard = self.object_lock();
             from_glib((*ptr).flags)
         }
     }
