@@ -1,17 +1,19 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use crate::format::{CompatibleFormattedValue, FormattedValue, GenericFormattedValue};
-use crate::structure::*;
+use std::{
+    borrow::{Borrow, BorrowMut},
+    ffi::CStr,
+    fmt, mem,
+    ops::{Deref, DerefMut},
+    ptr,
+};
 
-use std::borrow::{Borrow, BorrowMut};
-use std::ffi::CStr;
-use std::fmt;
-use std::mem;
-use std::ops::{Deref, DerefMut};
-use std::ptr;
+use glib::{object::IsA, translate::*};
 
-use glib::object::IsA;
-use glib::translate::*;
+use crate::{
+    format::{CompatibleFormattedValue, FormattedValue, GenericFormattedValue},
+    structure::*,
+};
 
 mini_object_wrapper!(Query, QueryRef, ffi::GstQuery, || {
     ffi::gst_query_get_type()
@@ -20,6 +22,7 @@ mini_object_wrapper!(Query, QueryRef, ffi::GstQuery, || {
 impl QueryRef {
     #[doc(alias = "get_structure")]
     #[doc(alias = "gst_query_get_structure")]
+    #[inline]
     pub fn structure(&self) -> Option<&StructureRef> {
         unsafe {
             let structure = ffi::gst_query_get_structure(self.as_mut_ptr());
@@ -33,6 +36,7 @@ impl QueryRef {
 
     #[doc(alias = "get_mut_structure")]
     #[doc(alias = "gst_query_writable_structure")]
+    #[inline]
     pub fn structure_mut(&mut self) -> &mut StructureRef {
         unsafe {
             let structure = ffi::gst_query_writable_structure(self.as_mut_ptr());
@@ -41,16 +45,19 @@ impl QueryRef {
     }
 
     #[doc(alias = "GST_QUERY_IS_DOWNSTREAM")]
+    #[inline]
     pub fn is_downstream(&self) -> bool {
         unsafe { ((*self.as_ptr()).type_ as u32) & ffi::GST_QUERY_TYPE_DOWNSTREAM != 0 }
     }
 
     #[doc(alias = "GST_QUERY_IS_UPSTREAM")]
+    #[inline]
     pub fn is_upstream(&self) -> bool {
         unsafe { ((*self.as_ptr()).type_ as u32) & ffi::GST_QUERY_TYPE_UPSTREAM != 0 }
     }
 
     #[doc(alias = "GST_QUERY_IS_SERIALIZED")]
+    #[inline]
     pub fn is_serialized(&self) -> bool {
         unsafe { ((*self.as_ptr()).type_ as u32) & ffi::GST_QUERY_TYPE_SERIALIZED != 0 }
     }
@@ -76,9 +83,9 @@ impl QueryRef {
                 ffi::GST_QUERY_CAPS => Caps::view(self),
                 ffi::GST_QUERY_DRAIN => Drain::view(self),
                 ffi::GST_QUERY_CONTEXT => Context::view(self),
-                #[cfg(any(feature = "v1_16", feature = "dox"))]
+                #[cfg(feature = "v1_16")]
                 ffi::GST_QUERY_BITRATE => Bitrate::view(self),
-                #[cfg(any(feature = "v1_22", feature = "dox"))]
+                #[cfg(feature = "v1_22")]
                 ffi::GST_QUERY_SELECTABLE => Selectable::view(self),
                 _ => Other::view(self),
             }
@@ -106,9 +113,9 @@ impl QueryRef {
                 ffi::GST_QUERY_CAPS => Caps::view_mut(self),
                 ffi::GST_QUERY_DRAIN => Drain::view_mut(self),
                 ffi::GST_QUERY_CONTEXT => Context::view_mut(self),
-                #[cfg(any(feature = "v1_16", feature = "dox"))]
+                #[cfg(feature = "v1_16")]
                 ffi::GST_QUERY_BITRATE => Bitrate::view_mut(self),
-                #[cfg(any(feature = "v1_22", feature = "dox"))]
+                #[cfg(feature = "v1_22")]
                 ffi::GST_QUERY_SELECTABLE => Selectable::view_mut(self),
                 _ => Other::view_mut(self),
             }
@@ -154,11 +161,11 @@ pub enum QueryView<'a> {
     Caps(&'a Caps),
     Drain(&'a Drain),
     Context(&'a Context),
-    #[cfg(any(feature = "v1_16", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+    #[cfg(feature = "v1_16")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
     Bitrate(&'a Bitrate),
-    #[cfg(any(feature = "v1_22", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+    #[cfg(feature = "v1_22")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
     Selectable(&'a Selectable),
     Other(&'a Other),
 }
@@ -182,35 +189,38 @@ pub enum QueryViewMut<'a> {
     Caps(&'a mut Caps),
     Drain(&'a mut Drain),
     Context(&'a mut Context),
-    #[cfg(any(feature = "v1_16", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+    #[cfg(feature = "v1_16")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
     Bitrate(&'a mut Bitrate),
-    #[cfg(any(feature = "v1_22", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+    #[cfg(feature = "v1_22")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
     Selectable(&'a mut Selectable),
     Other(&'a mut Other),
 }
 
 macro_rules! declare_concrete_query(
     ($name:ident, $param:ident) => {
-        #[derive(Debug)]
         #[repr(transparent)]
         pub struct $name<$param = QueryRef>($param);
 
         impl $name {
+            #[inline]
             pub fn query(&self) -> &QueryRef {
                 unsafe { &*(self as *const Self as *const QueryRef) }
             }
 
+            #[inline]
             pub fn query_mut(&mut self) -> &mut QueryRef {
                 unsafe { &mut *(self as *mut Self as *mut QueryRef) }
             }
 
+            #[inline]
             unsafe fn view(query: &QueryRef) -> QueryView<'_> {
                 let query = &*(query as *const QueryRef as *const Self);
                 QueryView::$name(query)
             }
 
+            #[inline]
             unsafe fn view_mut(query: &mut QueryRef) -> QueryViewMut<'_> {
                 let query = &mut *(query as *mut QueryRef as *mut Self);
                 QueryViewMut::$name(query)
@@ -220,12 +230,14 @@ macro_rules! declare_concrete_query(
         impl Deref for $name {
             type Target = QueryRef;
 
+            #[inline]
             fn deref(&self) -> &Self::Target {
                 self.query()
             }
         }
 
         impl DerefMut for $name {
+            #[inline]
             fn deref_mut(&mut self) -> &mut Self::Target {
                 self.query_mut()
             }
@@ -234,12 +246,14 @@ macro_rules! declare_concrete_query(
         impl ToOwned for $name {
             type Owned = $name<Query>;
 
+            #[inline]
             fn to_owned(&self) -> Self::Owned {
                 $name::<Query>(self.copy())
             }
         }
 
         impl $name<Query> {
+            #[inline]
             pub fn get_mut(&mut self) -> Option<&mut $name> {
                 self.0
                     .get_mut()
@@ -250,31 +264,36 @@ macro_rules! declare_concrete_query(
         impl Deref for $name<Query> {
             type Target = $name;
 
+            #[inline]
             fn deref(&self) -> &Self::Target {
                 unsafe { &*(self.0.as_ptr() as *const Self::Target) }
             }
         }
 
         impl DerefMut for $name<Query> {
+            #[inline]
             fn deref_mut(&mut self) -> &mut Self::Target {
-                assert!(self.0.is_writable());
+                debug_assert!(self.0.is_writable());
                 unsafe { &mut *(self.0.as_mut_ptr() as *mut Self::Target) }
             }
         }
 
         impl Borrow<$name> for $name<Query> {
+            #[inline]
             fn borrow(&self) -> &$name {
                 &*self
             }
         }
 
         impl BorrowMut<$name> for $name<Query> {
+            #[inline]
             fn borrow_mut(&mut self) -> &mut $name {
                 &mut *self
             }
         }
 
         impl From<$name<Query>> for Query {
+            #[inline]
             fn from(concrete: $name<Query>) -> Self {
                 skip_assert_initialized!();
                 concrete.0
@@ -331,6 +350,22 @@ impl Position {
     }
 }
 
+impl std::fmt::Debug for Position {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Position")
+            .field("structure", &self.query().structure())
+            .field("result", &self.result())
+            .field("format", &self.format())
+            .finish()
+    }
+}
+
+impl std::fmt::Debug for Position<Query> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        Position::<QueryRef>::fmt(self, f)
+    }
+}
+
 declare_concrete_query!(Duration, T);
 impl Duration<Query> {
     #[doc(alias = "gst_query_new_duration")]
@@ -379,6 +414,22 @@ impl Duration {
     }
 }
 
+impl std::fmt::Debug for Duration {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Duration")
+            .field("structure", &self.query().structure())
+            .field("result", &self.result())
+            .field("format", &self.format())
+            .finish()
+    }
+}
+
+impl std::fmt::Debug for Duration<Query> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        Duration::<QueryRef>::fmt(self, f)
+    }
+}
+
 declare_concrete_query!(Latency, T);
 impl Latency<Query> {
     #[doc(alias = "gst_query_new_latency")]
@@ -436,6 +487,21 @@ impl Latency {
     }
 }
 
+impl std::fmt::Debug for Latency {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Latency")
+            .field("structure", &self.query().structure())
+            .field("result", &self.result())
+            .finish()
+    }
+}
+
+impl std::fmt::Debug for Latency<Query> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        Latency::<QueryRef>::fmt(self, f)
+    }
+}
+
 declare_concrete_query!(Seeking, T);
 impl Seeking<Query> {
     #[doc(alias = "gst_query_new_seeking")]
@@ -509,6 +575,22 @@ impl Seeking {
     }
 }
 
+impl std::fmt::Debug for Seeking {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Seeking")
+            .field("structure", &self.query().structure())
+            .field("result", &self.result())
+            .field("format", &self.format())
+            .finish()
+    }
+}
+
+impl std::fmt::Debug for Seeking<Query> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        Seeking::<QueryRef>::fmt(self, f)
+    }
+}
+
 declare_concrete_query!(Segment, T);
 impl Segment<Query> {
     #[doc(alias = "gst_query_new_segment")]
@@ -581,6 +663,22 @@ impl Segment {
     }
 }
 
+impl std::fmt::Debug for Segment {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Segment")
+            .field("structure", &self.query().structure())
+            .field("result", &self.result())
+            .field("format", &self.format())
+            .finish()
+    }
+}
+
+impl std::fmt::Debug for Segment<Query> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        Segment::<QueryRef>::fmt(self, f)
+    }
+}
+
 declare_concrete_query!(Convert, T);
 impl Convert<Query> {
     #[doc(alias = "gst_query_new_convert")]
@@ -655,6 +753,24 @@ impl Convert {
     }
 }
 
+impl std::fmt::Debug for Convert {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        let (source, dest) = self.result();
+
+        f.debug_struct("Convert")
+            .field("structure", &self.query().structure())
+            .field("source", &source)
+            .field("dest", &dest)
+            .finish()
+    }
+}
+
+impl std::fmt::Debug for Convert<Query> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        Convert::<QueryRef>::fmt(self, f)
+    }
+}
+
 declare_concrete_query!(Formats, T);
 impl Formats<Query> {
     #[doc(alias = "gst_query_new_formats")]
@@ -701,6 +817,21 @@ impl Formats {
     }
 }
 
+impl std::fmt::Debug for Formats {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Formats")
+            .field("structure", &self.query().structure())
+            .field("result", &self.result())
+            .finish()
+    }
+}
+
+impl std::fmt::Debug for Formats<Query> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        Formats::<QueryRef>::fmt(self, f)
+    }
+}
+
 declare_concrete_query!(Buffering, T);
 impl Buffering<Query> {
     #[doc(alias = "gst_query_new_buffering")]
@@ -907,11 +1038,28 @@ impl Buffering {
     }
 }
 
+impl std::fmt::Debug for Buffering {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Buffering")
+            .field("structure", &self.query().structure())
+            .field("format", &self.format())
+            .field("percent", &self.percent())
+            .field("range", &self.range())
+            .finish()
+    }
+}
+
+impl std::fmt::Debug for Buffering<Query> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        Buffering::<QueryRef>::fmt(self, f)
+    }
+}
+
 declare_concrete_query!(Custom, T);
 impl Custom<Query> {
     #[doc(alias = "gst_query_new_custom")]
     pub fn new(structure: crate::Structure) -> Self {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
         unsafe {
             Self(from_glib_full(ffi::gst_query_new_custom(
                 ffi::GST_QUERY_CUSTOM,
@@ -921,6 +1069,20 @@ impl Custom<Query> {
     }
 }
 
+impl std::fmt::Debug for Custom {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Custom")
+            .field("structure", &self.query().structure())
+            .finish()
+    }
+}
+
+impl std::fmt::Debug for Custom<Query> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        Custom::<QueryRef>::fmt(self, f)
+    }
+}
+
 declare_concrete_query!(Uri, T);
 impl Uri<Query> {
     #[doc(alias = "gst_query_new_uri")]
@@ -996,14 +1158,33 @@ impl Uri {
     }
 }
 
+impl std::fmt::Debug for Uri {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        let (redirection, permanent) = self.redirection();
+        f.debug_struct("Uri")
+            .field("structure", &self.query().structure())
+            .field("uri", &self.uri())
+            .field("redirection", &redirection)
+            .field("redirection-permanent", &permanent)
+            .finish()
+    }
+}
+
+impl std::fmt::Debug for Uri<Query> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        Uri::<QueryRef>::fmt(self, f)
+    }
+}
+
 declare_concrete_query!(Allocation, T);
 impl Allocation<Query> {
     #[doc(alias = "gst_query_new_allocation")]
-    pub fn new(caps: &crate::Caps, need_pool: bool) -> Self {
-        assert_initialized_main_thread!();
+    pub fn new(caps: Option<&crate::Caps>, need_pool: bool) -> Self {
+        skip_assert_initialized!();
         unsafe {
             Self(from_glib_full(ffi::gst_query_new_allocation(
-                caps.as_mut_ptr(),
+                caps.map(|caps| caps.as_mut_ptr())
+                    .unwrap_or(ptr::null_mut()),
                 need_pool.into_glib(),
             )))
         }
@@ -1012,24 +1193,28 @@ impl Allocation<Query> {
 
 impl Allocation {
     #[doc(alias = "gst_query_parse_allocation")]
-    pub fn get(&self) -> (&crate::CapsRef, bool) {
+    pub fn get(&self) -> (Option<&crate::CapsRef>, bool) {
         unsafe {
             let mut caps = ptr::null_mut();
             let mut need_pool = mem::MaybeUninit::uninit();
 
             ffi::gst_query_parse_allocation(self.as_mut_ptr(), &mut caps, need_pool.as_mut_ptr());
             (
-                crate::CapsRef::from_ptr(caps),
+                if caps.is_null() {
+                    None
+                } else {
+                    Some(crate::CapsRef::from_ptr(caps))
+                },
                 from_glib(need_pool.assume_init()),
             )
         }
     }
 
     #[doc(alias = "gst_query_parse_allocation")]
-    pub fn get_owned(&self) -> (crate::Caps, bool) {
+    pub fn get_owned(&self) -> (Option<crate::Caps>, bool) {
         unsafe {
             let (caps, need_pool) = self.get();
-            (from_glib_none(caps.as_ptr()), need_pool)
+            (caps.map(|caps| from_glib_none(caps.as_ptr())), need_pool)
         }
     }
 
@@ -1254,6 +1439,26 @@ impl Allocation {
     }
 }
 
+impl std::fmt::Debug for Allocation {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        let (caps, need_pool) = self.get();
+        f.debug_struct("Allocation")
+            .field("structure", &self.query().structure())
+            .field("caps", &caps)
+            .field("need-pool", &need_pool)
+            .field("allocation-params", &self.allocation_params())
+            .field("allocation-pools", &self.allocation_pools())
+            .field("allocation-metas", &self.allocation_metas())
+            .finish()
+    }
+}
+
+impl std::fmt::Debug for Allocation<Query> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        Allocation::<QueryRef>::fmt(self, f)
+    }
+}
+
 declare_concrete_query!(Scheduling, T);
 impl Scheduling<Query> {
     #[doc(alias = "gst_query_new_scheduling")]
@@ -1362,11 +1567,27 @@ impl Scheduling {
     }
 }
 
+impl std::fmt::Debug for Scheduling {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Scheduling")
+            .field("structure", &self.query().structure())
+            .field("result", &self.result())
+            .field("scheduling-modes", &self.scheduling_modes())
+            .finish()
+    }
+}
+
+impl std::fmt::Debug for Scheduling<Query> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        Scheduling::<QueryRef>::fmt(self, f)
+    }
+}
+
 declare_concrete_query!(AcceptCaps, T);
 impl AcceptCaps<Query> {
     #[doc(alias = "gst_query_new_accept_caps")]
     pub fn new(caps: &crate::Caps) -> Self {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
         unsafe {
             Self(from_glib_full(ffi::gst_query_new_accept_caps(
                 caps.as_mut_ptr(),
@@ -1410,11 +1631,27 @@ impl AcceptCaps {
     }
 }
 
+impl std::fmt::Debug for AcceptCaps {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("AcceptCaps")
+            .field("structure", &self.query().structure())
+            .field("result", &self.result())
+            .field("caps", &self.caps())
+            .finish()
+    }
+}
+
+impl std::fmt::Debug for AcceptCaps<Query> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        AcceptCaps::<QueryRef>::fmt(self, f)
+    }
+}
+
 declare_concrete_query!(Caps, T);
 impl Caps<Query> {
     #[doc(alias = "gst_query_new_caps")]
     pub fn new(filter: Option<&crate::Caps>) -> Self {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
         unsafe {
             Self(from_glib_full(ffi::gst_query_new_caps(
                 filter.to_glib_none().0,
@@ -1477,6 +1714,22 @@ impl Caps {
     }
 }
 
+impl std::fmt::Debug for Caps {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Caps")
+            .field("structure", &self.query().structure())
+            .field("result", &self.result())
+            .field("filter", &self.filter())
+            .finish()
+    }
+}
+
+impl std::fmt::Debug for Caps<Query> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        Caps::<QueryRef>::fmt(self, f)
+    }
+}
+
 declare_concrete_query!(Drain, T);
 impl Drain<Query> {
     #[doc(alias = "gst_query_new_drain")]
@@ -1492,6 +1745,20 @@ impl Default for Drain<Query> {
     }
 }
 
+impl std::fmt::Debug for Drain {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Drain")
+            .field("structure", &self.query().structure())
+            .finish()
+    }
+}
+
+impl std::fmt::Debug for Drain<Query> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        Drain::<QueryRef>::fmt(self, f)
+    }
+}
+
 declare_concrete_query!(Context, T);
 impl Context<Query> {
     #[doc(alias = "gst_query_new_context")]
@@ -1553,12 +1820,28 @@ impl Context {
     }
 }
 
-#[cfg(any(feature = "v1_16", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+impl std::fmt::Debug for Context {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Context")
+            .field("structure", &self.query().structure())
+            .field("context", &self.context())
+            .field("context-type", &self.context_type())
+            .finish()
+    }
+}
+
+impl std::fmt::Debug for Context<Query> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        Context::<QueryRef>::fmt(self, f)
+    }
+}
+
+#[cfg(feature = "v1_16")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
 declare_concrete_query!(Bitrate, T);
 
-#[cfg(any(feature = "v1_16", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+#[cfg(feature = "v1_16")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
 impl Bitrate<Query> {
     #[doc(alias = "gst_query_new_bitrate")]
     pub fn new() -> Self {
@@ -1567,16 +1850,16 @@ impl Bitrate<Query> {
     }
 }
 
-#[cfg(any(feature = "v1_16", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+#[cfg(feature = "v1_16")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
 impl Default for Bitrate<Query> {
     fn default() -> Self {
         Self::new()
     }
 }
 
-#[cfg(any(feature = "v1_16", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
+#[cfg(feature = "v1_16")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_16")))]
 impl Bitrate {
     #[doc(alias = "get_bitrate")]
     #[doc(alias = "gst_query_parse_bitrate")]
@@ -1596,12 +1879,29 @@ impl Bitrate {
     }
 }
 
-#[cfg(any(feature = "v1_22", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+#[cfg(feature = "v1_16")]
+impl std::fmt::Debug for Bitrate {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Bitrate")
+            .field("structure", &self.query().structure())
+            .field("bitrate", &self.bitrate())
+            .finish()
+    }
+}
+
+#[cfg(feature = "v1_16")]
+impl std::fmt::Debug for Bitrate<Query> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        Bitrate::<QueryRef>::fmt(self, f)
+    }
+}
+
+#[cfg(feature = "v1_22")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
 declare_concrete_query!(Selectable, T);
 
-#[cfg(any(feature = "v1_22", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+#[cfg(feature = "v1_22")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
 impl Selectable<Query> {
     #[doc(alias = "gst_query_new_selectable")]
     pub fn new() -> Self {
@@ -1610,16 +1910,16 @@ impl Selectable<Query> {
     }
 }
 
-#[cfg(any(feature = "v1_22", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+#[cfg(feature = "v1_22")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
 impl Default for Selectable<Query> {
     fn default() -> Self {
         Self::new()
     }
 }
 
-#[cfg(any(feature = "v1_22", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_22")))]
+#[cfg(feature = "v1_22")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_22")))]
 impl Selectable {
     #[doc(alias = "get_selectable")]
     #[doc(alias = "gst_query_parse_selectable")]
@@ -1639,8 +1939,39 @@ impl Selectable {
     }
 }
 
+#[cfg(feature = "v1_22")]
+impl std::fmt::Debug for Selectable {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Selectable")
+            .field("structure", &self.query().structure())
+            .field("selectable", &self.selectable())
+            .finish()
+    }
+}
+
+#[cfg(feature = "v1_22")]
+impl std::fmt::Debug for Selectable<Query> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        Selectable::<QueryRef>::fmt(self, f)
+    }
+}
+
 declare_concrete_query!(Other, T);
 
+impl std::fmt::Debug for Other {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Other")
+            .field("structure", &self.query().structure())
+            .finish()
+    }
+}
+
+impl std::fmt::Debug for Other<Query> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        Other::<QueryRef>::fmt(self, f)
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -1729,7 +2060,7 @@ mod tests {
     fn allocation_need_pool() {
         crate::init().unwrap();
 
-        let mut a = Allocation::new(&crate::Caps::builder("foo/bar").build(), true);
+        let mut a = Allocation::new(Some(&crate::Caps::new_empty_simple("foo/bar")), true);
         let pool = crate::BufferPool::new();
         a.add_allocation_pool(Some(&pool), 1024, 1, 4);
     }
@@ -1738,7 +2069,7 @@ mod tests {
     fn allocation_do_not_need_pool() {
         crate::init().unwrap();
 
-        let mut a = Allocation::new(&crate::Caps::builder("foo/bar").build(), false);
+        let mut a = Allocation::new(Some(&crate::Caps::new_empty_simple("foo/bar")), false);
         a.add_allocation_pool(crate::BufferPool::NONE, 1024, 1, 4);
 
         // cannot infer type of the type parameter `T` declared on the enum `Option`
