@@ -12,7 +12,7 @@ use std::{
 use glib::{
     prelude::*,
     translate::*,
-    value::{FromValue, SendValue, ToSendValue},
+    value::{FromValue, SendValue},
     IntoGStr,
 };
 
@@ -65,19 +65,6 @@ impl Structure {
         }
     }
 
-    #[doc(alias = "gst_structure_new")]
-    #[deprecated = "Use `Structure::builder()` or `Structure::new_empty()`"]
-    pub fn new(name: impl IntoGStr, values: &[(&str, &(dyn ToSendValue + Sync))]) -> Structure {
-        skip_assert_initialized!();
-        let mut structure = Structure::new_empty(name);
-
-        for &(f, v) in values {
-            structure.set_value(f, v.to_send_value());
-        }
-
-        structure
-    }
-
     #[allow(clippy::should_implement_trait)]
     pub fn from_iter(
         name: impl IntoGStr,
@@ -750,13 +737,34 @@ impl StructureRef {
         }
     }
 
-    #[cfg(any(feature = "v1_20", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
+    #[cfg(feature = "v1_20")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_20")))]
     #[doc(alias = "gst_structure_serialize")]
     pub fn serialize(&self, flags: crate::SerializeFlags) -> glib::GString {
         unsafe { from_glib_full(ffi::gst_structure_serialize(&self.0, flags.into_glib())) }
     }
 
+    #[cfg(feature = "v1_24")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_24")))]
+    #[doc(alias = "gst_structure_serialize")]
+    #[doc(alias = "gst_structure_serialize_full")]
+    pub fn serialize_strict(
+        &self,
+        flags: crate::SerializeFlags,
+    ) -> Result<glib::GString, glib::BoolError> {
+        unsafe {
+            let res = ffi::gst_structure_serialize_full(
+                &self.0,
+                flags.into_glib() | ffi::GST_SERIALIZE_FLAG_STRICT,
+            );
+            if res.is_null() {
+                Err(glib::bool_error!("Failed to serialize structure to string"))
+            } else {
+                Ok(from_glib_full(res))
+            }
+        }
+    }
+
     #[doc(alias = "gst_structure_foreach")]
     pub fn foreach<F: FnMut(glib::Quark, &glib::Value) -> std::ops::ControlFlow<()>>(
         &self,
@@ -1124,7 +1132,6 @@ mod tests {
     use super::*;
 
     #[test]
-    #[allow(deprecated)]
     fn new_set_get() {
         use glib::{value, Type};
 
@@ -1176,24 +1183,12 @@ mod tests {
         assert_eq!(v[2].0, "f3");
         assert_eq!(v[2].1.get::<i32>(), Ok(123i32));
 
-        let s2 = Structure::new("test", &[("f1", &"abc"), ("f2", &"bcd"), ("f3", &123i32)]);
-        assert_eq!(s, s2);
-    }
-
-    #[test]
-    fn test_builder() {
-        crate::init().unwrap();
-
-        let s = Structure::builder("test")
+        let s2 = Structure::builder("test")
             .field("f1", "abc")
             .field("f2", String::from("bcd"))
             .field("f3", 123i32)
             .build();
-
-        assert_eq!(s.name(), "test");
-        assert_eq!(s.get::<&str>("f1"), Ok("abc"));
-        assert_eq!(s.get::<&str>("f2"), Ok("bcd"));
-        assert_eq!(s.get::<i32>("f3"), Ok(123i32));
+        assert_eq!(s, s2);
     }
 
     #[test]
@@ -1211,7 +1206,7 @@ mod tests {
 
     #[test]
     fn test_from_value_optional() {
-        use glib::ToValue;
+        use glib::value::ToValue;
 
         crate::init().unwrap();
 
