@@ -1,13 +1,15 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use super::prelude::*;
+use std::ptr;
+
 use glib::{
+    prelude::*,
     subclass::{prelude::*, InitializingObject},
     translate::*,
-    Cast, StaticType,
 };
 use libc::c_char;
 
+use super::prelude::*;
 use crate::{BufferPool, BufferPoolAcquireParams, BufferPoolConfigRef};
 
 pub trait BufferPoolImpl: BufferPoolImplExt + GstObjectImpl + Send + Sync {
@@ -62,35 +64,12 @@ pub trait BufferPoolImpl: BufferPoolImplExt + GstObjectImpl + Send + Sync {
     }
 }
 
-pub trait BufferPoolImplExt: ObjectSubclass {
-    fn parent_acquire_buffer(
-        &self,
-        params: Option<&BufferPoolAcquireParams>,
-    ) -> Result<crate::Buffer, crate::FlowError>;
-
-    fn parent_alloc_buffer(
-        &self,
-        params: Option<&BufferPoolAcquireParams>,
-    ) -> Result<crate::Buffer, crate::FlowError>;
-
-    fn parent_free_buffer(&self, buffer: crate::Buffer);
-
-    fn parent_release_buffer(&self, buffer: crate::Buffer);
-
-    fn parent_reset_buffer(&self, buffer: &mut crate::BufferRef);
-
-    fn parent_start(&self) -> bool;
-
-    fn parent_stop(&self) -> bool;
-
-    fn parent_set_config(&self, config: &mut BufferPoolConfigRef) -> bool;
-
-    fn parent_flush_start(&self);
-
-    fn parent_flush_stop(&self);
+mod sealed {
+    pub trait Sealed {}
+    impl<T: super::BufferPoolImplExt> Sealed for T {}
 }
 
-impl<T: BufferPoolImpl> BufferPoolImplExt for T {
+pub trait BufferPoolImplExt: sealed::Sealed + ObjectSubclass {
     fn parent_acquire_buffer(
         &self,
         params: Option<&BufferPoolAcquireParams>,
@@ -278,16 +257,7 @@ impl<T: BufferPoolImpl> BufferPoolImplExt for T {
     }
 }
 
-// Send+Sync wrapper around a NULL-terminated C string array
-struct CStrV(*mut *const libc::c_char);
-unsafe impl Send for CStrV {}
-unsafe impl Sync for CStrV {}
-
-impl Drop for CStrV {
-    fn drop(&mut self) {
-        unsafe { glib::ffi::g_strfreev(self.0 as *mut _) };
-    }
-}
+impl<T: BufferPoolImpl> BufferPoolImplExt for T {}
 
 unsafe impl<T: BufferPoolImpl> IsSubclassable<T> for BufferPool {
     fn class_init(klass: &mut glib::Class<Self>) {
@@ -312,8 +282,7 @@ unsafe impl<T: BufferPoolImpl> IsSubclassable<T> for BufferPool {
         // Store the pool options in the instance data
         // for later retrieval in buffer_pool_get_options
         let options = T::options();
-        let options = options.to_glib_full();
-        instance.set_instance_data(T::type_(), CStrV(options));
+        instance.set_instance_data(T::type_(), glib::StrV::from(options));
     }
 }
 
@@ -468,9 +437,9 @@ unsafe extern "C" fn buffer_pool_get_options<T: BufferPoolImpl>(
 ) -> *mut *const c_char {
     let instance = &*(ptr as *mut T::Instance);
     let imp = instance.imp();
-    T::instance_data::<CStrV>(imp, T::type_())
-        .unwrap_or(&CStrV(std::ptr::null_mut()))
-        .0
+    T::instance_data::<glib::StrV>(imp, T::type_())
+        .map(|p| p.as_ptr() as *mut *const _)
+        .unwrap_or(ptr::null_mut())
 }
 
 unsafe extern "C" fn buffer_pool_set_config<T: BufferPoolImpl>(
@@ -527,7 +496,7 @@ mod tests {
 
     impl Default for TestBufferPool {
         fn default() -> Self {
-            glib::Object::new(&[])
+            glib::Object::new()
         }
     }
 
@@ -608,7 +577,7 @@ mod tests {
         data: *mut libc::c_void,
         _mini_object: *mut ffi::GstMiniObject,
     ) {
-        let finalized = &*(data as *const AtomicBool);
+        let finalized = Arc::from_raw(data as *const AtomicBool);
         finalized.store(true, Ordering::SeqCst);
     }
 }
