@@ -1,41 +1,77 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use num_rational::Rational32;
-use std::cmp;
-use std::fmt;
-use std::ops;
-use std::slice;
+use std::{cmp, fmt, ops, slice};
 
-use glib::translate::{from_glib, FromGlibPtrFull, ToGlibPtr, ToGlibPtrMut, Uninitialized};
-use glib::value::ToSendValue;
-use glib::StaticType;
+use glib::{prelude::*, translate::*};
+use num_rational::Rational32;
 
 #[derive(Copy, Clone, Debug, Ord, PartialOrd, Eq, PartialEq, Hash)]
 pub struct Fraction(pub Rational32);
 
 impl Fraction {
-    pub fn new(num: i32, den: i32) -> Self {
-        assert_initialized_main_thread!();
-        (num, den).into()
+    // rustdoc-stripper-ignore-next
+    /// Creates a new `Ratio`.
+    ///
+    /// # Panics
+    ///
+    /// Panics if `denom` is zero.
+    #[inline]
+    pub fn new(numer: i32, denom: i32) -> Self {
+        skip_assert_initialized!();
+        (numer, denom).into()
+    }
+
+    // rustdoc-stripper-ignore-next
+    /// Creates a `Fraction` without checking for `denom == 0` or reducing.
+    ///
+    /// While this does not panic, there are several methods that will panic
+    /// if used on a `Fraction` with `denom == 0`.
+    #[inline]
+    pub const fn new_raw(numer: i32, denom: i32) -> Self {
+        skip_assert_initialized!();
+        Self(Rational32::new_raw(numer, denom))
+    }
+
+    // rustdoc-stripper-ignore-next
+    /// Creates a `Fraction` representing the integer `t`.
+    #[inline]
+    pub const fn from_integer(t: i32) -> Self {
+        skip_assert_initialized!();
+        Self::new_raw(t, 1)
     }
 
     pub fn approximate_f32(x: f32) -> Option<Self> {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
         Rational32::approximate_float(x).map(|r| r.into())
     }
 
     pub fn approximate_f64(x: f64) -> Option<Self> {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
         Rational32::approximate_float(x).map(|r| r.into())
     }
 
+    #[inline]
     pub fn numer(&self) -> i32 {
         *self.0.numer()
     }
 
+    #[inline]
     pub fn denom(&self) -> i32 {
         *self.0.denom()
     }
+
+    #[cfg(feature = "v1_24")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_24")))]
+    #[doc(alias = "gst_util_simplify_fraction")]
+    pub fn simplify(&mut self, n_terms: u32, threshold: u32) {
+        skip_assert_initialized!();
+        unsafe {
+            let mut numer = self.numer();
+            let mut denom = self.denom();
+            ffi::gst_util_simplify_fraction(&mut numer, &mut denom, n_terms, threshold);
+            *self = Self::new(numer, denom);
+        }
+    }
 }
 
 impl fmt::Display for Fraction {
@@ -47,18 +83,21 @@ impl fmt::Display for Fraction {
 impl ops::Deref for Fraction {
     type Target = Rational32;
 
+    #[inline]
     fn deref(&self) -> &Self::Target {
         &self.0
     }
 }
 
 impl ops::DerefMut for Fraction {
+    #[inline]
     fn deref_mut(&mut self) -> &mut Rational32 {
         &mut self.0
     }
 }
 
 impl AsRef<Rational32> for Fraction {
+    #[inline]
     fn as_ref(&self) -> &Rational32 {
         &self.0
     }
@@ -69,6 +108,7 @@ macro_rules! impl_fraction_binop {
         impl ops::$name<Fraction> for Fraction {
             type Output = Fraction;
 
+            #[inline]
             fn $f(self, other: Fraction) -> Self::Output {
                 Fraction((self.0).$f(other.0))
             }
@@ -77,6 +117,7 @@ macro_rules! impl_fraction_binop {
         impl ops::$name<Fraction> for &Fraction {
             type Output = Fraction;
 
+            #[inline]
             fn $f(self, other: Fraction) -> Self::Output {
                 Fraction((self.0).$f(other.0))
             }
@@ -85,6 +126,7 @@ macro_rules! impl_fraction_binop {
         impl ops::$name<&Fraction> for Fraction {
             type Output = Fraction;
 
+            #[inline]
             fn $f(self, other: &Fraction) -> Self::Output {
                 Fraction((self.0).$f(other.0))
             }
@@ -93,6 +135,7 @@ macro_rules! impl_fraction_binop {
         impl ops::$name<&Fraction> for &Fraction {
             type Output = Fraction;
 
+            #[inline]
             fn $f(self, other: &Fraction) -> Self::Output {
                 Fraction((self.0).$f(other.0))
             }
@@ -101,6 +144,7 @@ macro_rules! impl_fraction_binop {
         impl ops::$name<i32> for Fraction {
             type Output = Fraction;
 
+            #[inline]
             fn $f(self, other: i32) -> Self::Output {
                 self.$f(Fraction::from(other))
             }
@@ -109,6 +153,7 @@ macro_rules! impl_fraction_binop {
         impl ops::$name<i32> for &Fraction {
             type Output = Fraction;
 
+            #[inline]
             fn $f(self, other: i32) -> Self::Output {
                 self.$f(Fraction::from(other))
             }
@@ -117,6 +162,7 @@ macro_rules! impl_fraction_binop {
         impl ops::$name<&i32> for Fraction {
             type Output = Fraction;
 
+            #[inline]
             fn $f(self, other: &i32) -> Self::Output {
                 self.$f(Fraction::from(*other))
             }
@@ -125,6 +171,7 @@ macro_rules! impl_fraction_binop {
         impl ops::$name<&i32> for &Fraction {
             type Output = Fraction;
 
+            #[inline]
             fn $f(self, other: &i32) -> Self::Output {
                 self.$f(Fraction::from(*other))
             }
@@ -133,6 +180,7 @@ macro_rules! impl_fraction_binop {
         impl ops::$name<Fraction> for i32 {
             type Output = Fraction;
 
+            #[inline]
             fn $f(self, other: Fraction) -> Self::Output {
                 Fraction::from(self).$f(other)
             }
@@ -141,6 +189,7 @@ macro_rules! impl_fraction_binop {
         impl ops::$name<&Fraction> for i32 {
             type Output = Fraction;
 
+            #[inline]
             fn $f(self, other: &Fraction) -> Self::Output {
                 Fraction::from(self).$f(other)
             }
@@ -149,6 +198,7 @@ macro_rules! impl_fraction_binop {
         impl ops::$name<Fraction> for &i32 {
             type Output = Fraction;
 
+            #[inline]
             fn $f(self, other: Fraction) -> Self::Output {
                 Fraction::from(*self).$f(other)
             }
@@ -157,30 +207,35 @@ macro_rules! impl_fraction_binop {
         impl ops::$name<&Fraction> for &i32 {
             type Output = Fraction;
 
+            #[inline]
             fn $f(self, other: &Fraction) -> Self::Output {
                 Fraction::from(*self).$f(other)
             }
         }
 
         impl ops::$name_assign<Fraction> for Fraction {
+            #[inline]
             fn $f_assign(&mut self, other: Fraction) {
                 (self.0).$f_assign(other.0)
             }
         }
 
         impl ops::$name_assign<&Fraction> for Fraction {
+            #[inline]
             fn $f_assign(&mut self, other: &Fraction) {
                 (self.0).$f_assign(other.0)
             }
         }
 
         impl ops::$name_assign<i32> for Fraction {
+            #[inline]
             fn $f_assign(&mut self, other: i32) {
                 (self.0).$f_assign(other)
             }
         }
 
         impl ops::$name_assign<&i32> for Fraction {
+            #[inline]
             fn $f_assign(&mut self, other: &i32) {
                 (self.0).$f_assign(other)
             }
@@ -197,6 +252,7 @@ impl_fraction_binop!(Rem, rem, RemAssign, rem_assign);
 impl ops::Neg for Fraction {
     type Output = Fraction;
 
+    #[inline]
     fn neg(self) -> Self::Output {
         Fraction(self.0.neg())
     }
@@ -205,26 +261,30 @@ impl ops::Neg for Fraction {
 impl ops::Neg for &Fraction {
     type Output = Fraction;
 
+    #[inline]
     fn neg(self) -> Self::Output {
         Fraction(self.0.neg())
     }
 }
 
 impl From<i32> for Fraction {
+    #[inline]
     fn from(x: i32) -> Self {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
         Fraction(x.into())
     }
 }
 
 impl From<(i32, i32)> for Fraction {
+    #[inline]
     fn from(x: (i32, i32)) -> Self {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
         Fraction(x.into())
     }
 }
 
 impl From<Fraction> for (i32, i32) {
+    #[inline]
     fn from(f: Fraction) -> Self {
         skip_assert_initialized!();
         f.0.into()
@@ -232,13 +292,15 @@ impl From<Fraction> for (i32, i32) {
 }
 
 impl From<Rational32> for Fraction {
+    #[inline]
     fn from(x: Rational32) -> Self {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
         Fraction(x)
     }
 }
 
 impl From<Fraction> for Rational32 {
+    #[inline]
     fn from(x: Fraction) -> Self {
         skip_assert_initialized!();
         x.0
@@ -246,6 +308,7 @@ impl From<Fraction> for Rational32 {
 }
 
 impl glib::types::StaticType for Fraction {
+    #[inline]
     fn static_type() -> glib::types::Type {
         unsafe { from_glib(ffi::gst_fraction_get_type()) }
     }
@@ -258,6 +321,7 @@ impl glib::value::ValueType for Fraction {
 unsafe impl<'a> glib::value::FromValue<'a> for Fraction {
     type Checker = glib::value::GenericValueTypeChecker<Self>;
 
+    #[inline]
     unsafe fn from_value(value: &'a glib::Value) -> Self {
         skip_assert_initialized!();
         let n = ffi::gst_value_get_fraction_numerator(value.to_glib_none().0);
@@ -268,6 +332,7 @@ unsafe impl<'a> glib::value::FromValue<'a> for Fraction {
 }
 
 impl glib::value::ToValue for Fraction {
+    #[inline]
     fn to_value(&self) -> glib::Value {
         let mut value = glib::Value::for_value_type::<Self>();
         unsafe {
@@ -276,11 +341,20 @@ impl glib::value::ToValue for Fraction {
         value
     }
 
+    #[inline]
     fn value_type(&self) -> glib::Type {
         Self::static_type()
     }
 }
 
+impl From<Fraction> for glib::Value {
+    #[inline]
+    fn from(v: Fraction) -> glib::Value {
+        skip_assert_initialized!();
+        glib::value::ToValue::to_value(&v)
+    }
+}
+
 #[derive(Copy, Clone, Debug, Eq, PartialEq, Hash)]
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub struct IntRange<T> {
@@ -290,14 +364,17 @@ pub struct IntRange<T> {
 }
 
 impl<T: Copy> IntRange<T> {
+    #[inline]
     pub fn min(&self) -> T {
         self.min
     }
 
+    #[inline]
     pub fn max(&self) -> T {
         self.max
     }
 
+    #[inline]
     pub fn step(&self) -> T {
         self.step
     }
@@ -310,13 +387,15 @@ pub trait IntRangeType: Sized + Clone + Copy + 'static {
 }
 
 impl IntRangeType for i32 {
+    #[inline]
     fn with_min_max(min: i32, max: i32) -> IntRange<Self> {
         skip_assert_initialized!();
         IntRange { min, max, step: 1 }
     }
 
+    #[inline]
     fn with_step(min: i32, max: i32, step: i32) -> IntRange<Self> {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
 
         assert!(min <= max);
         assert!(step > 0);
@@ -326,13 +405,15 @@ impl IntRangeType for i32 {
 }
 
 impl IntRangeType for i64 {
+    #[inline]
     fn with_min_max(min: i64, max: i64) -> IntRange<Self> {
         skip_assert_initialized!();
         IntRange { min, max, step: 1 }
     }
 
+    #[inline]
     fn with_step(min: i64, max: i64, step: i64) -> IntRange<Self> {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
 
         assert!(min <= max);
         assert!(step > 0);
@@ -342,18 +423,21 @@ impl IntRangeType for i64 {
 }
 
 impl<T: IntRangeType> IntRange<T> {
+    #[inline]
     pub fn new(min: T, max: T) -> IntRange<T> {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
         T::with_min_max(min, max)
     }
 
+    #[inline]
     pub fn with_step(min: T, max: T, step: T) -> IntRange<T> {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
         T::with_step(min, max, step)
     }
 }
 
 impl From<(i32, i32)> for IntRange<i32> {
+    #[inline]
     fn from((min, max): (i32, i32)) -> Self {
         skip_assert_initialized!();
         Self::new(min, max)
@@ -361,6 +445,7 @@ impl From<(i32, i32)> for IntRange<i32> {
 }
 
 impl From<(i32, i32, i32)> for IntRange<i32> {
+    #[inline]
     fn from((min, max, step): (i32, i32, i32)) -> Self {
         skip_assert_initialized!();
         Self::with_step(min, max, step)
@@ -368,6 +453,7 @@ impl From<(i32, i32, i32)> for IntRange<i32> {
 }
 
 impl From<(i64, i64)> for IntRange<i64> {
+    #[inline]
     fn from((min, max): (i64, i64)) -> Self {
         skip_assert_initialized!();
         Self::new(min, max)
@@ -375,6 +461,7 @@ impl From<(i64, i64)> for IntRange<i64> {
 }
 
 impl From<(i64, i64, i64)> for IntRange<i64> {
+    #[inline]
     fn from((min, max, step): (i64, i64, i64)) -> Self {
         skip_assert_initialized!();
         Self::with_step(min, max, step)
@@ -382,6 +469,7 @@ impl From<(i64, i64, i64)> for IntRange<i64> {
 }
 
 impl glib::types::StaticType for IntRange<i32> {
+    #[inline]
     fn static_type() -> glib::types::Type {
         unsafe { from_glib(ffi::gst_int_range_get_type()) }
     }
@@ -394,6 +482,7 @@ impl glib::value::ValueType for IntRange<i32> {
 unsafe impl<'a> glib::value::FromValue<'a> for IntRange<i32> {
     type Checker = glib::value::GenericValueTypeChecker<Self>;
 
+    #[inline]
     unsafe fn from_value(value: &'a glib::Value) -> Self {
         skip_assert_initialized!();
         let min = ffi::gst_value_get_int_range_min(value.to_glib_none().0);
@@ -405,6 +494,7 @@ unsafe impl<'a> glib::value::FromValue<'a> for IntRange<i32> {
 }
 
 impl glib::value::ToValue for IntRange<i32> {
+    #[inline]
     fn to_value(&self) -> glib::Value {
         let mut value = glib::Value::for_value_type::<Self>();
         unsafe {
@@ -418,12 +508,22 @@ impl glib::value::ToValue for IntRange<i32> {
         value
     }
 
+    #[inline]
     fn value_type(&self) -> glib::Type {
         Self::static_type()
     }
 }
 
+impl From<IntRange<i32>> for glib::Value {
+    #[inline]
+    fn from(v: IntRange<i32>) -> glib::Value {
+        skip_assert_initialized!();
+        glib::value::ToValue::to_value(&v)
+    }
+}
+
 impl glib::types::StaticType for IntRange<i64> {
+    #[inline]
     fn static_type() -> glib::types::Type {
         unsafe { from_glib(ffi::gst_int64_range_get_type()) }
     }
@@ -436,6 +536,7 @@ impl glib::value::ValueType for IntRange<i64> {
 unsafe impl<'a> glib::value::FromValue<'a> for IntRange<i64> {
     type Checker = glib::value::GenericValueTypeChecker<Self>;
 
+    #[inline]
     unsafe fn from_value(value: &'a glib::Value) -> Self {
         skip_assert_initialized!();
         let min = ffi::gst_value_get_int64_range_min(value.to_glib_none().0);
@@ -447,6 +548,7 @@ unsafe impl<'a> glib::value::FromValue<'a> for IntRange<i64> {
 }
 
 impl glib::value::ToValue for IntRange<i64> {
+    #[inline]
     fn to_value(&self) -> glib::Value {
         let mut value = glib::Value::for_value_type::<Self>();
         unsafe {
@@ -460,11 +562,20 @@ impl glib::value::ToValue for IntRange<i64> {
         value
     }
 
+    #[inline]
     fn value_type(&self) -> glib::Type {
         Self::static_type()
     }
 }
 
+impl From<IntRange<i64>> for glib::Value {
+    #[inline]
+    fn from(v: IntRange<i64>) -> glib::Value {
+        skip_assert_initialized!();
+        glib::value::ToValue::to_value(&v)
+    }
+}
+
 #[derive(Copy, Clone, Debug, Eq, PartialEq, Hash)]
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub struct FractionRange {
@@ -473,8 +584,9 @@ pub struct FractionRange {
 }
 
 impl FractionRange {
+    #[inline]
     pub fn new<T: Into<Fraction>, U: Into<Fraction>>(min: T, max: U) -> Self {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
 
         let min = min.into();
         let max = max.into();
@@ -484,16 +596,19 @@ impl FractionRange {
         FractionRange { min, max }
     }
 
+    #[inline]
     pub fn min(&self) -> Fraction {
         self.min
     }
 
+    #[inline]
     pub fn max(&self) -> Fraction {
         self.max
     }
 }
 
 impl From<(Fraction, Fraction)> for FractionRange {
+    #[inline]
     fn from((min, max): (Fraction, Fraction)) -> Self {
         skip_assert_initialized!();
 
@@ -502,6 +617,7 @@ impl From<(Fraction, Fraction)> for FractionRange {
 }
 
 impl glib::types::StaticType for FractionRange {
+    #[inline]
     fn static_type() -> glib::types::Type {
         unsafe { from_glib(ffi::gst_fraction_range_get_type()) }
     }
@@ -514,6 +630,7 @@ impl glib::value::ValueType for FractionRange {
 unsafe impl<'a> glib::value::FromValue<'a> for FractionRange {
     type Checker = glib::value::GenericValueTypeChecker<Self>;
 
+    #[inline]
     unsafe fn from_value(value: &'a glib::Value) -> Self {
         skip_assert_initialized!();
         let min = ffi::gst_value_get_fraction_range_min(value.to_glib_none().0);
@@ -529,6 +646,7 @@ unsafe impl<'a> glib::value::FromValue<'a> for FractionRange {
 }
 
 impl glib::value::ToValue for FractionRange {
+    #[inline]
     fn to_value(&self) -> glib::Value {
         let mut value = glib::Value::for_value_type::<Self>();
         unsafe {
@@ -543,18 +661,28 @@ impl glib::value::ToValue for FractionRange {
         value
     }
 
+    #[inline]
     fn value_type(&self) -> glib::Type {
         Self::static_type()
     }
 }
 
+impl From<FractionRange> for glib::Value {
+    #[inline]
+    fn from(v: FractionRange) -> glib::Value {
+        skip_assert_initialized!();
+        glib::value::ToValue::to_value(&v)
+    }
+}
+
 #[derive(Copy, Clone, Debug, Eq, PartialEq, Hash)]
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub struct Bitmask(pub u64);
 
 impl Bitmask {
+    #[inline]
     pub fn new(v: u64) -> Self {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
         Bitmask(v)
     }
 }
@@ -562,12 +690,14 @@ impl Bitmask {
 impl ops::Deref for Bitmask {
     type Target = u64;
 
+    #[inline]
     fn deref(&self) -> &u64 {
         &self.0
     }
 }
 
 impl ops::DerefMut for Bitmask {
+    #[inline]
     fn deref_mut(&mut self) -> &mut u64 {
         &mut self.0
     }
@@ -576,6 +706,7 @@ impl ops::DerefMut for Bitmask {
 impl ops::BitAnd for Bitmask {
     type Output = Self;
 
+    #[inline]
     fn bitand(self, rhs: Self) -> Self {
         Bitmask(self.0.bitand(rhs.0))
     }
@@ -584,6 +715,7 @@ impl ops::BitAnd for Bitmask {
 impl ops::BitOr for Bitmask {
     type Output = Self;
 
+    #[inline]
     fn bitor(self, rhs: Self) -> Self {
         Bitmask(self.0.bitor(rhs.0))
     }
@@ -592,6 +724,7 @@ impl ops::BitOr for Bitmask {
 impl ops::BitXor for Bitmask {
     type Output = Self;
 
+    #[inline]
     fn bitxor(self, rhs: Self) -> Self {
         Bitmask(self.0.bitxor(rhs.0))
     }
@@ -600,12 +733,14 @@ impl ops::BitXor for Bitmask {
 impl ops::Not for Bitmask {
     type Output = Self;
 
+    #[inline]
     fn not(self) -> Self {
         Bitmask(self.0.not())
     }
 }
 
 impl From<u64> for Bitmask {
+    #[inline]
     fn from(v: u64) -> Self {
         skip_assert_initialized!();
         Self::new(v)
@@ -613,6 +748,7 @@ impl From<u64> for Bitmask {
 }
 
 impl glib::types::StaticType for Bitmask {
+    #[inline]
     fn static_type() -> glib::types::Type {
         unsafe { from_glib(ffi::gst_bitmask_get_type()) }
     }
@@ -625,6 +761,7 @@ impl glib::value::ValueType for Bitmask {
 unsafe impl<'a> glib::value::FromValue<'a> for Bitmask {
     type Checker = glib::value::GenericValueTypeChecker<Self>;
 
+    #[inline]
     unsafe fn from_value(value: &'a glib::Value) -> Self {
         skip_assert_initialized!();
         let v = ffi::gst_value_get_bitmask(value.to_glib_none().0);
@@ -633,6 +770,7 @@ unsafe impl<'a> glib::value::FromValue<'a> for Bitmask {
 }
 
 impl glib::value::ToValue for Bitmask {
+    #[inline]
     fn to_value(&self) -> glib::Value {
         let mut value = glib::Value::for_value_type::<Self>();
         unsafe {
@@ -641,44 +779,103 @@ impl glib::value::ToValue for Bitmask {
         value
     }
 
+    #[inline]
     fn value_type(&self) -> glib::Type {
         Self::static_type()
     }
 }
 
-#[derive(Clone, Debug)]
-pub struct Array(Vec<glib::SendValue>);
+impl From<Bitmask> for glib::Value {
+    #[inline]
+    fn from(v: Bitmask) -> glib::Value {
+        skip_assert_initialized!();
+        glib::value::ToValue::to_value(&v)
+    }
+}
+
+#[derive(Clone)]
+pub struct Array(glib::SendValue);
 
 unsafe impl Send for Array {}
 unsafe impl Sync for Array {}
 
+impl fmt::Debug for Array {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_tuple("Array").field(&self.as_slice()).finish()
+    }
+}
+
 impl Array {
-    pub fn new(values: impl IntoIterator<Item = impl ToSendValue + Send>) -> Self {
+    pub fn new(values: impl IntoIterator<Item = impl Into<glib::Value> + Send>) -> Self {
         assert_initialized_main_thread!();
 
-        Self(values.into_iter().map(|v| v.to_send_value()).collect())
+        unsafe {
+            let mut value = glib::Value::for_value_type::<Array>();
+            for v in values.into_iter() {
+                let mut v = v.into().into_raw();
+                ffi::gst_value_array_append_and_take_value(value.to_glib_none_mut().0, &mut v);
+            }
+
+            Self(glib::SendValue::unsafe_from(value.into_raw()))
+        }
     }
 
     pub fn from_values(values: impl IntoIterator<Item = glib::SendValue>) -> Self {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
 
-        Self(values.into_iter().collect())
+        Self::new(values)
     }
 
+    #[inline]
     pub fn as_slice(&self) -> &[glib::SendValue] {
-        self.0.as_slice()
+        unsafe {
+            let arr = (*self.0.as_ptr()).data[0].v_pointer as *const glib::ffi::GArray;
+            if arr.is_null() || (*arr).len == 0 {
+                &[]
+            } else {
+                #[allow(clippy::cast_ptr_alignment)]
+                slice::from_raw_parts((*arr).data as *const glib::SendValue, (*arr).len as usize)
+            }
+        }
+    }
+
+    pub fn append_value(&mut self, value: glib::SendValue) {
+        unsafe {
+            ffi::gst_value_array_append_and_take_value(
+                self.0.to_glib_none_mut().0,
+                &mut value.into_raw(),
+            );
+        }
+    }
+
+    pub fn append(&mut self, value: impl Into<glib::Value> + Send) {
+        self.append_value(glib::SendValue::from_owned(value));
+    }
+}
+
+impl Default for Array {
+    fn default() -> Self {
+        skip_assert_initialized!();
+
+        unsafe {
+            let value = glib::Value::for_value_type::<Array>();
+
+            Self(glib::SendValue::unsafe_from(value.into_raw()))
+        }
     }
 }
 
 impl ops::Deref for Array {
     type Target = [glib::SendValue];
 
+    #[inline]
     fn deref(&self) -> &[glib::SendValue] {
         self.as_slice()
     }
 }
 
 impl AsRef<[glib::SendValue]> for Array {
+    #[inline]
     fn as_ref(&self) -> &[glib::SendValue] {
         self.as_slice()
     }
@@ -686,16 +883,16 @@ impl AsRef<[glib::SendValue]> for Array {
 
 impl std::iter::FromIterator<glib::SendValue> for Array {
     fn from_iter<T: IntoIterator<Item = glib::SendValue>>(iter: T) -> Self {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
         Self::from_values(iter)
     }
 }
 
-impl From<Vec<glib::SendValue>> for Array {
-    fn from(values: Vec<glib::SendValue>) -> Self {
-        assert_initialized_main_thread!();
-
-        Self(values)
+impl std::iter::Extend<glib::SendValue> for Array {
+    fn extend<T: IntoIterator<Item = glib::SendValue>>(&mut self, iter: T) {
+        for v in iter.into_iter() {
+            self.append_value(v);
+        }
     }
 }
 
@@ -708,29 +905,13 @@ unsafe impl<'a> glib::value::FromValue<'a> for Array {
 
     unsafe fn from_value(value: &'a glib::Value) -> Self {
         skip_assert_initialized!();
-        let arr = (*value.to_glib_none().0).data[0].v_pointer as *const glib::ffi::GArray;
-        if arr.is_null() || (*arr).len == 0 {
-            Self(Vec::new())
-        } else {
-            #[allow(clippy::cast_ptr_alignment)]
-            Self::from_values(
-                slice::from_raw_parts((*arr).data as *const glib::SendValue, (*arr).len as usize)
-                    .iter()
-                    .cloned(),
-            )
-        }
+        Self(glib::SendValue::unsafe_from(value.clone().into_raw()))
     }
 }
 
 impl glib::value::ToValue for Array {
     fn to_value(&self) -> glib::Value {
-        let mut value = glib::Value::for_value_type::<Array>();
-        unsafe {
-            for v in self.as_slice() {
-                ffi::gst_value_array_append_value(value.to_glib_none_mut().0, v.to_glib_none().0);
-            }
-        }
-        value
+        self.0.clone().into()
     }
 
     fn value_type(&self) -> glib::Type {
@@ -738,7 +919,15 @@ impl glib::value::ToValue for Array {
     }
 }
 
+impl From<Array> for glib::Value {
+    fn from(v: Array) -> glib::Value {
+        skip_assert_initialized!();
+        v.0.into()
+    }
+}
+
 impl glib::types::StaticType for Array {
+    #[inline]
     fn static_type() -> glib::types::Type {
         unsafe { from_glib(ffi::gst_value_array_get_type()) }
     }
@@ -752,11 +941,12 @@ unsafe impl<'a> Sync for ArrayRef<'a> {}
 
 impl<'a> ArrayRef<'a> {
     pub fn new(values: &'a [glib::SendValue]) -> Self {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
 
         Self(values)
     }
 
+    #[inline]
     pub fn as_slice(&self) -> &'a [glib::SendValue] {
         self.0
     }
@@ -765,12 +955,14 @@ impl<'a> ArrayRef<'a> {
 impl<'a> ops::Deref for ArrayRef<'a> {
     type Target = [glib::SendValue];
 
+    #[inline]
     fn deref(&self) -> &[glib::SendValue] {
         self.as_slice()
     }
 }
 
 impl<'a> AsRef<[glib::SendValue]> for ArrayRef<'a> {
+    #[inline]
     fn as_ref(&self) -> &[glib::SendValue] {
         self.as_slice()
     }
@@ -779,9 +971,10 @@ impl<'a> AsRef<[glib::SendValue]> for ArrayRef<'a> {
 unsafe impl<'a> glib::value::FromValue<'a> for ArrayRef<'a> {
     type Checker = glib::value::GenericValueTypeChecker<Self>;
 
+    #[inline]
     unsafe fn from_value(value: &'a glib::Value) -> Self {
         skip_assert_initialized!();
-        let arr = (*value.to_glib_none().0).data[0].v_pointer as *const glib::ffi::GArray;
+        let arr = (*value.as_ptr()).data[0].v_pointer as *const glib::ffi::GArray;
         if arr.is_null() || (*arr).len == 0 {
             Self(&[])
         } else {
@@ -795,6 +988,7 @@ unsafe impl<'a> glib::value::FromValue<'a> for ArrayRef<'a> {
 }
 
 impl<'a> glib::value::ToValue for ArrayRef<'a> {
+    #[inline]
     fn to_value(&self) -> glib::Value {
         let mut value = glib::Value::for_value_type::<Array>();
         unsafe {
@@ -805,50 +999,110 @@ impl<'a> glib::value::ToValue for ArrayRef<'a> {
         value
     }
 
+    #[inline]
     fn value_type(&self) -> glib::Type {
         Self::static_type()
     }
 }
 
+impl<'a> From<ArrayRef<'a>> for glib::Value {
+    #[inline]
+    fn from(v: ArrayRef<'a>) -> glib::Value {
+        skip_assert_initialized!();
+        glib::value::ToValue::to_value(&v)
+    }
+}
+
 impl<'a> glib::types::StaticType for ArrayRef<'a> {
+    #[inline]
     fn static_type() -> glib::types::Type {
         unsafe { from_glib(ffi::gst_value_array_get_type()) }
     }
 }
 
-#[derive(Clone, Debug)]
-pub struct List(Vec<glib::SendValue>);
+#[derive(Clone)]
+pub struct List(glib::SendValue);
 
 unsafe impl Send for List {}
 unsafe impl Sync for List {}
 
+impl fmt::Debug for List {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_tuple("List").field(&self.as_slice()).finish()
+    }
+}
+
 impl List {
-    pub fn new(values: impl IntoIterator<Item = impl ToSendValue + Send>) -> Self {
+    pub fn new(values: impl IntoIterator<Item = impl Into<glib::Value> + Send>) -> Self {
         assert_initialized_main_thread!();
 
-        Self(values.into_iter().map(|v| v.to_send_value()).collect())
+        unsafe {
+            let mut value = glib::Value::for_value_type::<List>();
+            for v in values.into_iter() {
+                let mut v = v.into().into_raw();
+                ffi::gst_value_list_append_and_take_value(value.to_glib_none_mut().0, &mut v);
+            }
+
+            Self(glib::SendValue::unsafe_from(value.into_raw()))
+        }
     }
 
     pub fn from_values(values: impl IntoIterator<Item = glib::SendValue>) -> Self {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
 
-        Self(values.into_iter().collect())
+        Self::new(values)
     }
 
+    #[inline]
     pub fn as_slice(&self) -> &[glib::SendValue] {
-        self.0.as_slice()
+        unsafe {
+            let arr = (*self.0.as_ptr()).data[0].v_pointer as *const glib::ffi::GArray;
+            if arr.is_null() || (*arr).len == 0 {
+                &[]
+            } else {
+                #[allow(clippy::cast_ptr_alignment)]
+                slice::from_raw_parts((*arr).data as *const glib::SendValue, (*arr).len as usize)
+            }
+        }
+    }
+
+    pub fn append_value(&mut self, value: glib::SendValue) {
+        unsafe {
+            ffi::gst_value_list_append_and_take_value(
+                self.0.to_glib_none_mut().0,
+                &mut value.into_raw(),
+            );
+        }
+    }
+
+    pub fn append(&mut self, value: impl Into<glib::Value> + Send) {
+        self.append_value(glib::SendValue::from_owned(value));
+    }
+}
+
+impl Default for List {
+    fn default() -> Self {
+        skip_assert_initialized!();
+
+        unsafe {
+            let value = glib::Value::for_value_type::<List>();
+
+            Self(glib::SendValue::unsafe_from(value.into_raw()))
+        }
     }
 }
 
 impl ops::Deref for List {
     type Target = [glib::SendValue];
 
+    #[inline]
     fn deref(&self) -> &[glib::SendValue] {
         self.as_slice()
     }
 }
 
 impl AsRef<[glib::SendValue]> for List {
+    #[inline]
     fn as_ref(&self) -> &[glib::SendValue] {
         self.as_slice()
     }
@@ -856,16 +1110,16 @@ impl AsRef<[glib::SendValue]> for List {
 
 impl std::iter::FromIterator<glib::SendValue> for List {
     fn from_iter<T: IntoIterator<Item = glib::SendValue>>(iter: T) -> Self {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
         Self::from_values(iter)
     }
 }
 
-impl From<Vec<glib::SendValue>> for List {
-    fn from(values: Vec<glib::SendValue>) -> Self {
-        assert_initialized_main_thread!();
-
-        Self(values)
+impl std::iter::Extend<glib::SendValue> for List {
+    fn extend<T: IntoIterator<Item = glib::SendValue>>(&mut self, iter: T) {
+        for v in iter.into_iter() {
+            self.append_value(v);
+        }
     }
 }
 
@@ -878,29 +1132,13 @@ unsafe impl<'a> glib::value::FromValue<'a> for List {
 
     unsafe fn from_value(value: &'a glib::Value) -> Self {
         skip_assert_initialized!();
-        let arr = (*value.to_glib_none().0).data[0].v_pointer as *const glib::ffi::GArray;
-        if arr.is_null() || (*arr).len == 0 {
-            Self(Vec::new())
-        } else {
-            #[allow(clippy::cast_ptr_alignment)]
-            Self::from_values(
-                slice::from_raw_parts((*arr).data as *const glib::SendValue, (*arr).len as usize)
-                    .iter()
-                    .cloned(),
-            )
-        }
+        Self(glib::SendValue::unsafe_from(value.clone().into_raw()))
     }
 }
 
 impl glib::value::ToValue for List {
     fn to_value(&self) -> glib::Value {
-        let mut value = glib::Value::for_value_type::<List>();
-        unsafe {
-            for v in self.as_slice() {
-                ffi::gst_value_list_append_value(value.to_glib_none_mut().0, v.to_glib_none().0);
-            }
-        }
-        value
+        self.0.clone().into()
     }
 
     fn value_type(&self) -> glib::Type {
@@ -908,7 +1146,15 @@ impl glib::value::ToValue for List {
     }
 }
 
+impl From<List> for glib::Value {
+    fn from(v: List) -> glib::Value {
+        skip_assert_initialized!();
+        v.0.into()
+    }
+}
+
 impl glib::types::StaticType for List {
+    #[inline]
     fn static_type() -> glib::types::Type {
         unsafe { from_glib(ffi::gst_value_list_get_type()) }
     }
@@ -922,11 +1168,12 @@ unsafe impl<'a> Sync for ListRef<'a> {}
 
 impl<'a> ListRef<'a> {
     pub fn new(values: &'a [glib::SendValue]) -> Self {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
 
         Self(values)
     }
 
+    #[inline]
     pub fn as_slice(&self) -> &'a [glib::SendValue] {
         self.0
     }
@@ -935,12 +1182,14 @@ impl<'a> ListRef<'a> {
 impl<'a> ops::Deref for ListRef<'a> {
     type Target = [glib::SendValue];
 
+    #[inline]
     fn deref(&self) -> &[glib::SendValue] {
         self.as_slice()
     }
 }
 
 impl<'a> AsRef<[glib::SendValue]> for ListRef<'a> {
+    #[inline]
     fn as_ref(&self) -> &[glib::SendValue] {
         self.as_slice()
     }
@@ -949,9 +1198,10 @@ impl<'a> AsRef<[glib::SendValue]> for ListRef<'a> {
 unsafe impl<'a> glib::value::FromValue<'a> for ListRef<'a> {
     type Checker = glib::value::GenericValueTypeChecker<Self>;
 
+    #[inline]
     unsafe fn from_value(value: &'a glib::Value) -> Self {
         skip_assert_initialized!();
-        let arr = (*value.to_glib_none().0).data[0].v_pointer as *const glib::ffi::GArray;
+        let arr = (*value.as_ptr()).data[0].v_pointer as *const glib::ffi::GArray;
         if arr.is_null() || (*arr).len == 0 {
             Self(&[])
         } else {
@@ -965,6 +1215,7 @@ unsafe impl<'a> glib::value::FromValue<'a> for ListRef<'a> {
 }
 
 impl<'a> glib::value::ToValue for ListRef<'a> {
+    #[inline]
     fn to_value(&self) -> glib::Value {
         let mut value = glib::Value::for_value_type::<List>();
         unsafe {
@@ -975,12 +1226,22 @@ impl<'a> glib::value::ToValue for ListRef<'a> {
         value
     }
 
+    #[inline]
     fn value_type(&self) -> glib::Type {
         Self::static_type()
     }
 }
 
+impl<'a> From<ListRef<'a>> for glib::Value {
+    #[inline]
+    fn from(v: ListRef<'a>) -> glib::Value {
+        skip_assert_initialized!();
+        glib::value::ToValue::to_value(&v)
+    }
+}
+
 impl<'a> glib::types::StaticType for ListRef<'a> {
+    #[inline]
     fn static_type() -> glib::types::Type {
         unsafe { from_glib(ffi::gst_value_list_get_type()) }
     }
@@ -1014,8 +1275,8 @@ pub trait GstValueExt: Sized {
     fn serialize(&self) -> Result<glib::GString, glib::BoolError>;
     #[doc(alias = "gst_value_deserialize")]
     fn deserialize(s: &str, type_: glib::Type) -> Result<glib::Value, glib::BoolError>;
-    #[cfg(any(feature = "v1_20", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
+    #[cfg(feature = "v1_20")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_20")))]
     #[doc(alias = "gst_value_deserialize_with_pspec")]
     fn deserialize_with_pspec(
         s: &str,
@@ -1161,7 +1422,7 @@ impl GstValueExt for glib::Value {
     }
 
     fn deserialize(s: &str, type_: glib::Type) -> Result<glib::Value, glib::BoolError> {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
 
         unsafe {
             let mut value = glib::Value::from_type(type_);
@@ -1177,16 +1438,16 @@ impl GstValueExt for glib::Value {
         }
     }
 
-    #[cfg(any(feature = "v1_20", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
+    #[cfg(feature = "v1_20")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_20")))]
     fn deserialize_with_pspec(
         s: &str,
         pspec: &glib::ParamSpec,
     ) -> Result<glib::Value, glib::BoolError> {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
 
         unsafe {
-            let mut value = glib::Value::from_type(pspec.value_type());
+            let mut value = glib::Value::from_type_unchecked(pspec.value_type());
             let ret: bool = from_glib(ffi::gst_value_deserialize_with_pspec(
                 value.to_glib_none_mut().0,
                 s.to_glib_none().0,
